@@ -1,10 +1,9 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 1996-2006
- *	Oracle Corporation.  All rights reserved.
+ * Copyright (c) 1996,2007 Oracle.  All rights reserved.
  *
- * $Id: db_load.c,v 12.18 2006/08/26 09:23:08 bostic Exp $
+ * $Id: db_load.c,v 12.23 2007/05/17 17:17:42 bostic Exp $
  */
 
 #include "db_config.h"
@@ -15,7 +14,7 @@
 
 #ifndef lint
 static const char copyright[] =
-    "Copyright (c) 1996-2006\nOracle Corporation.  All rights reserved.\n";
+    "Copyright (c) 1996,2007 Oracle.  All rights reserved.\n";
 #endif
 
 typedef struct {			/* XXX: Globals. */
@@ -72,7 +71,7 @@ main(argc, argv)
 	int ch, existed, exitval, ret;
 	char **clist, **clp;
 
-	if ((progname = strrchr(argv[0], '/')) == NULL)
+	if ((progname = __db_rpath(argv[0])) == NULL)
 		progname = argv[0];
 	else
 		++progname;
@@ -395,7 +394,7 @@ retry_db:
 #endif
 
 	/* Open the DB file. */
-	if ((ret = dbp->open(dbp, NULL, name, subdb, dbtype,
+	if ((ret = (dbp->open)(dbp, NULL, name, subdb, dbtype,
 	    DB_CREATE | (TXN_ON(dbenv) ? DB_AUTO_COMMIT : 0),
 	    __db_omode("rw-rw-rw-"))) != 0) {
 		dbp->err(dbp, ret, "DB->open: %s", name);
@@ -609,7 +608,7 @@ db_init(dbenv, home, cache, is_private)
 	/* We may be loading into a live environment.  Try and join. */
 	flags = DB_USE_ENVIRON |
 	    DB_INIT_LOCK | DB_INIT_LOG | DB_INIT_MPOOL | DB_INIT_TXN;
-	if ((ret = dbenv->open(dbenv, home, flags, 0)) == 0)
+	if ((ret = (dbenv->open)(dbenv, home, flags, 0)) == 0)
 		return (0);
 	if (ret == DB_VERSION_MISMATCH)
 		goto err;
@@ -633,7 +632,7 @@ db_init(dbenv, home, cache, is_private)
 		dbenv->err(dbenv, ret, "set_cachesize");
 		return (1);
 	}
-	if ((ret = dbenv->open(dbenv, home, flags, 0)) == 0)
+	if ((ret = (dbenv->open)(dbenv, home, flags, 0)) == 0)
 		return (0);
 
 	/* An environment is required. */
@@ -843,7 +842,27 @@ rheader(dbenv, dbp, dbtypep, subdbp, checkprintp, keysp)
 
 		value = p--;
 
-		if (name[0] == '\0' || value[0] == '\0')
+		if (name[0] == '\0')
+			goto badfmt;
+
+		/*
+		 * The only values that may be zero-length are database names.
+		 * In the original Berkeley DB code it was possible to create
+		 * zero-length database names, and the db_load code was then
+		 * changed to allow such databases to be be dumped and loaded.
+		 * [#8204]
+		 */
+		if (strcmp(name, "database") == 0 ||
+		    strcmp(name, "subdatabase") == 0) {
+			if ((ret = convprintable(dbenv, value, subdbp)) != 0) {
+				dbp->err(dbp, ret, "error reading db name");
+				goto err;
+			}
+			continue;
+		}
+
+		/* No other values may be zero-length. */
+		if (value[0] == '\0')
 			goto badfmt;
 
 		if (strcmp(name, "HEADER") == 0)
@@ -894,14 +913,6 @@ rheader(dbenv, dbp, dbtypep, subdbp, checkprintp, keysp)
 			dbp->errx(dbp, "line %lu: unknown type", G(lineno));
 			goto err;
 		}
-		if (strcmp(name, "database") == 0 ||
-		    strcmp(name, "subdatabase") == 0) {
-			if ((ret = convprintable(dbenv, value, subdbp)) != 0) {
-				dbp->err(dbp, ret, "error reading db name");
-				goto err;
-			}
-			continue;
-		}
 		if (strcmp(name, "keys") == 0) {
 			if (strcmp(value, "1") == 0)
 				*keysp = 1;
@@ -1009,6 +1020,9 @@ convprintable(dbenv, instr, outstrp)
 	/*
 	 * Just malloc a string big enough for the whole input string;
 	 * the output string will be smaller (or of equal length).
+	 *
+	 * Note that we may be passed a zero-length string and need to
+	 * be able to duplicate it.
 	 */
 	if ((outstr = malloc(strlen(instr) + 1)) == NULL)
 		return (ENOMEM);
