@@ -1,18 +1,13 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 1998-2004
- *	Sleepycat Software.  All rights reserved.
+ * Copyright (c) 1998,2007 Oracle.  All rights reserved.
  *
- * $Id: xa_db.c,v 11.26 2004/01/28 03:36:40 bostic Exp $
+ * $Id: xa_db.c,v 12.10 2007/05/17 15:16:00 bostic Exp $
  */
 
 #include "db_config.h"
 
-#ifndef NO_SYSTEM_INCLUDES
-#include <sys/types.h>
-#endif
-
 #include "db_int.h"
 #include "dbinc/txn.h"
 
@@ -23,6 +18,8 @@ static int __xa_get __P((DB *, DB_TXN *, DBT *, DBT *, u_int32_t));
 static int __xa_open __P((DB *, DB_TXN *,
 	    const char *, const char *, DBTYPE, u_int32_t, int));
 static int __xa_put __P((DB *, DB_TXN *, DBT *, DBT *, u_int32_t));
+static int __xa_set_txn __P((DB *, DB_TXN **, int));
+static int __xa_truncate __P((DB *, DB_TXN *, u_int32_t *, u_int32_t));
 
 typedef struct __xa_methods {
 	int (*close) __P((DB *, u_int32_t));
@@ -32,18 +29,56 @@ typedef struct __xa_methods {
 	int (*open) __P((DB *, DB_TXN *,
 	    const char *, const char *, DBTYPE, u_int32_t, int));
 	int (*put) __P((DB *, DB_TXN *, DBT *, DBT *, u_int32_t));
+	int (*truncate) __P((DB *, DB_TXN *, u_int32_t *, u_int32_t));
 } XA_METHODS;
 
-#define	SET_TXN(PARAM, LOCAL) {						\
-	(LOCAL) = NULL;							\
-	if (!LF_ISSET(DB_AUTO_COMMIT)) {				\
-		if ((PARAM) != NULL)					\
-			(LOCAL) = (PARAM);				\
-		else if (__xa_get_txn(dbp->dbenv, &(LOCAL), 0) != 0)	\
-			(LOCAL) = NULL;					\
-		else if ((LOCAL) != NULL && (LOCAL)->txnid == TXN_INVALID) \
-			(LOCAL) = NULL;					\
-	}								\
+/*
+ * __xa_set_txn --
+ *	Find a transaction handle.
+ */
+static int
+__xa_set_txn(dbp, txnpp, no_xa_txn)
+	DB *dbp;
+	DB_TXN **txnpp;
+	int no_xa_txn;
+{
+	DB_ENV *dbenv;
+	int ret;
+
+	dbenv = dbp->dbenv;
+
+	/*
+	 * It doesn't make sense for a server to specify a DB_TXN handle.
+	 * As the server can't know if other operations it has done have
+	 * committed/aborted, it can self-deadlock.  If the server wants
+	 * other transactions, it can open other DB handles and use them.
+	 * Disallow specified DB_TXN handles.
+	 */
+	if (*txnpp != NULL) {
+		__db_errx(dbenv,
+    "transaction handles should not be directly specified to XA interfaces");
+		return (EINVAL);
+	}
+
+	/* See if the TM has declared a transaction. */
+	if ((ret = __xa_get_txn(dbenv, txnpp, 0)) != 0)
+		return (ret);
+	if ((*txnpp)->txnid != TXN_INVALID)
+		return (0);
+
+	/*
+	 * We may be opening databases in the server initialization routine.
+	 * In that case, it's reasonable not to have an XA transaction.  It's
+	 * also reasonable to open a database as part of an XA transaction,
+	 * allow both.
+	 */
+	if (no_xa_txn) {
+		*txnpp = NULL;
+		return (0);
+	}
+
+	__db_errx(dbenv, "no XA transaction declared");
+	return (EINVAL);
 }
 
 /*
@@ -60,8 +95,8 @@ __db_xa_create(dbp)
 	int ret;
 
 	/*
-	 * Interpose XA routines in front of any method that takes a TXN
-	 * ID as an argument.
+	 * Allocate the XA internal structure, and wrap the open and close
+	 * calls.
 	 */
 	if ((ret = __os_calloc(dbp->dbenv, 1, sizeof(XA_METHODS), &xam)) != 0)
 		return (ret);
@@ -79,7 +114,6 @@ __db_xa_create(dbp)
  * __xa_open --
  *	XA open wrapper.
  */
-
 static int
 __xa_open(dbp, txn, name, subdb, type, flags, mode)
 	DB *dbp;
@@ -89,24 +123,28 @@ __xa_open(dbp, txn, name, subdb, type, flags, mode)
 	u_int32_t flags;
 	int mode;
 {
-	DB_TXN *t;
 	XA_METHODS *xam;
 	int ret;
 
 	xam = (XA_METHODS *)dbp->xa_internal;
 
-	SET_TXN(txn, t);
-	if ((ret = (xam->open)(dbp, t, name, subdb, type, flags, mode)) != 0)
+	if ((ret =
+	    __xa_set_txn(dbp, &txn, LF_ISSET(DB_AUTO_COMMIT) ? 1 : 0)) != 0)
+		return (ret);
+	if ((ret = (xam->open)(dbp, txn, name, subdb, type, flags, mode)) != 0)
 		return (ret);
 
+	/* Wrap any DB handle method that takes a TXN ID as an argument. */
 	xam->cursor = dbp->cursor;
 	xam->del = dbp->del;
 	xam->get = dbp->get;
 	xam->put = dbp->put;
+	xam->truncate = dbp->truncate;
 	dbp->cursor = __xa_cursor;
 	dbp->del = __xa_del;
 	dbp->get = __xa_get;
 	dbp->put = __xa_put;
+	dbp->truncate = __xa_truncate;
 
 	return (0);
 }
@@ -118,15 +156,12 @@ __xa_cursor(dbp, txn, dbcp, flags)
 	DBC **dbcp;
 	u_int32_t flags;
 {
-	DB_TXN *t;
+	int ret;
 
-	if (txn != NULL)
-		t = txn;
-	else if (__xa_get_txn(dbp->dbenv, &t, 0) != 0 ||
-	    t->txnid== TXN_INVALID)
-		t = NULL;
-
-	return (((XA_METHODS *)dbp->xa_internal)->cursor (dbp, t, dbcp, flags));
+	if ((ret = __xa_set_txn(dbp, &txn, 0)) != 0)
+		return (ret);
+	return (((XA_METHODS *)
+	    dbp->xa_internal)->cursor(dbp, txn, dbcp, flags));
 }
 
 static int
@@ -136,10 +171,11 @@ __xa_del(dbp, txn, key, flags)
 	DBT *key;
 	u_int32_t flags;
 {
-	DB_TXN *t;
+	int ret;
 
-	SET_TXN(txn, t);
-	return (((XA_METHODS *)dbp->xa_internal)->del(dbp, t, key, flags));
+	if ((ret = __xa_set_txn(dbp, &txn, 0)) != 0)
+		return (ret);
+	return (((XA_METHODS *)dbp->xa_internal)->del(dbp, txn, key, flags));
 }
 
 static int
@@ -164,11 +200,12 @@ __xa_get(dbp, txn, key, data, flags)
 	DBT *key, *data;
 	u_int32_t flags;
 {
-	DB_TXN *t;
+	int ret;
 
-	SET_TXN(txn, t);
-	return (((XA_METHODS *)dbp->xa_internal)->get
-	    (dbp, t, key, data, flags));
+	if ((ret = __xa_set_txn(dbp, &txn, 0)) != 0)
+		return (ret);
+	return (((XA_METHODS *)
+	    dbp->xa_internal)->get(dbp, txn, key, data, flags));
 }
 
 static int
@@ -178,10 +215,24 @@ __xa_put(dbp, txn, key, data, flags)
 	DBT *key, *data;
 	u_int32_t flags;
 {
-	DB_TXN *t;
+	int ret;
 
-	SET_TXN(txn, t);
+	if ((ret = __xa_set_txn(dbp, &txn, 0)) != 0)
+		return (ret);
+	return (((XA_METHODS *)
+	    dbp->xa_internal)->put(dbp, txn, key, data, flags));
+}
 
-	return (((XA_METHODS *)dbp->xa_internal)->put
-	    (dbp, t, key, data, flags));
+static int
+__xa_truncate(dbp, txn, countp, flags)
+	DB *dbp;
+	DB_TXN *txn;
+	u_int32_t *countp, flags;
+{
+	int ret;
+
+	if ((ret = __xa_set_txn(dbp, &txn, 0)) != 0)
+		return (ret);
+	return (((XA_METHODS *)
+	    dbp->xa_internal)->truncate(dbp, txn, countp, flags));
 }
