@@ -55,7 +55,7 @@
 
 
 #ifndef lint
-FILE_RCSID("@(#)$File: compress.c,v 1.50 2007/03/01 22:14:54 christos Exp $")
+FILE_RCSID("@(#)$File: compress.c,v 1.51 2007/03/05 02:41:29 christos Exp $")
 #endif
 
 /*@-nullassign@*/
@@ -181,12 +181,12 @@ sread(int fd, void *buf, size_t n, int canbepipe)
 		goto nocheck;
 
 #ifdef FIONREAD
-	if (canbepipe && (ioctl(fd, FIONREAD, &t) == -1) || (t == 0)) {
+	if ((canbepipe && (ioctl(fd, FIONREAD, &t) == -1)) || (t == 0)) {
 #ifdef FD_ZERO
 		for (cnt = 0;; cnt++) {
 			fd_set check;
 			struct timeval tout = {0, 100 * 1000};
-			int rv;
+			int selrv;
 
 			FD_ZERO(&check);
 			FD_SET(fd, &check);
@@ -195,11 +195,11 @@ sread(int fd, void *buf, size_t n, int canbepipe)
 			 * Avoid soft deadlock: do not read if there
 			 * is nothing to read from sockets and pipes.
 			 */
-			rv = select(fd + 1, &check, NULL, NULL, &tout);
-			if (rv == -1) {
+			selrv = select(fd + 1, &check, NULL, NULL, &tout);
+			if (selrv == -1) {
 				if (errno == EINTR || errno == EAGAIN)
 					continue;
-			} else if (rv == 0 && cnt >= 5) {
+			} else if (selrv == 0 && cnt >= 5) {
 				return 0;
 			} else
 				break;
