@@ -32,13 +32,17 @@
 #include "file.h"
 #include "magic.h"
 #include <string.h>
+#if !(defined(HAVE_STRNDUP) && defined(__USE_GNU))
+char * strndup(const char *, size_t);
+#endif
+
 #include <ctype.h>
 #include <stdlib.h>
 #include <time.h>
 
 
 #ifndef	lint
-FILE_RCSID("@(#)$File: softmagic.c,v 1.95 2007/03/03 19:09:25 christos Exp $")
+FILE_RCSID("@(#)$File: softmagic.c,v 1.99 2007/05/08 14:44:18 christos Exp $")
 #endif	/* lint */
 
 private int match(struct magic_set *ms, struct magic *magic, uint32_t nmagic,
@@ -91,32 +95,6 @@ file_softmagic(struct magic_set *ms, const unsigned char *buf, size_t nbytes)
 	return 0;
 }
 
-#ifdef ENABLE_CONDITIONALS
-protected int
-#else
-private int
-#endif
-file_check_mem(struct magic_set *ms, unsigned int level)
-{
-	size_t len;
-
-	if (level >= ms->c.len) {
-		len = (ms->c.len += 20) * sizeof(*ms->c.li);
-		ms->c.li = (ms->c.li == NULL) ? malloc(len) :
-		    realloc(ms->c.li, len);
-		if (ms->c.li == NULL) {
-			file_oomem(ms, len);
-			return -1;
-		}
-	}
-	ms->c.li[level].got_match = 0;
-#ifdef ENABLE_CONDITIONALS
-	ms->c.li[level].last_match = 0;
-	ms->c.li[level].last_cond = COND_NONE;
-#endif /* ENABLE_CONDITIONALS */
-	return 0;
-}
-
 /*
  * Go through the whole list, stopping if you find a match.  Process all
  * the continuations of that match before returning.
@@ -331,8 +309,6 @@ check_fmt(struct magic_set *ms, struct magic *m)
 }
 
 #ifndef HAVE_STRNDUP
-char * strndup(const char *, size_t);
-
 char *
 strndup(const char *str, size_t n)
 {
@@ -354,7 +330,7 @@ private int32_t
 mprint(struct magic_set *ms, struct magic *m)
 {
 	uint64_t v;
-	int32_t t=0 ;
+	int64_t t = 0;
  	char buf[512];
 	union VALUETYPE *p = &ms->ms_value;
 
