@@ -3,7 +3,8 @@
  */
 
 #include "system.h"
-#include <rpmlib.h>
+#include <rpmio.h>
+#include <rpmlog.h>
 #include <rpmmacro.h>	/* XXX for rpmGetPath */
 
 #include "fs.h"
@@ -34,18 +35,16 @@ void rpmFreeFilesystems(void)
 {
     int i;
 
-/*@-boundswrite@*/
     if (filesystems)
     for (i = 0; i < numFilesystems; i++)
 	filesystems[i].mntPoint = _free(filesystems[i].mntPoint);
-/*@=boundswrite@*/
 
     filesystems = _free(filesystems);
     fsnames = _free(fsnames);
     numFilesystems = 0;
 }
 
-#if HAVE_MNTCTL
+#if defined(HAVE_MNTCTL)
 
 /* modeled after sample code from Till Bubeck */
 
@@ -77,7 +76,7 @@ static int getFilesystemList(void)
 
     num = mntctl(MCTL_QUERY, sizeof(size), (char *) &size);
     if (num < 0) {
-	rpmError(RPMERR_MTAB, _("mntctl() failed to return size: %s\n"), 
+	rpmlog(RPMLOG_ERR, _("mntctl() failed to return size: %s\n"), 
 		 strerror(errno));
 	return 1;
     }
@@ -92,7 +91,7 @@ static int getFilesystemList(void)
     buf = alloca(size);
     num = mntctl(MCTL_QUERY, size, buf);
     if ( num <= 0 ) {
-        rpmError(RPMERR_MTAB, _("mntctl() failed to return mount points: %s\n"), 
+        rpmlog(RPMLOG_ERR, _("mntctl() failed to return mount points: %s\n"), 
 		 strerror(errno));
 	return 1;
     }
@@ -112,7 +111,7 @@ static int getFilesystemList(void)
 	filesystems[i].mntPoint = fsnames[i] = fsn;
 	
 	if (stat(filesystems[i].mntPoint, &sb)) {
-	    rpmError(RPMERR_STAT, _("failed to stat %s: %s\n"), fsnames[i],
+	    rpmlog(RPMLOG_ERR, _("failed to stat %s: %s\n"), fsnames[i],
 			strerror(errno));
 
 	    rpmFreeFilesystems();
@@ -157,20 +156,24 @@ static int getFilesystemList(void)
 
 	mtab = fopen(MOUNTED, "r");
 	if (!mtab) {
-	    rpmError(RPMERR_MTAB, _("failed to open %s: %s\n"), MOUNTED, 
+	    rpmlog(RPMLOG_ERR, _("failed to open %s: %s\n"), MOUNTED, 
 		     strerror(errno));
 	    return 1;
 	}
-#   elif HAVE_GETMNTINFO_R
+#   elif defined(HAVE_GETMNTINFO_R)
     /* This is OSF */
     struct statfs * mounts = NULL;
     int mntCount = 0, bufSize = 0, flags = MNT_NOWAIT;
     int nextMount = 0;
 
 	getmntinfo_r(&mounts, flags, &mntCount, &bufSize);
-#   elif HAVE_GETMNTINFO
+#   elif defined(HAVE_GETMNTINFO)
     /* This is Mac OS X */
+#if defined(__NetBSD__)
+    struct statvfs * mounts = NULL;
+#else
     struct statfs * mounts = NULL;
+#endif
     int mntCount = 0, flags = MNT_NOWAIT;
     int nextMount = 0;
 
@@ -187,9 +190,7 @@ static int getFilesystemList(void)
 	    /*@-modunconnomods -moduncon @*/
 	    our_mntent * itemptr = getmntent(mtab);
 	    if (!itemptr) break;
-/*@-boundsread@*/
 	    item = *itemptr;	/* structure assignment */
-/*@=boundsread@*/
 	    mntdir = item.our_mntdir;
 #if defined(MNTOPT_RO)
 	    /*@-compdef@*/
@@ -202,11 +203,11 @@ static int getFilesystemList(void)
 	    /* Solaris, maybe others */
 	    if (getmntent(mtab, &item)) break;
 	    mntdir = item.our_mntdir;
-#	elif HAVE_GETMNTINFO_R
+#	elif defined(HAVE_GETMNTINFO_R)
 	    /* This is OSF */
 	    if (nextMount == mntCount) break;
 	    mntdir = mounts[nextMount++].f_mntonname;
-#	elif HAVE_GETMNTINFO
+#	elif defined(HAVE_GETMNTINFO)
 	    /* This is Mac OS X */
 	    if (nextMount == mntCount) break;
 	    mntdir = mounts[nextMount++].f_mntonname;
@@ -215,7 +216,7 @@ static int getFilesystemList(void)
 	if (stat(mntdir, &sb)) {
 	    switch(errno) {
 	    default:
-		rpmError(RPMERR_STAT, _("failed to stat %s: %s\n"), mntdir,
+		rpmlog(RPMLOG_ERR, _("failed to stat %s: %s\n"), mntdir,
 			strerror(errno));
 		rpmFreeFilesystems();
 		return 1;
@@ -237,7 +238,7 @@ static int getFilesystemList(void)
 	filesystems[numFilesystems].mntPoint = xstrdup(mntdir);
 	filesystems[numFilesystems].rdonly = rdonly;
 #if 0
-	rpmMessage(RPMMESS_DEBUG, _("%5d 0x%04x %s %s\n"),
+	rpmlog(RPMLOG_DEBUG, _("%5d 0x%04x %s %s\n"),
 		numFilesystems,
 		(unsigned) filesystems[numFilesystems].dev,
 		(filesystems[numFilesystems].rdonly ? "ro" : "rw"),
@@ -248,7 +249,7 @@ static int getFilesystemList(void)
 
 #   if GETMNTENT_ONE || GETMNTENT_TWO
 	(void) fclose(mtab);
-#   elif HAVE_GETMNTINFO_R
+#   elif defined(HAVE_GETMNTINFO_R)
 	mounts = _free(mounts);
 #   endif
 
@@ -256,12 +257,10 @@ static int getFilesystemList(void)
     filesystems[numFilesystems].mntPoint = NULL;
     filesystems[numFilesystems].rdonly = 0;
 
-/*@-boundswrite@*/
     fsnames = xcalloc((numFilesystems + 1), sizeof(*fsnames));
     for (i = 0; i < numFilesystems; i++)
 	fsnames[i] = filesystems[i].mntPoint;
     fsnames[numFilesystems] = NULL;
-/*@=boundswrite@*/
 
 /*@-nullstate@*/ /* FIX: fsnames[] may be NULL */
     return 0; 
@@ -269,24 +268,22 @@ static int getFilesystemList(void)
 }
 #endif	/* HAVE_MNTCTL */
 
-int rpmGetFilesystemList(const char *** listptr, int * num)
+int rpmGetFilesystemList(const char *** listptr, uint32_t * num)
 {
     if (!fsnames) 
 	if (getFilesystemList())
 	    return 1;
 
-/*@-boundswrite@*/
     if (listptr) *listptr = fsnames;
     if (num) *num = numFilesystems;
-/*@=boundswrite@*/
 
     return 0;
 }
 
-int rpmGetFilesystemUsage(const char ** fileList, int_32 * fssizes, int numFiles,
-			  uint_64 ** usagesPtr, /*@unused@*/ int flags)
+int rpmGetFilesystemUsage(const char ** fileList, uint32_t * fssizes, int numFiles,
+			  uint64_t ** usagesPtr, /*@unused@*/ int flags)
 {
-    int_64 * usages;
+    uint64_t * usages;
     int i, len, j;
     char * buf, * dirName;
     char * chptr;
@@ -306,14 +303,11 @@ int rpmGetFilesystemUsage(const char ** fileList, int_32 * fssizes, int numFiles
     sourceDir = rpmGetPath("%{_sourcedir}", NULL);
 
     maxLen = strlen(sourceDir);
-/*@-boundsread@*/
     for (i = 0; i < numFiles; i++) {
 	len = strlen(fileList[i]);
 	if (maxLen < len) maxLen = len;
     }
-/*@=boundsread@*/
     
-/*@-boundswrite@*/
     buf = alloca(maxLen + 1);
     lastDir = alloca(maxLen + 1);
     dirName = alloca(maxLen + 1);
@@ -339,7 +333,7 @@ int rpmGetFilesystemUsage(const char ** fileList, int_32 * fssizes, int numFiles
 	    chptr = dirName + strlen(dirName) - 1;
 	    while (stat(dirName, &sb)) {
 		if (errno != ENOENT) {
-		    rpmError(RPMERR_STAT, _("failed to stat %s: %s\n"), buf,
+		    rpmlog(RPMLOG_ERR, _("failed to stat %s: %s\n"), buf,
 				strerror(errno));
 		    sourceDir = _free(sourceDir);
 		    usages = _free(usages);
@@ -361,7 +355,7 @@ int rpmGetFilesystemUsage(const char ** fileList, int_32 * fssizes, int numFiles
 			/*@innerbreak@*/ break;
 
 		if (j == numFilesystems) {
-		    rpmError(RPMERR_BADDEV, 
+		    rpmlog(RPMLOG_ERR, 
 				_("file %s is on an unknown device\n"), buf);
 		    sourceDir = _free(sourceDir);
 		    usages = _free(usages);
@@ -376,18 +370,13 @@ int rpmGetFilesystemUsage(const char ** fileList, int_32 * fssizes, int numFiles
 	strcpy(lastDir, buf);
 	usages[lastfs] += fssizes[i];
     }
-/*@=boundswrite@*/
 
     sourceDir = _free(sourceDir);
 
-/*@-boundswrite@*/
-    /*@-branchstate@*/
     if (usagesPtr)
 	*usagesPtr = usages;
     else
 	usages = _free(usages);
-    /*@=branchstate@*/
-/*@=boundswrite@*/
 
     return 0;
 }
