@@ -4,14 +4,16 @@
 
 #include "system.h"
 
-#include <rpm/rpmlog.h>
-#include <rpm/rpmfileutil.h>
-#include <rpm/argv.h>
-
-#include "lib/manifest.h"
+#include <rpmio_internal.h>	/* XXX fdGetFp */
+#include <rpmlog.h>
+#include <rpmlib.h>
+#include <rpmmacro.h>
 
+#include "stringbuf.h"
+#include "manifest.h"
 #include "debug.h"
 
+/*@access StringBuf @*/
 
 char * rpmPermsString(int mode)
 {
@@ -25,8 +27,10 @@ char * rpmPermsString(int mode)
 	perms[0] = 'l';
     else if (S_ISFIFO(mode)) 
 	perms[0] = 'p';
+    /*@-unrecog@*/
     else if (S_ISSOCK(mode)) 
 	perms[0] = 's';
+    /*@=unrecog@*/
     else if (S_ISCHR(mode))
 	perms[0] = 'c';
     else if (S_ISBLK(mode))
@@ -59,20 +63,32 @@ char * rpmPermsString(int mode)
 }
 
 /**@todo Infinite loops through manifest files exist, operator error for now. */
-rpmRC rpmReadPackageManifest(FD_t fd, int * argcPtr, char *** argvPtr)
+rpmRC rpmReadPackageManifest(FD_t fd, int * argcPtr, const char *** argvPtr)
 {
-    ARGV_t sb = NULL;
+    StringBuf sb = newStringBuf();
     char * s = NULL;
     char * se;
     int ac = 0;
-    char ** av = NULL;
+    const char ** av = NULL;
     int argc = (argcPtr ? *argcPtr : 0);
-    char ** argv = (argvPtr ? *argvPtr : NULL);
-    FILE * f = fdopen(Fileno(fd), "r");
+    const char ** argv = (argvPtr ? *argvPtr : NULL);
+    FD_t xfd;
+    FILE * f;
     rpmRC rpmrc = RPMRC_OK;
     int i, j, next, npre;
 
-    if (f != NULL)
+    if (fdGetFp(fd) == NULL)
+	xfd = Fdopen(fd, "r.fpio");
+    else
+	xfd = fd;
+
+/*@+voidabstract@*/
+    if ((f = (FILE *) fdGetFp(xfd)) == NULL) {
+/*@=voidabstract@*/
+	rpmrc = RPMRC_NOTFOUND;
+	goto exit;
+    }
+
     while (1) {
 	char line[BUFSIZ];
 
@@ -83,6 +99,13 @@ rpmRC rpmReadPackageManifest(FD_t fd, int * argcPtr, char *** argvPtr)
 	    break;
 	}
 
+	/* XXX stop processing manifest if HTML is found. */
+#define	DOCTYPE_HTML_PUBLIC	"<!DOCTYPE HTML PUBLIC"
+	if (!strncmp(line, DOCTYPE_HTML_PUBLIC, sizeof(DOCTYPE_HTML_PUBLIC)-1)) {
+	    rpmrc = RPMRC_NOTFOUND;
+	    goto exit;
+	}
+
 	/* Skip comments. */
 	if ((se = strchr(s, '#')) != NULL) *se = '\0';
 
@@ -94,30 +117,31 @@ rpmRC rpmReadPackageManifest(FD_t fd, int * argcPtr, char *** argvPtr)
 	    s++;
 	if (*s == '\0') continue;
 
-	/* Sanity checks: skip obviously binary lines and dash (for stdin) */
-	if (*s < 32 || rstreq(s, "-")) {
-	    s = NULL;
-	    rpmrc = RPMRC_NOTFOUND;
+	/* Insure that file contains only ASCII */
+	if (*s < 32) {
+	    rpmrc = RPMRC_FAIL;	/* XXX reject non-printable manifests. */
 	    goto exit;
 	}
 
 	/* Concatenate next line in buffer. */
+	*se++ = ' ';
 	*se = '\0';
-	argvAdd(&sb, s);
+	appendStringBuf(sb, s);
     }
 
-    s = argvJoin(sb, " ");
+    if (s == NULL)		/* XXX always true */
+	s = getStringBuf(sb);
 
     if (!(s && *s)) {
-	rpmrc = RPMRC_NOTFOUND;
+	rpmrc = RPMRC_FAIL;	/* XXX force manifests to have content. */
 	goto exit;
     }
 
     /* Glob manifest items. */
-    rpmrc = (rpmGlob(s, &ac, &av) == 0 ? RPMRC_OK : RPMRC_FAIL);
+    rpmrc = rpmGlob(s, &ac, &av);
     if (rpmrc != RPMRC_OK) goto exit;
 
-    rpmlog(RPMLOG_DEBUG, "adding %d args from manifest.\n", ac);
+    rpmlog(RPMLOG_DEBUG, D_("adding %d args from manifest.\n"), ac);
 
     /* Count non-NULL args, keeping track of 1st arg after last NULL. */
     npre = 0;
@@ -133,7 +157,7 @@ rpmRC rpmReadPackageManifest(FD_t fd, int * argcPtr, char *** argvPtr)
     /* Copy old arg list, inserting manifest before argv[next]. */
     if (argv != NULL) {
 	int nac = npre + ac;
-	char ** nav = xcalloc((nac + 1), sizeof(*nav));
+	const char ** nav = xcalloc((nac + 1), sizeof(*nav));
 
 	for (i = 0, j = 0; i < next; i++) {
 	    if (argv[i] != NULL)
@@ -165,11 +189,11 @@ exit:
     if (argvPtr == NULL || (rpmrc != RPMRC_OK && av)) {
 	if (av)
 	for (i = 0; i < ac; i++)
-	   av[i] = _free(av[i]);
-	av = _free(av);
+	    /*@-unqualifiedtrans@*/av[i] = _free(av[i]); /*@=unqualifiedtrans@*/
+	/*@-dependenttrans@*/ av = _free(av); /*@=dependenttrans@*/
     }
-    argvFree(sb);
-    free(s);
-    /* FIX: *argvPtr may be NULL. */
+    sb = freeStringBuf(sb);
+    /*@-nullstate@*/ /* FIX: *argvPtr may be NULL. */
     return rpmrc;
+    /*@=nullstate@*/
 }
