@@ -6,6 +6,8 @@
  *
  */
 
+#include <string.h>
+
 #ifdef __cplusplus
 extern "C" {
 #endif
@@ -58,6 +60,7 @@ void freeSplitString( /*@only@*/ char ** list)
 
 /**
  * Like the libc function, but malloc()'s the space needed.
+ * @deprecated Use setenv(3) instead.
  * @param name		variable name
  * @param value		variable value
  * @param overwrite	should an existing variable be changed?
@@ -69,6 +72,7 @@ int dosetenv(const char * name, const char * value, int overwrite)
 
 /**
  * Like the libc function, but malloc()'s the space needed.
+ * @deprecated Use setenv(3) instead.
  * @param str		"name=value" string
  * @return		0 on success
  */
@@ -76,27 +80,6 @@ int doputenv(const char * str)
 	/*@globals environ@*/
 	/*@modifies *environ @*/;
 
-/**
- * Return file handle for a temporaray file.
- * A unique temporaray file path will be generated using
- *	rpmGenPath(prefix, "%{_tmppath}/", "rpm-tmp.XXXXX")
- * where "XXXXXX" is filled in using rand(3). The file is opened, and
- * the link count and (dev,ino) location are verified after opening.
- * The file name and the open file handle are returned.
- *
- * @param prefix	leading part of temp file path
- * @retval fnptr	temp file name (or NULL)
- * @retval fdptr	temp file handle
- * @return		0 on success
- */
-int makeTempFile(/*@null@*/ const char * prefix,
-		/*@null@*/ /*@out@*/ const char ** fnptr,
-		/*@out@*/ FD_t * fdptr)
-	/*@globals rpmGlobalMacroContext, h_errno,
-		fileSystem, internalState @*/
-	/*@modifies *fnptr, *fdptr, rpmGlobalMacroContext,
-		fileSystem, internalState @*/;
-
 /**
  * Return (malloc'd) current working directory.
  * @return		current working directory (malloc'ed)
