@@ -5,13 +5,18 @@
 
 #include "system.h"
 
-#include <rpm/rpmcli.h>
-#include <rpm/rpmgi.h>	/* XXX for giFlags */
+#include <rpmio.h>
+#include <rpmcli.h>
+#include <rpmgi.h>		/* XXX for giFlags */
 
 #include "debug.h"
 
+/*@unchecked@*/
 struct rpmQVKArguments_s rpmQVKArgs;
 
+/*@unchecked@*/
+int specedit = 0;
+
 #define POPT_QUERYFORMAT	-1000
 #define POPT_WHATREQUIRES	-1001
 #define POPT_WHATPROVIDES	-1002
@@ -26,11 +31,19 @@ struct rpmQVKArguments_s rpmQVKArgs;
 #define POPT_HDLIST		-1011
 #define POPT_FTSWALK		-1012
 
+/* -1025 thrugh -1033 are common in rpmcli.h. */
+#define	POPT_TRUST		-1037
+#define	POPT_WHATNEEDS		-1038
+#define	POPT_SPECSRPM		-1039
+#define POPT_QUERYBYSOURCEPKGID	-1040
+
 /* ========== Query/Verify/Signature source args */
-static void rpmQVSourceArgCallback( poptContext con,
-		enum poptCallbackReason reason,
-		const struct poptOption * opt, const char * arg, 
-		const void * data)
+static void rpmQVSourceArgCallback( /*@unused@*/ poptContext con,
+		/*@unused@*/ enum poptCallbackReason reason,
+		const struct poptOption * opt, /*@unused@*/ const char * arg, 
+		/*@unused@*/ const void * data)
+	/*@globals rpmQVKArgs @*/
+	/*@modifies rpmQVKArgs @*/
 {
     QVA_t qva = &rpmQVKArgs;
 
@@ -52,12 +65,16 @@ static void rpmQVSourceArgCallback( poptContext con,
     case 'f': qva->qva_source |= RPMQV_PATH; qva->qva_sourceCount++; break;
     case 'g': qva->qva_source |= RPMQV_GROUP; qva->qva_sourceCount++; break;
     case 'p': qva->qva_source |= RPMQV_RPM; qva->qva_sourceCount++; break;
+    case POPT_WHATNEEDS: qva->qva_source |= RPMQV_WHATNEEDS; 
+				qva->qva_sourceCount++; break;
     case POPT_WHATPROVIDES: qva->qva_source |= RPMQV_WHATPROVIDES; 
 				qva->qva_sourceCount++; break;
     case POPT_WHATREQUIRES: qva->qva_source |= RPMQV_WHATREQUIRES; 
 				qva->qva_sourceCount++; break;
     case POPT_TRIGGEREDBY: qva->qva_source |= RPMQV_TRIGGEREDBY;
 				qva->qva_sourceCount++; break;
+    case POPT_QUERYBYSOURCEPKGID: qva->qva_source |= RPMQV_SOURCEPKGID;
+				qva->qva_sourceCount++; break;
     case POPT_QUERYBYPKGID: qva->qva_source |= RPMQV_PKGID;
 				qva->qva_sourceCount++; break;
     case POPT_QUERYBYHDRID: qva->qva_source |= RPMQV_HDRID;
@@ -76,6 +93,11 @@ static void rpmQVSourceArgCallback( poptContext con,
 	qva->qva_source |= RPMQV_SPECFILE;
 	qva->qva_sourceCount++;
 	break;
+/* XXX SPECSRPM is not verify sources */
+    case POPT_SPECSRPM:
+	qva->qva_source |= RPMQV_SPECSRPM;
+	qva->qva_sourceCount++;
+	break;
     case POPT_QUERYBYNUMBER:
 	qva->qva_source |= RPMQV_DBOFFSET; 
 	qva->qva_sourceCount++;
@@ -86,48 +108,58 @@ static void rpmQVSourceArgCallback( poptContext con,
 /**
  * Common query/verify mode options.
  */
+/*@unchecked@*/
 struct poptOption rpmQVSourcePoptTable[] = {
-/* FIX: cast? */
+/*@-type@*/ /* FIX: cast? */
  { NULL, '\0', POPT_ARG_CALLBACK | POPT_CBFLAG_INC_DATA, 
 	rpmQVSourceArgCallback, 0, NULL, NULL },
- { "all", 'a', 0, 0, 'a',
+/*@=type@*/
+ { "all", 'a', 0, NULL, 'a',
 	N_("query/verify all packages"), NULL },
  { "checksig", 'K', POPT_ARGFLAG_DOC_HIDDEN, NULL, 'K',
 	N_("rpm checksig mode"), NULL },
- { "file", 'f', 0, 0, 'f',
+ { "file", 'f', 0, NULL, 'f',
 	N_("query/verify package(s) owning file"), "FILE" },
- { "group", 'g', 0, 0, 'g',
+ { "group", 'g', 0, NULL, 'g',
 	N_("query/verify package(s) in group"), "GROUP" },
- { "package", 'p', 0, 0, 'p',
+ { "package", 'p', 0, NULL, 'p',
 	N_("query/verify a package file"), NULL },
 
- { "ftswalk", 'W', 0, 0, POPT_FTSWALK,
+ { "ftswalk", 'W', 0, NULL, POPT_FTSWALK,
 	N_("query/verify package(s) from TOP file tree walk"), "TOP" },
  { "hdlist", 'H', POPT_ARGFLAG_DOC_HIDDEN, 0, POPT_HDLIST,
 	N_("query/verify package(s) from system HDLIST"), "HDLIST" },
 
- { "pkgid", '\0', 0, 0, POPT_QUERYBYPKGID,
+ { "sourcepkgid", '\0', 0, NULL, POPT_QUERYBYSOURCEPKGID,
+	N_("query/verify package(s) with source package identifier"), "MD5" },
+ { "pkgid", '\0', 0, NULL, POPT_QUERYBYPKGID,
 	N_("query/verify package(s) with package identifier"), "MD5" },
- { "hdrid", '\0', 0, 0, POPT_QUERYBYHDRID,
+ { "hdrid", '\0', 0, NULL, POPT_QUERYBYHDRID,
 	N_("query/verify package(s) with header identifier"), "SHA1" },
- { "fileid", '\0', 0, 0, POPT_QUERYBYFILEID,
+ { "fileid", '\0', 0, NULL, POPT_QUERYBYFILEID,
 	N_("query/verify package(s) with file identifier"), "MD5" },
 
  { "query", 'q', POPT_ARGFLAG_DOC_HIDDEN, NULL, 'q',
 	N_("rpm query mode"), NULL },
- { "querybynumber", '\0', POPT_ARGFLAG_DOC_HIDDEN, 0, POPT_QUERYBYNUMBER,
+ { "querybynumber", '\0', POPT_ARGFLAG_DOC_HIDDEN, NULL, POPT_QUERYBYNUMBER,
 	N_("query/verify a header instance"), "HDRNUM" },
- { "specfile", '\0', 0, 0, POPT_SPECFILE,
+ { "specfile", '\0', 0, NULL, POPT_SPECFILE,
 	N_("query a spec file"), N_("<spec>") },
- { "tid", '\0', POPT_ARGFLAG_DOC_HIDDEN, 0, POPT_QUERYBYTID,
+ { "specsrpm", '\0', POPT_ARGFLAG_DOC_HIDDEN, NULL, POPT_SPECSRPM,
+	N_("query source metadata from spec file parse"), N_("<spec>") },
+ { "tid", '\0', POPT_ARGFLAG_DOC_HIDDEN, NULL, POPT_QUERYBYTID,
 	N_("query/verify package(s) from install transaction"), "TID" },
- { "triggeredby", '\0', 0, 0, POPT_TRIGGEREDBY, 
+ { "triggeredby", '\0', 0, NULL, POPT_TRIGGEREDBY, 
 	N_("query the package(s) triggered by the package"), "PACKAGE" },
  { "verify", 'V', POPT_ARGFLAG_DOC_HIDDEN, NULL, 'V',
 	N_("rpm verify mode"), NULL },
- { "whatrequires", '\0', 0, 0, POPT_WHATREQUIRES, 
+ { "whatrequires", '\0', 0, NULL, POPT_WHATREQUIRES, 
 	N_("query/verify the package(s) which require a dependency"), "CAPABILITY" },
- { "whatprovides", '\0', 0, 0, POPT_WHATPROVIDES, 
+ { "whatneeds", '\0', POPT_ARGFLAG_DOC_HIDDEN, NULL, POPT_WHATNEEDS, 
+	N_("query/verify the package(s) which require any contained provide"),
+	"CAPABILITY" },
+
+ { "whatprovides", '\0', 0, NULL, POPT_WHATPROVIDES, 
 	N_("query/verify the package(s) which provide a dependency"), "CAPABILITY" },
 
  { "transaction", 'T', POPT_BIT_SET|POPT_ARGFLAG_DOC_HIDDEN, &giFlags, (RPMGI_TSADD|RPMGI_TSORDER),
@@ -148,9 +180,11 @@ struct poptOption rpmQVSourcePoptTable[] = {
 /* ========== Query specific popt args */
 
 static void queryArgCallback(poptContext con,
-		enum poptCallbackReason reason,
+		/*@unused@*/enum poptCallbackReason reason,
 		const struct poptOption * opt, const char * arg, 
-		const void * data)
+		/*@unused@*/ const void * data)
+	/*@globals rpmQVKArgs @*/
+	/*@modifies con, rpmQVKArgs @*/
 {
     QVA_t qva = &rpmQVKArgs;
 
@@ -180,7 +214,9 @@ static void queryArgCallback(poptContext con,
 
     case 'i':
 	if (qva->qva_mode == 'q') {
+	    /*@-nullassign -readonlytrans@*/
 	    const char * infoCommand[] = { "--info", NULL };
+	    /*@=nullassign =readonlytrans@*/
 	    (void) poptStuffArgs(con, infoCommand);
 	}
 	break;
@@ -201,38 +237,40 @@ static void queryArgCallback(poptContext con,
 	qva->qva_flags |= VERIFY_DEPS;
 	break;
 
-    case RPMCLI_POPT_NOFILEDIGEST:
-	qva->qva_flags |= VERIFY_FILEDIGEST;
+    case RPMCLI_POPT_NOFDIGESTS:
+	qva->qva_flags |= VERIFY_FDIGEST;
 	break;
 
     case RPMCLI_POPT_NOCONTEXTS:
 	qva->qva_flags |= VERIFY_CONTEXTS;
 	break;
 
-#ifdef	NOTYET
-    case RPMCLI_POPT_FORCE:
-	ia->probFilter |=
-		( RPMPROB_FILTER_REPLACEPKG
-		| RPMPROB_FILTER_REPLACEOLDFILES
-		| RPMPROB_FILTER_REPLACENEWFILES
-		| RPMPROB_FILTER_OLDPACKAGE );
-	break;
-#endif
-
     case RPMCLI_POPT_NOSCRIPTS:
 	qva->qva_flags |= VERIFY_SCRIPT;
 	break;
 
+    /* XXX perhaps POPT_ARG_INT instead of callback. */
+    case POPT_TRUST:
+    {	char * end = NULL;
+	long trust = (int) strtol(arg, &end, 0);
+	/* XXX range checks on trust. */
+	/* XXX if (end && *end) argerror(_("non-numeric trust metric.")); */
+	qva->trust = trust;
+    }	break;
     }
 }
 
 /**
  * Query mode options.
  */
+/*@unchecked@*/
 struct poptOption rpmQueryPoptTable[] = {
-/* FIX: cast? */
+/*@-type@*/ /* FIX: cast? */
  { NULL, '\0', POPT_ARG_CALLBACK | POPT_CBFLAG_INC_DATA | POPT_CBFLAG_CONTINUE, 
 	queryArgCallback, 0, NULL, NULL },
+/*@=type@*/
+ { NULL, '\0', POPT_ARG_INCLUDE_TABLE, rpmQVSourcePoptTable, 0,
+	NULL, NULL },
  { "configfiles", 'c', 0, 0, 'c',
 	N_("list all configuration files"), NULL },
  { "docfiles", 'd', 0, 0, 'd',
@@ -244,17 +282,39 @@ struct poptOption rpmQueryPoptTable[] = {
  { "list", 'l', 0, 0, 'l',
 	N_("list files in package"), NULL },
 
+ { "aid", '\0', POPT_BIT_SET|POPT_ARGFLAG_DOC_HIDDEN,
+	&rpmQVKArgs.depFlags, RPMDEPS_FLAG_ADDINDEPS,
+	N_("add suggested packages to transaction"), NULL },
+
  /* Duplicate file attr flags from packages into command line options. */
- { "noghost", '\0', POPT_BIT_CLR|POPT_ARGFLAG_DOC_HIDDEN,
+ { "noconfig", '\0', POPT_BIT_SET|POPT_ARGFLAG_DOC_HIDDEN,
+	&rpmQVKArgs.qva_fflags, RPMFILE_CONFIG,
+        N_("skip %%config files"), NULL },
+ { "nodoc", '\0', POPT_BIT_SET|POPT_ARGFLAG_DOC_HIDDEN,
+	&rpmQVKArgs.qva_fflags, RPMFILE_DOC,
+        N_("skip %%doc files"), NULL },
+ { "noghost", '\0', POPT_BIT_SET|POPT_ARGFLAG_DOC_HIDDEN,
 	&rpmQVKArgs.qva_fflags, RPMFILE_GHOST,
         N_("skip %%ghost files"), NULL },
+#ifdef	NOTEVER		/* XXX there's hardly a need for these */
+ { "nolicense", '\0', POPT_BIT_SET|POPT_ARGFLAG_DOC_HIDDEN,
+	&rpmQVKArgs.qva_fflags, RPMFILE_LICENSE,
+        N_("skip %%license files"), NULL },
+ { "noreadme", '\0', POPT_BIT_SET|POPT_ARGFLAG_DOC_HIDDEN,
+	&rpmQVKArgs.qva_fflags, RPMFILE_README,
+        N_("skip %%readme files"), NULL },
+#endif
 
  { "qf", '\0', POPT_ARG_STRING | POPT_ARGFLAG_DOC_HIDDEN, 0, 
 	POPT_QUERYFORMAT, NULL, NULL },
  { "queryformat", '\0', POPT_ARG_STRING, 0, POPT_QUERYFORMAT,
-	N_("use the following query format"), "QUERYFORMAT" },
+	N_("use the following query format"), N_("QUERYFORMAT") },
+ { "specedit", '\0', POPT_ARG_VAL|POPT_ARGFLAG_DOC_HIDDEN, &specedit, -1,
+	N_("substitute i18n sections into spec file"), NULL },
  { "state", 's', 0, 0, 's',
 	N_("display the states of the listed files"), NULL },
+ { "target", '\0', POPT_ARG_STRING|POPT_ARGFLAG_DOC_HIDDEN, 0,  RPMCLI_POPT_TARGETPLATFORM,
+        N_("specify target platform"), N_("CPU-VENDOR-OS") },
    POPT_TABLEEND
 };
 
@@ -262,14 +322,36 @@ struct poptOption rpmQueryPoptTable[] = {
  * Verify mode options.
  */
 struct poptOption rpmVerifyPoptTable[] = {
-/* FIX: cast? */
+/*@-type@*/ /* FIX: cast? */
  { NULL, '\0', POPT_ARG_CALLBACK | POPT_CBFLAG_INC_DATA | POPT_CBFLAG_CONTINUE, 
 	queryArgCallback, 0, NULL, NULL },
+/*@=type@*/
+ { NULL, '\0', POPT_ARG_INCLUDE_TABLE, rpmQVSourcePoptTable, 0,
+	NULL, NULL },
 
- { "nofiledigest", '\0', 0, NULL, RPMCLI_POPT_NOFILEDIGEST,
-	N_("don't verify digest of files"), NULL },
- { "nomd5", '\0', 0, NULL, RPMCLI_POPT_NOFILEDIGEST,
-	N_("don't verify digest of files"), NULL },
+ { "aid", '\0', POPT_BIT_SET|POPT_ARGFLAG_DOC_HIDDEN,
+	&rpmQVKArgs.depFlags, RPMDEPS_FLAG_ADDINDEPS,
+	N_("add suggested packages to transaction"), NULL },
+
+ /* Duplicate file attr flags from packages into command line options. */
+ { "noconfig", '\0', POPT_BIT_SET|POPT_ARGFLAG_DOC_HIDDEN,
+	&rpmQVKArgs.qva_fflags, RPMFILE_CONFIG,
+        N_("skip %%config files"), NULL },
+ { "nodoc", '\0', POPT_BIT_SET|POPT_ARGFLAG_DOC_HIDDEN,
+	&rpmQVKArgs.qva_fflags, RPMFILE_DOC,
+        N_("skip %%doc files"), NULL },
+
+ /* Duplicate file verify flags from packages into command line options. */
+/** @todo Add --nomd5 alias to rpmpopt, eliminate. */
+#ifdef	DYING
+ { "nomd5", '\0', POPT_BIT_SET, &rpmQVKArgs.qva_flags, VERIFY_FDIGEST,
+	N_("don't verify file digests"), NULL },
+#else
+ { "nomd5", '\0', POPT_ARGFLAG_DOC_HIDDEN, NULL, RPMCLI_POPT_NOFDIGESTS,
+	N_("don't verify file digests"), NULL },
+ { "nofdigests", '\0', 0, NULL, RPMCLI_POPT_NOFDIGESTS,
+	N_("don't verify file digests"), NULL },
+#endif
  { "nosize", '\0', POPT_BIT_SET|POPT_ARGFLAG_DOC_HIDDEN,
 	&rpmQVKArgs.qva_flags, VERIFY_SIZE,
         N_("don't verify size of files"), NULL },
@@ -291,37 +373,51 @@ struct poptOption rpmVerifyPoptTable[] = {
  { "nordev", '\0', POPT_BIT_SET|POPT_ARGFLAG_DOC_HIDDEN,
 	&rpmQVKArgs.qva_flags, VERIFY_RDEV,
         N_("don't verify mode of files"), NULL },
- { "nocaps", '\0', POPT_BIT_SET|POPT_ARGFLAG_DOC_HIDDEN,
-	&rpmQVKArgs.qva_flags, VERIFY_CAPS,
-	N_("don't verify capabilities of files"), NULL },
 
  { "nocontexts", '\0', POPT_ARGFLAG_DOC_HIDDEN, NULL, RPMCLI_POPT_NOCONTEXTS,
 	N_("don't verify file security contexts"), NULL },
  { "nofiles", '\0', POPT_BIT_SET, &rpmQVKArgs.qva_flags, VERIFY_FILES,
 	N_("don't verify files in package"), NULL},
+#ifdef	DYING
+ { "nodeps", '\0', POPT_BIT_SET, &rpmQVKArgs.qva_flags, VERIFY_DEPS,
+	N_("don't verify package dependencies"), NULL },
+#else
  { "nodeps", '\0', 0, NULL, RPMCLI_POPT_NODEPS,
 	N_("don't verify package dependencies"), NULL },
+#endif
 
+#ifdef	DYING
+ { "noscript", '\0', POPT_BIT_SET,&rpmQVKArgs.qva_flags, VERIFY_SCRIPT,
+        N_("don't execute verify script(s)"), NULL },
+ /* XXX legacy had a trailing s on --noscript */
+ { "noscripts", '\0', POPT_BIT_SET|POPT_ARGFLAG_DOC_HIDDEN,
+	&rpmQVKArgs.qva_flags, VERIFY_SCRIPT,
+        N_("don't execute verify script(s)"), NULL },
+#else
  { "noscript", '\0', 0, NULL, RPMCLI_POPT_NOSCRIPTS,
         N_("don't execute verify script(s)"), NULL },
  /* XXX legacy had a trailing s on --noscript */
  { "noscripts", '\0', POPT_ARGFLAG_DOC_HIDDEN, NULL, RPMCLI_POPT_NOSCRIPTS,
         N_("don't execute verify script(s)"), NULL },
+#endif
 
+#ifdef	DYING
+ { "nodigest", '\0', POPT_BIT_SET, &rpmQVKArgs.qva_flags, VERIFY_DIGEST,
+        N_("don't verify package digest(s)"), NULL },
+ { "nohdrchk", '\0', POPT_BIT_SET|POPT_ARGFLAG_DOC_HIDDEN,
+	&rpmQVKArgs.qva_flags, VERIFY_HDRCHK,
+        N_("don't verify database header(s) when retrieved"), NULL },
+ { "nosignature", '\0', POPT_BIT_SET,
+	&rpmQVKArgs.qva_flags, VERIFY_SIGNATURE,
+        N_("don't verify package signature(s)"), NULL },
+#else
  { "nodigest", '\0', POPT_ARGFLAG_DOC_HIDDEN, 0, RPMCLI_POPT_NODIGEST,
         N_("don't verify package digest(s)"), NULL },
  { "nohdrchk", '\0', POPT_ARGFLAG_DOC_HIDDEN, 0, RPMCLI_POPT_NOHDRCHK,
         N_("don't verify database header(s) when retrieved"), NULL },
  { "nosignature", '\0', POPT_ARGFLAG_DOC_HIDDEN, 0, RPMCLI_POPT_NOSIGNATURE,
         N_("don't verify package signature(s)"), NULL },
-
-/** @todo Add --nogpg/--nopgp aliases to rpmpopt, eliminate. */
- { "nogpg", '\0', POPT_BIT_SET|POPT_ARGFLAG_DOC_HIDDEN,
-	&rpmQVKArgs.qva_flags, VERIFY_SIGNATURE,
-        N_("don't verify GPG V3 DSA signature(s)"), NULL },
- { "nopgp", '\0', POPT_BIT_SET|POPT_ARGFLAG_DOC_HIDDEN,
-	&rpmQVKArgs.qva_flags, VERIFY_SIGNATURE,
-        N_("don't verify PGP V3 RSA/MD5 signature(s)"), NULL },
+#endif
 
     POPT_TABLEEND
 };
@@ -329,10 +425,12 @@ struct poptOption rpmVerifyPoptTable[] = {
 /**
  * Signature mode options.
  */
+/*@unchecked@*/
 struct poptOption rpmSignPoptTable[] = {
-/* FIX: cast? */
+/*@-type@*/ /* FIX: cast? */
  { NULL, '\0', POPT_ARG_CALLBACK | POPT_CBFLAG_INC_DATA | POPT_CBFLAG_CONTINUE,
 	rpmQVSourceArgCallback, 0, NULL, NULL },
+/*@=type@*/
  { "addsign", '\0', 0, NULL, 'A',
 	N_("sign package(s) (identical to --resign)"), NULL },
  { "checksig", 'K', 0, NULL, 'K',
@@ -345,6 +443,15 @@ struct poptOption rpmSignPoptTable[] = {
 	N_("sign package(s) (identical to --addsign)"), NULL },
  { "sign", '\0', POPT_ARGFLAG_DOC_HIDDEN, &rpmQVKArgs.sign, 0,
 	N_("generate signature"), NULL },
+ /* XXX perhaps POPT_ARG_INT instead of callback. */
+ { "trust", '\0', POPT_ARG_STRING|POPT_ARGFLAG_DOC_HIDDEN, 0,  POPT_TRUST,
+        N_("specify trust metric"), "TRUST" },
+ { "trusted", '\0', POPT_ARG_VAL|POPT_ARGFLAG_DOC_HIDDEN,
+	&rpmQVKArgs.trust, 1,
+        N_("set ultimate trust when importing pubkey(s)"), NULL },
+ { "untrusted", '\0', POPT_ARG_VAL|POPT_ARGFLAG_DOC_HIDDEN,
+	&rpmQVKArgs.trust, -1,
+        N_("unset ultimate trust when importing pubkey(s)"), NULL },
 
  { "nodigest", '\0', POPT_BIT_SET, &rpmQVKArgs.qva_flags, VERIFY_DIGEST,
         N_("don't verify package digest(s)"), NULL },
