@@ -4,6 +4,8 @@
 
 #include "system.h"
 
+#include <rpmio.h>
+#include <rpmcb.h>		/* XXX fnpyKey */
 #include <rpmlib.h>
 
 #include "rpmal.h"
@@ -35,7 +37,7 @@ struct availablePackage_s {
 /*@refcounted@*/ /*@null@*/
     rpmfi fi;			/*!< File info set. */
 
-    uint_32 tscolor;		/*!< Transaction color bits. */
+    uint32_t tscolor;		/*!< Transaction color bits. */
 
 /*@exposed@*/ /*@dependent@*/ /*@null@*/
     fnpyKey key;		/*!< Associated file name/python object */
@@ -84,7 +86,7 @@ struct fileIndexEntry_s {
     const char * baseName;	/*!< File basename. */
     int baseNameLen;
     alNum pkgNum;		/*!< Containing package index. */
-    uint_32 ficolor;
+    uint32_t ficolor;
 };
 
 typedef /*@abstract@*/ struct dirInfo_s *		dirInfo;
@@ -112,7 +114,7 @@ struct rpmal_s {
     int delta;			/*!< Delta for pkg list reallocation. */
     int size;			/*!< No. of pkgs in list. */
     int alloced;		/*!< No. of pkgs allocated for list. */
-    uint_32 tscolor;		/*!< Transaction color. */
+    uint32_t tscolor;		/*!< Transaction color. */
     int numDirs;		/*!< No. of directories. */
 /*@owned@*/ /*@null@*/
     dirInfo dirs;		/*!< Set of directories. */
@@ -150,7 +152,9 @@ static inline alNum alKey2Num(/*@unused@*/ /*@null@*/ const rpmal al,
 	/*@*/
 {
     /*@-nullret -temptrans -retalias @*/
-    return ((alNum)pkgKey);
+    union { alKey key; alNum num; } u;
+    u.key = pkgKey;
+    return u.num;
     /*@=nullret =temptrans =retalias @*/
 }
 
@@ -159,7 +163,9 @@ static inline alKey alNum2Key(/*@unused@*/ /*@null@*/ const rpmal al,
 	/*@*/
 {
     /*@-nullret -temptrans -retalias @*/
-    return ((alKey)pkgNum);
+    union { alKey key; alNum num; } u;
+    u.num = pkgNum;
+    return u.key;
     /*@=nullret =temptrans =retalias @*/
 }
 
@@ -337,10 +343,8 @@ fprintf(stderr, "*** del %p[%d]\n", al->list, pkgNum);
 	    /*@=assignexpose =dependenttrans =observertrans@*/
 	    dieNeedle->dirNameLen = (dieNeedle->dirName != NULL
 			? strlen(dieNeedle->dirName) : 0);
-/*@-boundswrite@*/
 	    die = bsearch(dieNeedle, al->dirs, al->numDirs,
 			       sizeof(*dieNeedle), dieCompare);
-/*@=boundswrite@*/
 	    if (die == NULL)
 		continue;
 
@@ -362,9 +366,7 @@ if (_rpmal_debug)
 fprintf(stderr, "\t%p[%3d] memmove(%p:%p,%p:%p,0x%x) %s <- %s\n", die->files, die->numFiles, fie, fie->baseName, fie+1, (fie+1)->baseName, (unsigned) ((die->numFiles - i) * sizeof(*fie)), fie->baseName, (fie+1)->baseName);
 /*@=modfilesys@*/
 
-/*@-bounds@*/
 		    memmove(fie, fie+1, (die->numFiles - i) * sizeof(*fie));
-/*@=bounds@*/
 		}
 /*@-modfilesys@*/
 if (_rpmal_debug)
@@ -388,9 +390,7 @@ if (_rpmal_debug)
 fprintf(stderr, "    die[%5d] memmove(%p,%p,0x%x)\n", (int)(die - al->dirs), die, die+1, (unsigned)((al->numDirs - (die - al->dirs)) * sizeof(*die)));
 /*@=modfilesys@*/
 
-/*@-bounds@*/
 		memmove(die, die+1, (al->numDirs - (die - al->dirs)) * sizeof(*die));
-/*@=bounds@*/
 	    }
 
 /*@-modfilesys@*/
@@ -411,15 +411,12 @@ fprintf(stderr, "    die[%5d] memset(%p,0,0x%x)\n", al->numDirs, al->dirs + al->
     alp->provides = rpmdsFree(alp->provides);
     alp->fi = rpmfiFree(alp->fi);
 
-/*@-boundswrite@*/
     memset(alp, 0, sizeof(*alp));	/* XXX trash and burn */
-/*@=boundswrite@*/
     return;
 }
 
-/*@-bounds@*/
 alKey rpmalAdd(rpmal * alistp, alKey pkgKey, fnpyKey key,
-		rpmds provides, rpmfi fi, uint_32 tscolor)
+		rpmds provides, rpmfi fi, uint32_t tscolor)
 {
     alNum pkgNum;
     rpmal al;
@@ -508,16 +505,7 @@ fprintf(stderr, "*** add %p[%d] 0x%x\n", al->list, pkgNum, tscolor);
 	    (void) rpmfiSetDX(fi, dx);
 
 	    /*@-assignexpose -dependenttrans -observertrans@*/
-	    {   DN = rpmfiDN(fi);
-
-#if defined(__ia64__)
-/* XXX Make sure that autorelocated file dependencies are satisfied. */
-#define	DNPREFIX	"/emul/ia32-linux"
-		if (!strncmp(DN, DNPREFIX, sizeof(DNPREFIX)-1))
-		    DN += sizeof(DNPREFIX)-1;
-#endif
-		dieNeedle->dirName = DN;
-	    }
+	    dieNeedle->dirName = rpmfiDN(fi);
 	    /*@=assignexpose =dependenttrans =observertrans@*/
 
 	    dieNeedle->dirNameLen = (dieNeedle->dirName != NULL
@@ -600,7 +588,6 @@ fprintf(stderr, "\t%p[%3d] %p:%p[%2d] %s\n", die->files, die->numFiles, fie, fie
 assert(((alNum)(alp - al->list)) == pkgNum);
     return ((alKey)(alp - al->list));
 }
-/*@=bounds@*/
 
 /**
  * Compare two available index entries by name (qsort/bsearch).
@@ -624,9 +611,9 @@ static int indexcmp(const void * one, const void * two)
     return strcmp(a->entry, b->entry);
 }
 
-void rpmalAddProvides(rpmal al, alKey pkgKey, rpmds provides, uint_32 tscolor)
+void rpmalAddProvides(rpmal al, alKey pkgKey, rpmds provides, uint32_t tscolor)
 {
-    uint_32 dscolor;
+    uint32_t dscolor;
     const char * Name;
     alNum pkgNum = alKey2Num(al, pkgKey);
     availableIndex ai = &al->index;
@@ -688,7 +675,7 @@ void rpmalMakeIndex(rpmal al)
     ai->k = 0;
     for (i = 0; i < al->size; i++) {
 	alp = al->list + i;
-	rpmalAddProvides(al, (alKey)i, alp->provides, alp->tscolor);
+	rpmalAddProvides(al, alNum2Key(NULL, (alNum)i), alp->provides, alp->tscolor);
     }
 
     /* Reset size to the no. of provides added. */
@@ -699,8 +686,8 @@ void rpmalMakeIndex(rpmal al)
 fnpyKey *
 rpmalAllFileSatisfiesDepend(const rpmal al, const rpmds ds, alKey * keyp)
 {
-    uint_32 tscolor;
-    uint_32 ficolor;
+    uint32_t tscolor;
+    uint32_t ficolor;
     int found = 0;
     const char * dirName;
     const char * baseName;
@@ -746,7 +733,6 @@ rpmalAllFileSatisfiesDepend(const rpmal al, const rpmds ds, alKey * keyp)
 	goto exit;
     baseName++;
 
-    /*@-branchstate@*/ /* FIX: ret is a problem */
     for (found = 0, ret = NULL;
 	 die < al->dirs + al->numDirs && dieCompare(die, dieNeedle) == 0;
 	 die++)
@@ -788,7 +774,6 @@ fprintf(stderr, "==> fie %p %s\n", fie, (fie->baseName ? fie->baseName : "(nil)"
 	    *keyp = alNum2Key(al, fie->pkgNum);
 	found++;
     }
-    /*@=branchstate@*/
 
 exit:
     dirName = _free(dirName);
@@ -863,7 +848,6 @@ rpmalAllSatisfiesDepend(const rpmal al, const rpmds ds, alKey * keyp)
 	    /*@switchbreak@*/ break;
 	}
 
-	/*@-branchstate@*/
 	if (rc) {
 	    ret = xrealloc(ret, (found + 2) * sizeof(*ret));
 	    if (ret)	/* can't happen */
@@ -874,7 +858,6 @@ rpmalAllSatisfiesDepend(const rpmal al, const rpmds ds, alKey * keyp)
 /*@=dependenttrans@*/
 	    found++;
 	}
-	/*@=branchstate@*/
     }
 
     if (ret)
