@@ -2,9 +2,13 @@
 
 #include <signal.h>	/* getOutputFrom() */
 
+#include <rpmio.h>
+#include <rpmcb.h>		/* XXX fnpyKey */
+#include <rpmtag.h>
 #define	_RPMEVR_INTERNAL
 #include <rpmbuild.h>
 #include <argv.h>
+#include <rpmmg.h>
 
 #define	_RPMFC_INTERNAL
 #include <rpmfc.h>
@@ -23,8 +27,8 @@
 /**
  */
 static int rpmfcExpandAppend(/*@out@*/ ARGV_t * argvp, const ARGV_t av)
-	/*@globals rpmGlobalMacroContext, h_errno @*/
-	/*@modifies *argvp, rpmGlobalMacroContext @*/
+	/*@globals rpmGlobalMacroContext, h_errno, internalState @*/
+	/*@modifies *argvp, rpmGlobalMacroContext, internalState @*/
 	/*@requires maxRead(argvp) >= 0 @*/
 {
     ARGV_t argv = *argvp;
@@ -32,9 +36,7 @@ static int rpmfcExpandAppend(/*@out@*/ ARGV_t * argvp, const ARGV_t av)
     int ac = argvCount(av);
     int i;
 
-/*@-bounds@*/	/* LCL: internal error */
     argv = xrealloc(argv, (argc + ac + 1) * sizeof(*argv));
-/*@=bounds@*/
     for (i = 0; i < ac; i++)
 	argv[argc + i] = rpmExpand(av[i], NULL);
     argv[argc + ac] = NULL;
@@ -90,18 +92,18 @@ static StringBuf getOutputFrom(/*@null@*/ const char * dir, ARGV_t argv,
 	    (void) Chdir(dir);
 	}
 	
-	rpmMessage(RPMMESS_DEBUG, _("\texecv(%s) pid %d\n"),
+	rpmlog(RPMLOG_DEBUG, D_("\texecv(%s) pid %d\n"),
                         argv[0], (unsigned)getpid());
 
 	unsetenv("MALLOC_CHECK_");
 	(void) execvp(argv[0], (char *const *)argv);
 	/* XXX this error message is probably not seen. */
-	rpmError(RPMERR_EXEC, _("Couldn't exec %s: %s\n"),
+	rpmlog(RPMLOG_ERR, _("Couldn't exec %s: %s\n"),
 		argv[0], strerror(errno));
-	_exit(RPMERR_EXEC);
+	_exit(EXIT_FAILURE);
     }
     if (child < 0) {
-	rpmError(RPMERR_FORK, _("Couldn't fork %s: %s\n"),
+	rpmlog(RPMLOG_ERR, _("Couldn't fork %s: %s\n"),
 		argv[0], strerror(errno));
 	return NULL;
     }
@@ -161,14 +163,12 @@ top:
 	}
 	
 	/* Read any data from prog */
-/*@-boundswrite@*/
 	{   char buf[BUFSIZ+1];
 	    while ((nbr = read(fromProg[0], buf, sizeof(buf)-1)) > 0) {
 		buf[nbr] = '\0';
 		appendStringBuf(readBuff, buf);
 	    }
 	}
-/*@=boundswrite@*/
 
 	/* terminate on (non-blocking) EOF or error */
 	done = (nbr == 0 || (nbr < 0 && errno != EAGAIN));
@@ -186,19 +186,19 @@ top:
 
     /* Collect status from prog */
     reaped = waitpid(child, &status, 0);
-    rpmMessage(RPMMESS_DEBUG, _("\twaitpid(%d) rc %d status %x\n"),
+    rpmlog(RPMLOG_DEBUG, D_("\twaitpid(%d) rc %d status %x\n"),
         (unsigned)child, (unsigned)reaped, status);
 
     if (failNonZero && (!WIFEXITED(status) || WEXITSTATUS(status))) {
 	const char *cmd = argvJoin(argv);
 	int rc = (WIFEXITED(status) ? WEXITSTATUS(status) : -1);
 
-	rpmError(RPMERR_EXEC, _("Command \"%s\" failed, exit(%d)\n"), cmd, rc);
+	rpmlog(RPMLOG_ERR, _("Command \"%s\" failed, exit(%d)\n"), cmd, rc);
 	cmd = _free(cmd);
 	return NULL;
     }
     if (writeBytesLeft) {
-	rpmError(RPMERR_EXEC, _("failed to write all data to %s\n"), argv[0]);
+	rpmlog(RPMLOG_ERR, _("failed to write all data to %s\n"), argv[0]);
 	return NULL;
     }
     return readBuff;
@@ -235,11 +235,9 @@ int rpmfcExec(ARGV_t av, StringBuf sb_stdin, StringBuf * sb_stdoutp,
 
     /* Build argv, appending args to the executable args. */
     xav = NULL;
-/*@-boundswrite@*/
     xx = argvAppend(&xav, pav);
     if (av[1])
 	xx = rpmfcExpandAppend(&xav, av + 1);
-/*@=boundswrite@*/
 
     if (sb_stdin != NULL) {
 	buf_stdin = getStringBuf(sb_stdin);
@@ -249,12 +247,10 @@ int rpmfcExec(ARGV_t av, StringBuf sb_stdin, StringBuf * sb_stdoutp,
     /* Read output from exec'd helper. */
     sb = getOutputFrom(NULL, xav, buf_stdin, buf_stdin_len, failnonzero);
 
-/*@-branchstate@*/
     if (sb_stdoutp != NULL) {
 	*sb_stdoutp = sb;
 	sb = NULL;	/* XXX don't free */
     }
-/*@=branchstate@*/
 
     ec = 0;
 
@@ -285,11 +281,12 @@ static int rpmfcSaveArg(/*@out@*/ ARGV_t * argvp, const char * key)
  */
 static char * rpmfcFileDep(/*@returned@*/ char * buf, int ix,
 		/*@null@*/ rpmds ds)
-	/*@modifies buf @*/
+	/*@globals fileSystem, internalState @*/
+	/*@modifies buf, fileSystem, internalState @*/
 	/*@requires maxSet(buf) >= 0 @*/
 	/*@ensures maxRead(buf) == 0 @*/
 {
-    int_32 tagN = rpmdsTagN(ds);
+    uint32_t tagN = rpmdsTagN(ds);
     char deptype = 'X';
 
     buf[0] = '\0';
@@ -328,7 +325,9 @@ static int rpmfcHelper(rpmfc fc, unsigned char deptype, const char * nsdep)
     rpmds * depsp, ds;
     const char * N;
     const char * EVR;
-    int_32 Flags, dsContext, tagN;
+    rpmTag tagN;
+    evrFlags Flags;
+    evrFlags dsContext;
     ARGV_t pav;
     const char * s;
     int pac;
@@ -363,9 +362,7 @@ static int rpmfcHelper(rpmfc fc, unsigned char deptype, const char * nsdep)
     sb_stdin = newStringBuf();
     appendLineStringBuf(sb_stdin, fn);
     sb_stdout = NULL;
-/*@-boundswrite@*/
     xx = rpmfcExec(av, sb_stdin, &sb_stdout, 0);
-/*@=boundswrite@*/
     sb_stdin = freeStringBuf(sb_stdin);
 
     if (xx == 0 && sb_stdout != NULL) {
@@ -377,7 +374,6 @@ static int rpmfcHelper(rpmfc fc, unsigned char deptype, const char * nsdep)
 	    N = pav[i];
 	    EVR = "";
 	    Flags = dsContext;
-/*@-branchstate@*/
 	    if (pav[i+1] && strchr("=<>", *pav[i+1])) {
 		i++;
 		for (s = pav[i]; *s; s++) {
@@ -400,7 +396,6 @@ assert(*s != '\0');
 		EVR = pav[i];
 assert(EVR != NULL);
 	    }
-/*@=branchstate@*/
 
 
 	    /* Add tracking dependency for versioned Provides: */
@@ -419,9 +414,7 @@ assert(EVR != NULL);
 	    xx = rpmdsMerge(depsp, ds);
 
 	    /* Add to file dependencies. */
-/*@-boundswrite@*/
 	    xx = rpmfcSaveArg(&fc->ddict, rpmfcFileDep(buf, fc->ix, ds));
-/*@=boundswrite@*/
 
 	    ds = rpmdsFree(ds);
 	}
@@ -476,6 +469,8 @@ static struct rpmfcTokens_s rpmfcTokens[] = {
 
   { "Java ",			RPMFC_JAVA|RPMFC_INCLUDE },
 
+  { "PE executable",            RPMFC_MONO|RPMFC_INCLUDE },
+
   { "current ar archive",	RPMFC_STATIC|RPMFC_LIBRARY|RPMFC_ARCHIVE|RPMFC_INCLUDE },
 
   { "Zip archive data",		RPMFC_COMPRESSED|RPMFC_ARCHIVE|RPMFC_INCLUDE },
@@ -689,7 +684,6 @@ static int rpmfcSCRIPT(rpmfc fc)
     }
 
     /* Look for #! interpreter in first 10 lines. */
-/*@-boundswrite@*/
     for (i = 0; i < 10; i++) {
 
 	s = fgets(buf, sizeof(buf) - 1, fp);
@@ -727,7 +721,9 @@ static int rpmfcSCRIPT(rpmfc fc)
 
 	/* Set color based on interpreter name. */
 	/* XXX magic token should have already done this?!? */
+/*@-moduncon@*/
 	bn = basename(s);
+/*@=moduncon@*/
 	if (!strcmp(bn, "perl"))
 	    fc->fcolor->vals[fc->ix] |= RPMFC_PERL;
 	else if (!strncmp(bn, "python", sizeof("python")-1))
@@ -737,7 +733,6 @@ static int rpmfcSCRIPT(rpmfc fc)
 
 	break;
     }
-/*@=boundswrite@*/
 
     (void) fclose(fp);
 
@@ -781,8 +776,12 @@ static int rpmfcSCRIPT(rpmfc fc)
 	xx = rpmfcHelper(fc, 'P', "php");
 	if (is_executable)
 	    xx = rpmfcHelper(fc, 'R', "php");
+    } else
+    if (fc->fcolor->vals[fc->ix] & RPMFC_MONO) {
+	xx = rpmfcHelper(fc, 'P', "mono");
+	if (is_executable)
+	    xx = rpmfcHelper(fc, 'R', "mono");
     }
-
     return 0;
 }
 
@@ -794,16 +793,15 @@ static int rpmfcSCRIPT(rpmfc fc)
  * @return		0 on success
  */
 static int rpmfcMergePR(void * context, rpmds ds)
-	/*@modifies ds @*/
+	/*@globals fileSystem, internalState @*/
+	/*@modifies ds, fileSystem, internalState @*/
 {
     rpmfc fc = context;
     char buf[BUFSIZ];
     int rc = -1;
 
-/*@-modfilesys@*/
 if (_rpmfc_debug < 0)
-fprintf(stderr, "*** %s(%p, %p) %s\n", __FUNCTION__, context, ds, tagName(rpmdsTagN(ds)));
-/*@=modfilesys@*/
+fprintf(stderr, "*** rpmfcMergePR(%p, %p) %s\n", context, ds, tagName(rpmdsTagN(ds)));
     switch(rpmdsTagN(ds)) {
     default:
 	break;
@@ -858,11 +856,11 @@ typedef struct rpmfcApplyTbl_s {
 /*@unchecked@*/
 static struct rpmfcApplyTbl_s rpmfcApplyTable[] = {
     { rpmfcELF,		RPMFC_ELF },
-    { rpmfcSCRIPT,	(RPMFC_SCRIPT|RPMFC_PERL|RPMFC_PYTHON|RPMFC_LIBTOOL|RPMFC_PKGCONFIG|RPMFC_BOURNE|RPMFC_JAVA|RPMFC_PHP) },
+    { rpmfcSCRIPT,	(RPMFC_SCRIPT|RPMFC_PERL|RPMFC_PYTHON|RPMFC_LIBTOOL|RPMFC_PKGCONFIG|RPMFC_BOURNE|RPMFC_JAVA|RPMFC_PHP|RPMFC_MONO) },
     { NULL, 0 }
 };
 
-int rpmfcApply(rpmfc fc)
+rpmRC rpmfcApply(rpmfc fc)
 {
     rpmfcApplyTbl fcat;
     const char * s;
@@ -870,7 +868,7 @@ int rpmfcApply(rpmfc fc)
     rpmds ds;
     const char * N;
     const char * EVR;
-    int_32 Flags;
+    evrFlags Flags;
     unsigned char deptype;
     int nddict;
     int previx;
@@ -881,6 +879,8 @@ int rpmfcApply(rpmfc fc)
     int xx;
     int skipping;
 
+/* Make sure something didn't go wrong previously! */
+assert(fc->fn != NULL);
     /* Generate package and per-file dependencies. */
     for (fc->ix = 0; fc->fn[fc->ix] != NULL; fc->ix++) {
 
@@ -889,7 +889,8 @@ int rpmfcApply(rpmfc fc)
 	{   const char *fn = strstr(fc->fn[fc->ix], "/usr/lib");
 	    if (fn) {
 		fn += sizeof("/usr/lib")-1;
-		if (fn[0] == '6' && fn[1] == '4')
+		if ((fn[0] == '3' && fn[1] == '2') || 
+			(fn[0] == '6' && fn[1] == '4'))
 		    fn += 2;
 		if (!strncmp(fn, "/python", sizeof("/python")-1))
 		    fc->fcolor->vals[fc->ix] |= RPMFC_PYTHON;
@@ -904,7 +905,6 @@ int rpmfcApply(rpmfc fc)
 	}
     }
 
-/*@-boundswrite@*/
     /* Generate per-file indices into package dependencies. */
     nddict = argvCount(fc->ddict);
     previx = -1;
@@ -963,29 +963,29 @@ assert(dix >= 0);
 	if (fc->fddictn && fc->fddictn->vals && !skipping)
 	    fc->fddictn->vals[ix]++;
     }
-/*@=boundswrite@*/
 
-    return 0;
+    return RPMRC_OK;
 }
 
-int rpmfcClassify(rpmfc fc, ARGV_t argv, int_16 * fmode)
+rpmRC rpmfcClassify(rpmfc fc, ARGV_t argv, uint16_t * fmode)
 {
     ARGV_t fcav = NULL;
     ARGV_t dav;
+    rpmmg mg = NULL;
     const char * s, * se;
     size_t slen;
     int fcolor;
     int xx;
-    const char * magicfile;
-    int msflags = MAGIC_CHECK;	/* XXX MAGIC_COMPRESS flag? */
-    magic_t ms = NULL;
+    const char * magicfile = NULL;
 
     if (fc == NULL || argv == NULL)
-	return 0;
+	return RPMRC_OK;
 
     magicfile = rpmExpand("%{?_rpmfc_magic_path}", NULL);
-    if (magicfile == NULL || *magicfile == '\0' || *magicfile == '%')
-	goto exit;
+    if (magicfile == NULL || *magicfile == '\0')
+	magicfile = _free(magicfile);
+    mg = rpmmgNew(magicfile, 0);
+assert(mg != NULL);	/* XXX figger a proper return path. */
 
     fc->nfiles = argvCount(argv);
 
@@ -997,32 +997,17 @@ int rpmfcClassify(rpmfc fc, ARGV_t argv, int_16 * fmode)
     xx = argvAdd(&fc->cdict, "");
     xx = argvAdd(&fc->cdict, "directory");
 
-    ms = magic_open(msflags);
-    if (ms == NULL) {
-	xx = RPMERR_EXEC;
-	rpmError(xx, _("magic_open(0x%x) failed: %s\n"),
-		msflags, strerror(errno));
-assert(ms != NULL);	/* XXX figger a proper return path. */
-    }
-
-    xx = magic_load(ms, magicfile);
-    if (xx == -1) {
-	xx = RPMERR_EXEC;
-	rpmError(xx, _("magic_load(ms, \"%s\") failed: %s\n"),
-		magicfile, magic_error(ms));
-assert(xx != -1);	/* XXX figger a proper return path. */
-    }
-
     for (fc->ix = 0; fc->ix < fc->nfiles; fc->ix++) {
 	const char * ftype;
-	int_16 mode = (fmode ? fmode[fc->ix] : 0);
+	int freeftype;
+	uint16_t mode = (fmode ? fmode[fc->ix] : 0);
 	int urltype;
 
+	ftype = "";	freeftype = 0;
 	urltype = urlPath(argv[fc->ix], &s);
 assert(s != NULL && *s == '/');
 	slen = strlen(s);
 
-/*@-branchstate@*/
 	switch (mode & S_IFMT) {
 	case S_IFCHR:	ftype = "character special";	/*@switchbreak@*/ break;
 	case S_IFBLK:	ftype = "block special";	/*@switchbreak@*/ break;
@@ -1069,21 +1054,16 @@ assert(s != NULL && *s == '/');
 	    /* XXX skip all files in /dev/ which are (or should be) %dev dummies. */
 	    else if (slen >= fc->brlen+sizeof("/dev/") && !strncmp(s+fc->brlen, "/dev/", sizeof("/dev/")-1))
 		ftype = "";
-	    else
-		ftype = magic_file(ms, s);
-
-	    if (ftype == NULL) {
-		xx = RPMERR_EXEC;
-		rpmError(xx, _("magic_file(ms, \"%s\") failed: mode %06o %s\n"),
-			s, mode, magic_error(ms));
-assert(ftype != NULL);	/* XXX figger a proper return path. */
+	    else if (magicfile) {
+		ftype = rpmmgFile(mg, s);
+assert(ftype != NULL);	/* XXX never happens, rpmmgFile() returns "" */
+		freeftype = 1;
 	    }
 	    /*@switchbreak@*/ break;
 	}
-/*@=branchstate@*/
 
 	se = ftype;
-        rpmMessage(RPMMESS_DEBUG, "%s: %s\n", s, se);
+        rpmlog(RPMLOG_DEBUG, "%s: %s\n", s, se);
 
 	/* Save the path. */
 	xx = argvAdd(&fc->fn, s);
@@ -1095,10 +1075,13 @@ assert(ftype != NULL);	/* XXX figger a proper return path. */
 	fcolor = rpmfcColoring(se);
 	xx = argiAdd(&fc->fcolor, fc->ix, fcolor);
 
-/*@-boundswrite@*/
 	if (fcolor != RPMFC_WHITE && (fcolor & RPMFC_INCLUDE))
 	    xx = rpmfcSaveArg(&fc->cdict, se);
-/*@=boundswrite@*/
+
+/*@-modobserver -observertrans @*/	/* XXX mixed types in variable */
+	if (freeftype)
+	    ftype = _free(ftype);
+/*@=modobserver =observertrans @*/
     }
 
     /* Build per-file class index array. */
@@ -1119,13 +1102,10 @@ assert(se != NULL);
 
     fcav = argvFree(fcav);
 
-    if (ms != NULL)
-	magic_close(ms);
-
-exit:
+    mg = rpmmgFree(mg);
     magicfile = _free(magicfile);
 
-    return 0;
+    return RPMRC_OK;
 }
 
 /**
@@ -1201,7 +1181,7 @@ static void printDeps(Header h)
     rpmds ds = NULL;
     int flags = 0x2;	/* XXX no filtering, !scareMem */
     const char * DNEVR;
-    int_32 Flags;
+    evrFlags Flags;
     int bingo = 0;
 
     for (dm = DepMsgs; dm->msg != NULL; dm++) {
@@ -1224,22 +1204,22 @@ static void printDeps(Header h)
 	    if (!((Flags & dm->mask) ^ dm->xor))
 		/*@innercontinue@*/ continue;
 	    if (bingo == 0) {
-		rpmMessage(RPMMESS_NORMAL, "%s:", (dm->msg ? dm->msg : ""));
+		rpmlog(RPMLOG_NOTICE, "%s:", (dm->msg ? dm->msg : ""));
 		bingo = 1;
 	    }
 	    if ((DNEVR = rpmdsDNEVR(ds)) == NULL)
 		/*@innercontinue@*/ continue;	/* XXX can't happen */
-	    rpmMessage(RPMMESS_NORMAL, " %s", DNEVR+2);
+	    rpmlog(RPMLOG_NOTICE, " %s", DNEVR+2);
 	}
 	if (bingo)
-	    rpmMessage(RPMMESS_NORMAL, "\n");
+	    rpmlog(RPMLOG_NOTICE, "\n");
     }
     ds = rpmdsFree(ds);
 }
 
 /**
  */
-static int rpmfcGenerateDependsHelper(const Spec spec, Package pkg, rpmfi fi)
+static rpmRC rpmfcGenerateDependsHelper(const Spec spec, Package pkg, rpmfi fi)
 	/*@globals rpmGlobalMacroContext, h_errno, fileSystem, internalState @*/
         /*@modifies fi, rpmGlobalMacroContext, fileSystem, internalState @*/
 {
@@ -1247,7 +1227,7 @@ static int rpmfcGenerateDependsHelper(const Spec spec, Package pkg, rpmfi fi)
     StringBuf sb_stdout;
     DepMsg_t dm;
     int failnonzero = 0;
-    int rc = 0;
+    rpmRC rc = RPMRC_OK;
 
     /*
      * Create file manifest buffer to deliver to dependency finder.
@@ -1285,20 +1265,18 @@ static int rpmfcGenerateDependsHelper(const Spec spec, Package pkg, rpmfi fi)
 	    /*@notreached@*/ /*@switchbreak@*/ break;
 	}
 
-/*@-boundswrite@*/
 	xx = rpmfcExec(dm->argv, sb_stdin, &sb_stdout, failnonzero);
-/*@=boundswrite@*/
 	if (xx == -1)
 	    continue;
 
 	s = rpmExpand(dm->argv[0], NULL);
-	rpmMessage(RPMMESS_NORMAL, _("Finding  %s: %s\n"), dm->msg,
+	rpmlog(RPMLOG_NOTICE, _("Finding  %s: %s\n"), dm->msg,
 		(s ? s : ""));
 	s = _free(s);
 
 	if (sb_stdout == NULL) {
-	    rc = RPMERR_EXEC;
-	    rpmError(rc, _("Failed to find %s:\n"), dm->msg);
+	    rpmlog(RPMLOG_ERR, _("Failed to find %s:\n"), dm->msg);
+	    rc = RPMRC_FAIL;
 	    break;
 	}
 
@@ -1309,7 +1287,7 @@ static int rpmfcGenerateDependsHelper(const Spec spec, Package pkg, rpmfi fi)
 	sb_stdout = freeStringBuf(sb_stdout);
 
 	if (rc) {
-	    rpmError(rc, _("Failed to find %s:\n"), dm->msg);
+	    rpmlog(RPMLOG_ERR, _("Failed to find %s:\n"), dm->msg);
 	    break;
 	}
     }
@@ -1347,40 +1325,42 @@ static int rpmfcGenerateScriptletDeps(const Spec spec, Package pkg)
 	/*@globals rpmGlobalMacroContext, h_errno, fileSystem, internalState @*/
         /*@modifies rpmGlobalMacroContext, fileSystem, internalState @*/
 {
-    HGE_t hge = (HGE_t)headerGetEntryMinMemory;
+    HE_t he = memset(alloca(sizeof(*he)), 0, sizeof(*he));
     StringBuf sb_stdin = newStringBuf();
     StringBuf sb_stdout = NULL;
     DepMsg_t dm;
     int failnonzero = 0;
     int rc = 0;
+    int xx;
 
-/*@-branchstate@*/
     for (dm = ScriptMsgs; dm->msg != NULL; dm++) {
 	int tag, tagflags;
 	char * s;
-	int xx;
 
 	tag = dm->ftag;
 	tagflags = RPMSENSE_FIND_REQUIRES | dm->mask;
 
 	/* Retrieve scriptlet interpreter. */
-	s = NULL;
-	if (!hge(pkg->header, dm->ntag, NULL, (void **)&s, NULL) || s == NULL)
+	he->tag = dm->ntag;
+	xx = headerGet(pkg->header, he, 0);
+	if (!xx || he->p.str == NULL)
 	    continue;
-	if (strcmp(s, "/bin/sh") && strcmp(s, "/bin/bash"))
+	xx = strcmp(he->p.str, "/bin/sh") && strcmp(he->p.str, "/bin/bash");
+	he->p.ptr = _free(he->p.ptr);
+	if (xx)
 	    continue;
 
 	/* Retrieve scriptlet body. */
-	s = NULL;
-	if (!hge(pkg->header, dm->vtag, NULL, (void **)&s, NULL) || s == NULL)
+	he->tag = dm->vtag;
+	xx = headerGet(pkg->header, he, 0);
+	if (!xx || he->p.str == NULL)
 	    continue;
 	truncStringBuf(sb_stdin);
-	appendLineStringBuf(sb_stdin, s);
+	appendLineStringBuf(sb_stdin, he->p.str);
 	stripTrailingBlanksStringBuf(sb_stdin);
+	he->p.ptr = _free(he->p.ptr);
 
-/*@-boundswrite@*/
 	xx = rpmfcExec(dm->argv, sb_stdin, &sb_stdout, failnonzero);
-/*@=boundswrite@*/
 	if (xx == -1)
 	    continue;
 
@@ -1405,15 +1385,15 @@ static int rpmfcGenerateScriptletDeps(const Spec spec, Package pkg)
 	sb_stdout = freeStringBuf(sb_stdout);
 
     }
-/*@=branchstate@*/
 
     sb_stdin = freeStringBuf(sb_stdin);
 
     return rc;
 }
 
-int rpmfcGenerateDepends(void * specp, void * pkgp)
+rpmRC rpmfcGenerateDepends(void * specp, void * pkgp)
 {
+    HE_t he = memset(alloca(sizeof(*he)), 0, sizeof(*he));
     const Spec spec = specp;
     Package pkg = pkgp;
     rpmfi fi = pkg->cpioList;
@@ -1421,24 +1401,23 @@ int rpmfcGenerateDepends(void * specp, void * pkgp)
     rpmds ds;
     int flags = 0x2;	/* XXX no filtering, !scareMem */
     ARGV_t av;
-    int_16 * fmode;
+    uint16_t * fmode;
     int ac = rpmfiFC(fi);
-    const void ** p;
     char buf[BUFSIZ];
     const char * N;
     const char * EVR;
     int genConfigDeps, internaldeps;
-    int c;
-    int rc = 0;
+    rpmRC rc = RPMRC_OK;
+    int i;
     int xx;
 
     /* Skip packages with no files. */
     if (ac <= 0)
-	return 0;
+	return RPMRC_OK;
 
     /* Skip packages that have dependency generation disabled. */
     if (! (pkg->autoReq || pkg->autoProv))
-	return 0;
+	return RPMRC_OK;
 
     /* If new-fangled dependency generation is disabled ... */
     internaldeps = rpmExpandNumeric("%{?_use_internal_dependency_generator}");
@@ -1454,25 +1433,24 @@ int rpmfcGenerateDepends(void * specp, void * pkgp)
 	xx = rpmfcGenerateScriptletDeps(spec, pkg);
 
     /* Extract absolute file paths in argv format. */
+    /* XXX TODO: should use argvFoo ... */
     av = xcalloc(ac+1, sizeof(*av));
     fmode = xcalloc(ac+1, sizeof(*fmode));
 
-/*@-boundswrite@*/
     genConfigDeps = 0;
     fi = rpmfiInit(fi, 0);
     if (fi != NULL)
-    while ((c = rpmfiNext(fi)) >= 0) {
+    while ((i = rpmfiNext(fi)) >= 0) {
 	rpmfileAttrs fileAttrs;
 
 	/* Does package have any %config files? */
 	fileAttrs = rpmfiFFlags(fi);
 	genConfigDeps |= (fileAttrs & RPMFILE_CONFIG);
 
-	av[c] = xstrdup(rpmfiFN(fi));
-	fmode[c] = rpmfiFMode(fi);
+	av[i] = xstrdup(rpmfiFN(fi));
+	fmode[i] = rpmfiFMode(fi);
     }
     av[ac] = NULL;
-/*@=boundswrite@*/
 
     fc = rpmfcNew();
     fc->skipProv = !pkg->autoProv;
@@ -1493,9 +1471,12 @@ int rpmfcGenerateDepends(void * specp, void * pkgp)
 	ds = rpmdsNew(pkg->header, RPMTAG_PROVIDENAME, flags);
 	xx = rpmdsMerge(&fc->provides, ds);
 	ds = rpmdsFree(ds);
-	xx = headerRemoveEntry(pkg->header, RPMTAG_PROVIDENAME);
-	xx = headerRemoveEntry(pkg->header, RPMTAG_PROVIDEVERSION);
-	xx = headerRemoveEntry(pkg->header, RPMTAG_PROVIDEFLAGS);
+	he->tag = RPMTAG_PROVIDENAME;
+	xx = headerDel(pkg->header, he, 0);
+	he->tag = RPMTAG_PROVIDEVERSION;
+	xx = headerDel(pkg->header, he, 0);
+	he->tag = RPMTAG_PROVIDEFLAGS;
+	xx = headerDel(pkg->header, he, 0);
 
 	/* Add config dependency, Provides: config(N) = EVR */
 	if (genConfigDeps) {
@@ -1515,9 +1496,12 @@ assert(EVR != NULL);
 	ds = rpmdsNew(pkg->header, RPMTAG_REQUIRENAME, flags);
 	xx = rpmdsMerge(&fc->requires, ds);
 	ds = rpmdsFree(ds);
-	xx = headerRemoveEntry(pkg->header, RPMTAG_REQUIRENAME);
-	xx = headerRemoveEntry(pkg->header, RPMTAG_REQUIREVERSION);
-	xx = headerRemoveEntry(pkg->header, RPMTAG_REQUIREFLAGS);
+	he->tag = RPMTAG_REQUIRENAME;
+	xx = headerDel(pkg->header, he, 0);
+	he->tag = RPMTAG_REQUIREVERSION;
+	xx = headerDel(pkg->header, he, 0);
+	he->tag = RPMTAG_REQUIREFLAGS;
+	xx = headerDel(pkg->header, he, 0);
 
 	/* Add config dependency,  Requires: config(N) = EVR */
 	if (genConfigDeps) {
@@ -1540,96 +1524,118 @@ assert(EVR != NULL);
     xx = rpmfcApply(fc);
 
     /* Add per-file colors(#files) */
-    p = (const void **) argiData(fc->fcolor);
-    c = argiCount(fc->fcolor);
-assert(ac == c);
-    if (p != NULL && c > 0) {
-	int_32 * fcolors = (int_32 *)p;
-	int i;
+    he->tag = RPMTAG_FILECOLORS;
+    he->t = RPM_UINT32_TYPE;
+    he->p.ui32p = argiData(fc->fcolor);
+    he->c = argiCount(fc->fcolor);
+assert(ac == he->c);
+    if (he->p.ptr != NULL && he->c > 0) {
+	uint32_t * fcolors = he->p.ui32p;
 
 	/* XXX Make sure only primary (i.e. Elf32/Elf64) colors are added. */
-	for (i = 0; i < c; i++)
+	for (i = 0; i < he->c; i++)
 	    fcolors[i] &= 0x0f;
-	xx = headerAddEntry(pkg->header, RPMTAG_FILECOLORS, RPM_INT32_TYPE,
-			p, c);
+
+	xx = headerPut(pkg->header, he, 0);
     }
 
     /* Add classes(#classes) */
-    p = (const void **) argvData(fc->cdict);
-    c = argvCount(fc->cdict);
-    if (p != NULL && c > 0)
-	xx = headerAddEntry(pkg->header, RPMTAG_CLASSDICT, RPM_STRING_ARRAY_TYPE,
-			p, c);
+    he->tag = RPMTAG_CLASSDICT;
+    he->t = RPM_STRING_ARRAY_TYPE;
+    he->p.argv = argvData(fc->cdict);
+    he->c = argvCount(fc->cdict);
+    if (he->p.ptr != NULL && he->c > 0) {
+	xx = headerPut(pkg->header, he, 0);
+    }
 
     /* Add per-file classes(#files) */
-    p = (const void **) argiData(fc->fcdictx);
-    c = argiCount(fc->fcdictx);
-assert(ac == c);
-    if (p != NULL && c > 0)
-	xx = headerAddEntry(pkg->header, RPMTAG_FILECLASS, RPM_INT32_TYPE,
-			p, c);
+    he->tag = RPMTAG_FILECLASS;
+    he->t = RPM_UINT32_TYPE;
+    he->p.ui32p = argiData(fc->fcdictx);
+    he->c = argiCount(fc->fcdictx);
+assert(ac == he->c);
+    if (he->p.ptr != NULL && he->c > 0) {
+	xx = headerPut(pkg->header, he, 0);
+    }
 
     /* Add Provides: */
-/*@-branchstate@*/
-    if (fc->provides != NULL && (c = rpmdsCount(fc->provides)) > 0 && !fc->skipProv) {
-	p = (const void **) fc->provides->N;
-	xx = headerAddEntry(pkg->header, RPMTAG_PROVIDENAME, RPM_STRING_ARRAY_TYPE,
-			p, c);
+    if (fc->provides != NULL && (he->c = rpmdsCount(fc->provides)) > 0
+     && !fc->skipProv)
+    {
+	he->tag = RPMTAG_PROVIDENAME;
+	he->t = RPM_STRING_ARRAY_TYPE;
+	he->p.argv = fc->provides->N;
+	xx = headerPut(pkg->header, he, 0);
+
 	/* XXX rpm prior to 3.0.2 did not always supply EVR and Flags. */
 /*@-nullpass@*/
-	p = (const void **) fc->provides->EVR;
-assert(p != NULL);
-	xx = headerAddEntry(pkg->header, RPMTAG_PROVIDEVERSION, RPM_STRING_ARRAY_TYPE,
-			p, c);
-	p = (const void **) fc->provides->Flags;
-assert(p != NULL);
-	xx = headerAddEntry(pkg->header, RPMTAG_PROVIDEFLAGS, RPM_INT32_TYPE,
-			p, c);
+	he->tag = RPMTAG_PROVIDEVERSION;
+	he->t = RPM_STRING_ARRAY_TYPE;
+	he->p.argv = fc->provides->EVR;
+assert(he->p.ptr != NULL);
+	xx = headerPut(pkg->header, he, 0);
+
+	he->tag = RPMTAG_PROVIDEFLAGS;
+	he->t = RPM_UINT32_TYPE;
+	he->p.ui32p = (uint32_t *) fc->provides->Flags;
+assert(he->p.ptr != NULL);
+	xx = headerPut(pkg->header, he, 0);
 /*@=nullpass@*/
     }
-/*@=branchstate@*/
 
     /* Add Requires: */
-/*@-branchstate@*/
-    if (fc->requires != NULL && (c = rpmdsCount(fc->requires)) > 0 && !fc->skipReq) {
-	p = (const void **) fc->requires->N;
-	xx = headerAddEntry(pkg->header, RPMTAG_REQUIRENAME, RPM_STRING_ARRAY_TYPE,
-			p, c);
+    if (fc->requires != NULL && (he->c = rpmdsCount(fc->requires)) > 0
+     && !fc->skipReq)
+    {
+	he->tag = RPMTAG_REQUIRENAME;
+	he->t = RPM_STRING_ARRAY_TYPE;
+	he->p.argv = fc->requires->N;
+assert(he->p.ptr != NULL);
+	xx = headerPut(pkg->header, he, 0);
+
 	/* XXX rpm prior to 3.0.2 did not always supply EVR and Flags. */
 /*@-nullpass@*/
-	p = (const void **) fc->requires->EVR;
-assert(p != NULL);
-	xx = headerAddEntry(pkg->header, RPMTAG_REQUIREVERSION, RPM_STRING_ARRAY_TYPE,
-			p, c);
-	p = (const void **) fc->requires->Flags;
-assert(p != NULL);
-	xx = headerAddEntry(pkg->header, RPMTAG_REQUIREFLAGS, RPM_INT32_TYPE,
-			p, c);
+	he->tag = RPMTAG_REQUIREVERSION;
+	he->t = RPM_STRING_ARRAY_TYPE;
+	he->p.argv = fc->requires->EVR;
+assert(he->p.ptr != NULL);
+	xx = headerPut(pkg->header, he, 0);
+
+	he->tag = RPMTAG_REQUIREFLAGS;
+	he->t = RPM_UINT32_TYPE;
+	he->p.ui32p = (uint32_t *) fc->requires->Flags;
+assert(he->p.ptr != NULL);
+	xx = headerPut(pkg->header, he, 0);
 /*@=nullpass@*/
     }
-/*@=branchstate@*/
 
     /* Add dependency dictionary(#dependencies) */
-    p = (const void **) argiData(fc->ddictx);
-    c = argiCount(fc->ddictx);
-    if (p != NULL)
-	xx = headerAddEntry(pkg->header, RPMTAG_DEPENDSDICT, RPM_INT32_TYPE,
-			p, c);
+    he->tag = RPMTAG_DEPENDSDICT;
+    he->t = RPM_UINT32_TYPE;
+    he->p.ui32p = argiData(fc->ddictx);
+    he->c = argiCount(fc->ddictx);
+    if (he->p.ptr != NULL) {
+	xx = headerPut(pkg->header, he, 0);
+    }
 
     /* Add per-file dependency (start,number) pairs (#files) */
-    p = (const void **) argiData(fc->fddictx);
-    c = argiCount(fc->fddictx);
-assert(ac == c);
-    if (p != NULL)
-	xx = headerAddEntry(pkg->header, RPMTAG_FILEDEPENDSX, RPM_INT32_TYPE,
-			p, c);
+    he->tag = RPMTAG_FILEDEPENDSX;
+    he->t = RPM_UINT32_TYPE;
+    he->p.ui32p = argiData(fc->fddictx);
+    he->c = argiCount(fc->fddictx);
+assert(ac == he->c);
+    if (he->p.ptr != NULL) {
+	xx = headerPut(pkg->header, he, 0);
+    }
 
-    p = (const void **) argiData(fc->fddictn);
-    c = argiCount(fc->fddictn);
-assert(ac == c);
-    if (p != NULL)
-	xx = headerAddEntry(pkg->header, RPMTAG_FILEDEPENDSN, RPM_INT32_TYPE,
-			p, c);
+    he->tag = RPMTAG_FILEDEPENDSN;
+    he->t = RPM_UINT32_TYPE;
+    he->p.ui32p = argiData(fc->fddictn);
+    he->c = argiCount(fc->fddictn);
+assert(ac == he->c);
+    if (he->p.ptr != NULL) {
+	xx = headerPut(pkg->header, he, 0);
+    }
 
     printDeps(pkg->header);
 
