@@ -1,3 +1,8 @@
+/*@-bounds@*/
+/*@-retalias@*/
+/*@-shiftimplementation@*/
+/*@-temptrans@*/
+/*@-unreachable@*/
 /* Copyright (C) 1991-1993, 1996-1999, 2000 Free Software Foundation, Inc.
    This file is part of the GNU C Library.
 
@@ -20,7 +25,7 @@
 
 /* Find the first occurrence of C in S or the final NUL byte.  */
 static inline char *
-__strchrnul (const char *s, int c)
+internal__strchrnul (const char *s, int c)
 {
   const unsigned char *char_ptr;
   const unsigned long int *longword_ptr;
@@ -231,17 +236,6 @@ __strchrnul (const char *s, int c)
     || STREQ (string, "cntrl") || STREQ (string, "blank"))
 # endif
 
-/* Avoid depending on library functions or files
-   whose names are inconsistent.  */
-
-# if !defined _LIBC && !defined getenv
-extern char *getenv ();
-# endif
-
-# ifndef errno
-extern int errno;
-# endif
-
 /* Match STRING against the filename pattern PATTERN, returning zero if
    it matches, nonzero if not.  */
 static int
@@ -348,7 +342,7 @@ internal_fnmatch (const char *pattern, const char *string,
 	    {
 	      const char *endp;
 
-	      endp = __strchrnul (n, (flags & FNM_FILE_NAME) ? '/' : '\0');
+	      endp = internal__strchrnul (n, (flags & FNM_FILE_NAME) ? '/' : '\0');
 
 	      if (c == '[')
 		{
@@ -608,3 +602,8 @@ fnmatch (const char *pattern, const char *string, int flags)
 }
 
 #endif	/* _LIBC or not __GNU_LIBRARY__.  */
+/*@=unreachable@*/
+/*@=temptrans@*/
+/*@=shiftimplementation@*/
+/*@=retalias@*/
+/*@=bounds@*/
