@@ -6,7 +6,7 @@
 #define COMMENTCHAR '#'
 #endif
 
-#if HAVE_STRUCT_MNTTAB 
+#if defined(HAVE_STRUCT_MNTTAB)
 our_mntent * getmntent(FILE *filep) {
     static struct mnttab entry;
     static our_mntent item;
