@@ -48,6 +48,7 @@
 
 # include <assert.h>
 
+#undef __alloca
 #define	__alloca	alloca
 #define	__stat		stat
 #define	NAMLEN(_d)	NLENGTH(_d)
@@ -1066,9 +1067,15 @@ glob_in_dir (const char *pattern, const char *directory, int flags,
 		{
 		  const char *name;
 		  size_t len;
+#ifdef _LARGEFILE64_SOURCE
+		  struct dirent64 *d = ((flags & GLOB_ALTDIRFUNC)
+				      ? (*pglob->gl_readdir) (stream)
+				      : readdir64 ((DIR *) stream));
+#else
 		  struct dirent *d = ((flags & GLOB_ALTDIRFUNC)
 				      ? (*pglob->gl_readdir) (stream)
 				      : readdir ((DIR *) stream));
+#endif
 		  if (d == NULL)
 		    break;
 		  if (! REAL_DIR_ENTRY (d))
@@ -1161,7 +1168,7 @@ glob_in_dir (const char *pattern, const char *directory, int flags,
 
  memory_error:
   {
-    int save = errno;
+    save = errno;
     if (flags & GLOB_ALTDIRFUNC)
       (*pglob->gl_closedir) (stream);
     else
