@@ -53,9 +53,11 @@ extern "C" {
 #if !defined(__size_t) && !defined(_BSD_SIZE_T_DEFINED_)
 # if defined __GNUC__ && __GNUC__ >= 2
 typedef __SIZE_TYPE__ __size_t;
+#  if 0
 #  ifdef _XOPEN_SOURCE
 typedef __SIZE_TYPE__ size_t;
 #  endif
+#  endif
 # else
 /* This is a guess.  */
 typedef unsigned long int __size_t;
@@ -172,6 +174,7 @@ extern void globfree __P ((glob_t *__pglob)) __asm__ ("globfree64");
 # else
 #  define glob glob64
 #  define globfree globfree64
+#  define glob_t glob64_t
 # endif
 #endif
 
