@@ -50,16 +50,6 @@ PyObject * labelCompare (PyObject * self, PyObject * args)
 PyObject * versionCompare (PyObject * self, PyObject * args, PyObject * kwds)
 	/*@*/;
 
-/** \ingroup py_c
- */
-PyObject * rpmMergeHeadersFromFD(PyObject * self, PyObject * args, PyObject * kwds)
-	/*@*/;
-
-/** \ingroup py_c
- */
-int rpmMergeHeaders(PyObject * list, FD_t fd, int matchTag)
-	/*@*/;
-
 /** \ingroup py_c
  */
 PyObject * rpmHeaderFromFile(PyObject * self, PyObject * args, PyObject * kwds)
