@@ -3,21 +3,25 @@
 
 #include "rpmio_internal.h"
 
-/** \ingroup python
+/** \ingroup py_c
  * \file python/rpmbc-py.h
  */
 
-/**
+/** \name _rpm.bc */
+/*@{*/
+
+/** \ingroup py_c
  */
 typedef struct rpmbcObject_s {
     PyObject_HEAD
     PyObject *md_dict;		/*!< to look like PyModuleObject */
-    mp32number n;
 } rpmbcObject;
 
-/**
+/** \ingroup py_c
  */
 /*@unchecked@*/
 extern PyTypeObject rpmbc_Type;
 
+/*@}*/
+
 #endif
