@@ -12,10 +12,13 @@ static int _debug = 1;	/* XXX if < 0 debugging, > 0 unusual error returns */
 #include <sys/ipc.h>
 #endif
 
-#include <rpmlib.h>
+#include <rpmio.h>
+#include <rpmlog.h>
 #include <rpmmacro.h>
 #include <rpmurl.h>	/* XXX urlPath proto */
 
+#define	_RPMTAG_INTERNAL
+#include <rpmtag.h>
 #define	_RPMDB_INTERNAL
 #include <rpmdb.h>
 
@@ -159,10 +162,10 @@ static int cvtdberr(/*@unused@*/ dbiIndex dbi, const char * msg, int error, int
     if (printit && rc) {
 	/*@-moduncon@*/ /* FIX: annotate db3 methods */
 	if (msg)
-	    rpmError(RPMERR_DBERR, _("db%d error(%d) from %s: %s\n"),
+	    rpmlog(RPMLOG_ERR, _("db%d error(%d) from %s: %s\n"),
 		DB_VERSION_MAJOR, rc, msg, db_strerror(error));
 	else
-	    rpmError(RPMERR_DBERR, _("db%d error(%d): %s\n"),
+	    rpmlog(RPMLOG_ERR, _("db%d error(%d): %s\n"),
 		DB_VERSION_MAJOR, rc, db_strerror(error));
 	/*@=moduncon@*/
     }
@@ -171,6 +174,30 @@ static int cvtdberr(/*@unused@*/ dbiIndex dbi, const char * msg, int error, int
 }
 /*@=globuse =mustmod @*/
 
+/**
+ * Return (possibly renamed) tagName. Handles arbitrary tags.
+ * @param rpmdb		rpm database
+ * @param dbi		rpm database index
+ * @return		tag string
+ */
+/*@observer@*/
+static const char * mapTagName(rpmdb rpmdb, dbiIndex dbi)
+	/*@*/
+{
+    tagStore_t dbiTags = rpmdb->db_tags;
+    int dbix = 0;
+
+    if (dbiTags != NULL)
+    while (dbix < rpmdb->db_ndbi) {
+	if (dbi->dbi_rpmtag == dbiTags->tag)
+	    return dbiTags->str;
+	dbiTags++;
+	dbix++;
+    }
+    /* XXX should never reach here */
+    return tagName(dbi->dbi_rpmtag);
+}
+
 static int db_fini(dbiIndex dbi, const char * dbhome,
 		/*@null@*/ const char * dbfile,
 		/*@unused@*/ /*@null@*/ const char * dbsubfile)
@@ -188,7 +215,7 @@ static int db_fini(dbiIndex dbi, const char * dbhome,
     rc = cvtdberr(dbi, "dbenv->close", rc, _debug);
 
     if (dbfile)
-	rpmMessage(RPMMESS_DEBUG, _("closed   db environment %s/%s\n"),
+	rpmlog(RPMLOG_DEBUG, D_("closed   db environment %s/%s\n"),
 			dbhome, dbfile);
 
     if (rpmdb->db_remove_env) {
@@ -207,7 +234,7 @@ static int db_fini(dbiIndex dbi, const char * dbhome,
 	    xx = cvtdberr(dbi, "dbenv->remove", xx, _debug);
 
 	    if (dbfile)
-		rpmMessage(RPMMESS_DEBUG, _("removed  db environment %s/%s\n"),
+		rpmlog(RPMLOG_DEBUG, D_("removed  db environment %s/%s\n"),
 			dbhome, dbfile);
 	}
 
@@ -221,57 +248,6 @@ static int db3_fsync_disable(/*@unused@*/ int fd)
     return 0;
 }
 
-#if 0
-#if HAVE_LIBPTHREAD
-#if HAVE_PTHREAD_H
-#include <pthread.h>
-#endif
-
-/**
- * Check that posix mutexes are shared.
- * @return		0 == shared.
- */
-static int db3_pthread_nptl(void)
-	/*@*/
-{
-    pthread_mutex_t mutex;
-    pthread_mutexattr_t mutexattr, *mutexattrp = NULL;
-    pthread_cond_t cond;
-    pthread_condattr_t condattr, *condattrp = NULL;
-    int ret = 0;
-
-    ret = pthread_mutexattr_init(&mutexattr);
-    if (ret == 0) {
-	ret = pthread_mutexattr_setpshared(&mutexattr, PTHREAD_PROCESS_SHARED);
-	mutexattrp = &mutexattr;
-    }
-
-    if (ret == 0)
-	ret = pthread_mutex_init(&mutex, mutexattrp);
-    if (mutexattrp != NULL)
-	pthread_mutexattr_destroy(mutexattrp);
-    if (ret)
-	return ret;
-    (void) pthread_mutex_destroy(&mutex);
-
-    ret = pthread_condattr_init(&condattr);
-    if (ret == 0) {
-	ret = pthread_condattr_setpshared(&condattr, PTHREAD_PROCESS_SHARED);
-	condattrp = &condattr;
-    }
-
-    if (ret == 0)
-	ret = pthread_cond_init(&cond, condattrp);
-
-    if (condattrp != NULL)
-	(void)pthread_condattr_destroy(condattrp);
-    if (ret == 0)
-	(void) pthread_cond_destroy(&cond);
-    return ret;
-}
-#endif
-#endif
-
 #if (DB_VERSION_MAJOR == 4 && DB_VERSION_MINOR >= 5)
 /**
  * Is process/thread still alive?
@@ -282,7 +258,7 @@ static int db3_pthread_nptl(void)
  * @return		
  */
 static int db3is_alive(/*@unused@*/ DB_ENV *dbenv, pid_t pid, /*@unused@*/ db_threadid_t tid,
-		u_int32_t flags)
+		uint32_t flags)
 	/*@*/
 {
     int is_alive = 1;	/* assume all processes are alive */
@@ -307,12 +283,21 @@ static int db_init(dbiIndex dbi, const char * dbhome,
 		fileSystem @*/
 	/*@modifies dbi, *dbenvp, fileSystem @*/
 {
+    static int oneshot = 0;
     rpmdb rpmdb = dbi->dbi_rpmdb;
     DB_ENV *dbenv = NULL;
     int eflags;
     int rc;
     int xx;
 
+    if (!oneshot) {
+#if (DB_VERSION_MAJOR == 3 && DB_VERSION_MINOR != 0) || (DB_VERSION_MAJOR == 4)
+	xx = db_env_set_func_open((int (*)(const char *, int, ...))Open);
+	xx = cvtdberr(dbi, "db_env_set_func_open", xx, _debug);
+#endif
+	oneshot++;
+    }
+
     if (dbenvp == NULL)
 	return 1;
 
@@ -328,7 +313,7 @@ static int db_init(dbiIndex dbi, const char * dbhome,
     if (eflags & DB_JOINENV) eflags &= DB_JOINENV;
 
     if (dbfile)
-	rpmMessage(RPMMESS_DEBUG, _("opening  db environment %s/%s %s\n"),
+	rpmlog(RPMLOG_DEBUG, D_("opening  db environment %s/%s %s\n"),
 		dbhome, dbfile, prDbiOpenFlags(eflags, 1));
 
     /* XXX Can't do RPC w/o host. */
@@ -356,7 +341,9 @@ static int db_init(dbiIndex dbi, const char * dbhome,
  /* 4.1: dbenv->set_data_dir(???) */
  /* 4.1: dbenv->set_encrypt(???) */
 
-    dbenv->set_errcall(dbenv, rpmdb->db_errcall);
+/*@-castfcnptr@*/
+    dbenv->set_errcall(dbenv, (void *)rpmdb->db_errcall);
+/*@=castfcnptr@*/
     dbenv->set_errfile(dbenv, rpmdb->db_errfile);
     dbenv->set_errpfx(dbenv, rpmdb->db_errpfx);
     /*@=noeffectuncon@*/
@@ -407,6 +394,14 @@ static int db_init(dbiIndex dbi, const char * dbhome,
 #endif
 	xx = dbenv->set_verbose(dbenv, DB_VERB_WAITSFOR,
 		(dbi->dbi_verbose & DB_VERB_WAITSFOR));
+#if defined(DB_VERB_FILEOPS)
+	xx = dbenv->set_verbose(dbenv, DB_VERB_FILEOPS,
+		(dbi->dbi_verbose & DB_VERB_FILEOPS));
+#endif
+#if defined(DB_VERB_FILEOPS_ALL)
+	xx = dbenv->set_verbose(dbenv, DB_VERB_FILEOPS_ALL,
+		(dbi->dbi_verbose & DB_VERB_FILEOPS_ALL));
+#endif
 
 	if (dbi->dbi_mmapsize) {
 	    xx = dbenv->set_mp_mmapsize(dbenv, dbi->dbi_mmapsize);
@@ -417,10 +412,8 @@ static int db_init(dbiIndex dbi, const char * dbhome,
 	    const char * tmpdir;
 
 	    root = (dbi->dbi_root ? dbi->dbi_root : rpmdb->db_root);
-/*@-boundsread@*/
 	    if ((root[0] == '/' && root[1] == '\0') || rpmdb->db_chrootDone)
 		root = NULL;
-/*@=boundsread@*/
 /*@-mods@*/
 	    tmpdir = rpmGenPath(root, dbi->dbi_tmpdir, NULL);
 /*@=mods@*/
@@ -546,9 +539,9 @@ static int db_init(dbiIndex dbi, const char * dbhome,
 #endif
 
 #if (DB_VERSION_MAJOR == 3 && DB_VERSION_MINOR != 0) || (DB_VERSION_MAJOR == 4)
-    rc = dbenv->open(dbenv, dbhome, eflags, dbi->dbi_perms);
+    rc = (dbenv->open)(dbenv, dbhome, eflags, dbi->dbi_perms);
 #else
-    rc = dbenv->open(dbenv, dbhome, NULL, eflags, dbi->dbi_perms);
+    rc = (dbenv->open)(dbenv, dbhome, NULL, eflags, dbi->dbi_perms);
 #endif
     xx = _debug;
 #if defined(DB_VERSION_MISMATCH)
@@ -574,9 +567,7 @@ static int db_init(dbiIndex dbi, const char * dbhome,
     }
 #endif
 
-/*@-boundswrite@*/
     *dbenvp = dbenv;
-/*@=boundswrite@*/
 
     return 0;
 
@@ -616,11 +607,14 @@ static int db3cdup(dbiIndex dbi, DBC * dbcursor, DBC ** dbcp,
 {
     int rc;
 
-/*@-boundswrite@*/
     if (dbcp) *dbcp = NULL;
-/*@=boundswrite@*/
+#if (DB_VERSION_MAJOR == 4 && DB_VERSION_MINOR >= 6)
+    rc = dbcursor->dup(dbcursor, dbcp, flags);
+    rc = cvtdberr(dbi, "dbcursor->dup", rc, _debug);
+#else
     rc = dbcursor->c_dup(dbcursor, dbcp, flags);
     rc = cvtdberr(dbi, "dbcursor->c_dup", rc, _debug);
+#endif
     /*@-nullstate @*/ /* FIX: *dbcp can be NULL */
     return rc;
     /*@=nullstate @*/
@@ -636,8 +630,13 @@ static int db3cclose(dbiIndex dbi, /*@only@*/ /*@null@*/ DBC * dbcursor,
 
     /* XXX db3copen error pathways come through here. */
     if (dbcursor != NULL) {
+#if (DB_VERSION_MAJOR == 4 && DB_VERSION_MINOR >= 6)
+	rc = dbcursor->close(dbcursor);
+	rc = cvtdberr(dbi, "dbcursor->close", rc, _debug);
+#else
 	rc = dbcursor->c_close(dbcursor);
 	rc = cvtdberr(dbi, "dbcursor->c_close", rc, _debug);
+#endif
     }
     return rc;
 }
@@ -666,7 +665,7 @@ static int db3copen(dbiIndex dbi, DB_TXN * txnid,
     rc = cvtdberr(dbi, "db->cursor", rc, _debug);
 
     if (dbcp)
-	/*@-boundswrite -onlytrans@*/ *dbcp = dbcursor; /*@=boundswrite =onlytrans@*/
+	*dbcp = dbcursor;
     else
 	(void) db3cclose(dbi, dbcursor, 0);
 
@@ -686,8 +685,13 @@ static int db3cput(dbiIndex dbi, DBC * dbcursor, DBT * key, DBT * data,
 	rc = db->put(db, dbi->dbi_txnid, key, data, 0);
 	rc = cvtdberr(dbi, "db->put", rc, _debug);
     } else {
+#if (DB_VERSION_MAJOR == 4 && DB_VERSION_MINOR >= 6)
+	rc = dbcursor->put(dbcursor, key, data, DB_KEYLAST);
+	rc = cvtdberr(dbi, "dbcursor->put", rc, _debug);
+#else
 	rc = dbcursor->c_put(dbcursor, key, data, DB_KEYLAST);
 	rc = cvtdberr(dbi, "dbcursor->c_put", rc, _debug);
+#endif
     }
 
     return rc;
@@ -710,14 +714,26 @@ static int db3cdel(dbiIndex dbi, DBC * dbcursor, DBT * key, DBT * data,
 	int _printit;
 
 	/* XXX TODO: insure that cursor is positioned with duplicates */
+#if (DB_VERSION_MAJOR == 4 && DB_VERSION_MINOR >= 6)
+	rc = dbcursor->get(dbcursor, key, data, DB_SET);
+	/* XXX DB_NOTFOUND can be returned */
+	_printit = (rc == DB_NOTFOUND ? 0 : _debug);
+	rc = cvtdberr(dbi, "dbcursor->get", rc, _printit);
+#else
 	rc = dbcursor->c_get(dbcursor, key, data, DB_SET);
 	/* XXX DB_NOTFOUND can be returned */
 	_printit = (rc == DB_NOTFOUND ? 0 : _debug);
 	rc = cvtdberr(dbi, "dbcursor->c_get", rc, _printit);
+#endif
 
 	if (rc == 0) {
+#if (DB_VERSION_MAJOR == 4 && DB_VERSION_MINOR >= 6)
+	    rc = dbcursor->del(dbcursor, flags);
+	    rc = cvtdberr(dbi, "dbcursor->del", rc, _debug);
+#else
 	    rc = dbcursor->c_del(dbcursor, flags);
 	    rc = cvtdberr(dbi, "dbcursor->c_del", rc, _debug);
+#endif
 	}
     }
 
@@ -743,11 +759,19 @@ static int db3cget(dbiIndex dbi, DBC * dbcursor, DBT * key, DBT * data,
 	_printit = (rc == DB_NOTFOUND ? 0 : _debug);
 	rc = cvtdberr(dbi, "db->get", rc, _printit);
     } else {
+#if (DB_VERSION_MAJOR == 4 && DB_VERSION_MINOR >= 6)
+	/* XXX db3 does DB_FIRST on uninitialized cursor */
+	rc = dbcursor->get(dbcursor, key, data, flags);
+	/* XXX DB_NOTFOUND can be returned */
+	_printit = (rc == DB_NOTFOUND ? 0 : _debug);
+	rc = cvtdberr(dbi, "dbcursor->get", rc, _printit);
+#else
 	/* XXX db3 does DB_FIRST on uninitialized cursor */
 	rc = dbcursor->c_get(dbcursor, key, data, flags);
 	/* XXX DB_NOTFOUND can be returned */
 	_printit = (rc == DB_NOTFOUND ? 0 : _debug);
 	rc = cvtdberr(dbi, "dbcursor->c_get", rc, _printit);
+#endif
     }
 
     return rc;
@@ -767,11 +791,19 @@ static int db3cpget(dbiIndex dbi, DBC * dbcursor, DBT * key, DBT * pkey,
     assert(db != NULL);
     assert(dbcursor != NULL);
 
+#if (DB_VERSION_MAJOR == 4 && DB_VERSION_MINOR >= 6)
+    /* XXX db3 does DB_FIRST on uninitialized cursor */
+    rc = dbcursor->pget(dbcursor, key, pkey, data, flags);
+    /* XXX DB_NOTFOUND can be returned */
+    _printit = (rc == DB_NOTFOUND ? 0 : _debug);
+    rc = cvtdberr(dbi, "dbcursor->pget", rc, _printit);
+#else
     /* XXX db3 does DB_FIRST on uninitialized cursor */
     rc = dbcursor->c_pget(dbcursor, key, pkey, data, flags);
     /* XXX DB_NOTFOUND can be returned */
     _printit = (rc == DB_NOTFOUND ? 0 : _debug);
     rc = cvtdberr(dbi, "dbcursor->c_pget", rc, _printit);
+#endif
 
     return rc;
 }
@@ -787,12 +819,15 @@ static int db3ccount(dbiIndex dbi, DBC * dbcursor,
     int rc = 0;
 
     flags = 0;
+#if (DB_VERSION_MAJOR == 4 && DB_VERSION_MINOR >= 6)
+    rc = dbcursor->count(dbcursor, &count, flags);
+    rc = cvtdberr(dbi, "dbcursor->count", rc, _debug);
+#else
     rc = dbcursor->c_count(dbcursor, &count, flags);
     rc = cvtdberr(dbi, "dbcursor->c_count", rc, _debug);
+#endif
     if (rc) return rc;
-/*@-boundswrite@*/
     if (countp) *countp = count;
-/*@=boundswrite@*/
 
     return rc;
 }
@@ -908,6 +943,7 @@ static int db3close(/*@only@*/ dbiIndex dbi, /*@unused@*/ unsigned int flags)
     const char * dbfile;
     const char * dbsubfile;
     DB * db = dbi->dbi_db;
+    const char * dbiBN = mapTagName(rpmdb, dbi);
     int _printit;
     int rc = 0, xx;
 
@@ -917,10 +953,8 @@ static int db3close(/*@only@*/ dbiIndex dbi, /*@unused@*/ unsigned int flags)
      * Get the prefix/root component and directory path.
      */
     root = (dbi->dbi_root ? dbi->dbi_root : rpmdb->db_root);
-/*@-boundsread@*/
     if ((root[0] == '/' && root[1] == '\0') || rpmdb->db_chrootDone)
 	root = NULL;
-/*@=boundsread@*/
     home = (dbi->dbi_home ? dbi->dbi_home : rpmdb->db_home);
 
     /*
@@ -937,9 +971,9 @@ static int db3close(/*@only@*/ dbiIndex dbi, /*@unused@*/ unsigned int flags)
     } else {
 #ifdef	HACK	/* XXX necessary to support dbsubfile */
 	dbfile = (dbi->dbi_file ? dbi->dbi_file : db3basename);
-	dbsubfile = (dbi->dbi_subfile ? dbi->dbi_subfile : tagName(dbi->dbi_rpmtag));
+	dbsubfile = (dbi->dbi_subfile ? dbi->dbi_subfile : dbiBN);
 #else
-	dbfile = (dbi->dbi_file ? dbi->dbi_file : tagName(dbi->dbi_rpmtag));
+	dbfile = (dbi->dbi_file ? dbi->dbi_file : dbiBN);
 	dbsubfile = NULL;
 #endif
     }
@@ -951,8 +985,8 @@ static int db3close(/*@only@*/ dbiIndex dbi, /*@unused@*/ unsigned int flags)
 	rc = cvtdberr(dbi, "db->close", rc, _printit);
 	db = dbi->dbi_db = NULL;
 
-	rpmMessage(RPMMESS_DEBUG, _("closed   db index       %s/%s\n"),
-		dbhome, (dbfile ? dbfile : tagName(dbi->dbi_rpmtag)));
+	rpmlog(RPMLOG_DEBUG, D_("closed   db index       %s/%s\n"),
+		dbhome, (dbfile ? dbfile : dbiBN));
 
     }
 
@@ -977,7 +1011,9 @@ static int db3close(/*@only@*/ dbiIndex dbi, /*@unused@*/ unsigned int flags)
 	if (rc || dbenv == NULL) goto exit;
 
 	/*@-noeffectuncon@*/ /* FIX: annotate db3 methods */
-	dbenv->set_errcall(dbenv, rpmdb->db_errcall);
+/*@-castfcnptr@*/
+	dbenv->set_errcall(dbenv, (void *)rpmdb->db_errcall);
+/*@=castfcnptr@*/
 	dbenv->set_errfile(dbenv, rpmdb->db_errfile);
 	dbenv->set_errpfx(dbenv, rpmdb->db_errpfx);
  /*	dbenv->set_paniccall(???) */
@@ -1004,7 +1040,7 @@ static int db3close(/*@only@*/ dbiIndex dbi, /*@unused@*/ unsigned int flags)
 	}
 	    
 	eflags = DB_CREATE | DB_INIT_MPOOL | DB_PRIVATE | DB_USE_ENVIRON;
-	rc = dbenv->open(dbenv, dbhome, eflags, 0);
+	rc = (dbenv->open)(dbenv, dbhome, eflags, 0);
 	rc = cvtdberr(dbi, "dbenv->open", rc, _debug);
 	if (rc) goto exit;
 
@@ -1021,9 +1057,9 @@ static int db3close(/*@only@*/ dbiIndex dbi, /*@unused@*/ unsigned int flags)
 		rc = db->verify(db, dbf, NULL, NULL, flags);
 		rc = cvtdberr(dbi, "db->verify", rc, _debug);
 
-		rpmMessage(RPMMESS_DEBUG, _("verified db index       %s/%s\n"),
+		rpmlog(RPMLOG_DEBUG, D_("verified db index       %s/%s\n"),
 			(dbhome ? dbhome : ""),
-			(dbfile ? dbfile : tagName(dbi->dbi_rpmtag)));
+			(dbfile ? dbfile : dbiBN));
 
 	        /*
 		 * The DB handle may not be accessed again after
@@ -1062,6 +1098,7 @@ static int db3open(rpmdb rpmdb, rpmTag rpmtag, dbiIndex * dbip)
     const char * dbhome;
     const char * dbfile;
     const char * dbsubfile;
+    const char * dbiBN;
     dbiIndex dbi = NULL;
     int rc = 0;
     int xx;
@@ -1072,13 +1109,11 @@ static int db3open(rpmdb rpmdb, rpmTag rpmtag, dbiIndex * dbip)
     DB_TXN * txnid = NULL;
 #endif
     DBTYPE dbi_type = DB_UNKNOWN;
-    u_int32_t oflags;
+    uint32_t oflags;
     int _printit;
 
-/*@-boundswrite@*/
     if (dbip)
 	*dbip = NULL;
-/*@=boundswrite@*/
 
     /*
      * Parse db configuration parameters.
@@ -1090,15 +1125,14 @@ static int db3open(rpmdb rpmdb, rpmTag rpmtag, dbiIndex * dbip)
 	/*@=nullstate@*/
     /*@=mods@*/
     dbi->dbi_api = DB_VERSION_MAJOR;
+    dbiBN = mapTagName(rpmdb, dbi);
 
     /*
      * Get the prefix/root component and directory path.
      */
     root = (dbi->dbi_root ? dbi->dbi_root : rpmdb->db_root);
-/*@-boundsread@*/
     if ((root[0] == '/' && root[1] == '\0') || rpmdb->db_chrootDone)
 	root = NULL;
-/*@=boundsread@*/
     home = (dbi->dbi_home ? dbi->dbi_home : rpmdb->db_home);
 
     /*
@@ -1115,9 +1149,9 @@ static int db3open(rpmdb rpmdb, rpmTag rpmtag, dbiIndex * dbip)
     } else {
 #ifdef	HACK	/* XXX necessary to support dbsubfile */
 	dbfile = (dbi->dbi_file ? dbi->dbi_file : db3basename);
-	dbsubfile = (dbi->dbi_subfile ? dbi->dbi_subfile : tagName(dbi->dbi_rpmtag));
+	dbsubfile = (dbi->dbi_subfile ? dbi->dbi_subfile : dbiBN);
 #else
-	dbfile = (dbi->dbi_file ? dbi->dbi_file : tagName(dbi->dbi_rpmtag));
+	dbfile = (dbi->dbi_file ? dbi->dbi_file : dbiBN);
 	dbsubfile = NULL;
 #endif
     }
@@ -1158,19 +1192,6 @@ static int db3open(rpmdb rpmdb, rpmTag rpmtag, dbiIndex * dbip)
      */
     if (dbi->dbi_use_dbenv) {
 
-#if 0
-#if HAVE_LIBPTHREAD
-	if (rpmdb->db_dbenv == NULL) {
-	    /* Set DB_PRIVATE if posix mutexes are not shared. */
-	    xx = db3_pthread_nptl();
-	    if (xx) {
-		dbi->dbi_eflags |= DB_PRIVATE;
-		rpmMessage(RPMMESS_DEBUG, _("unshared posix mutexes found(%d), adding DB_PRIVATE, using fcntl lock\n"), xx);
-	    }
-	}
-#endif
-#endif
-
 	if (access(dbhome, W_OK) == -1) {
 
 	    /* dbhome is unwritable, don't attempt DB_CREATE on DB->open ... */
@@ -1204,7 +1225,17 @@ static int db3open(rpmdb rpmdb, rpmTag rpmtag, dbiIndex * dbip)
 	    const char * dbf = rpmGetPath(dbhome, "/__db.001", NULL);
 	    /*@=mods@*/
 
+#if defined(RPM_VENDOR_OPENPKG) /* bdb-allow-zero-sized-files */
+	    /* Make sure RPM passes DB_CREATE to Berkeley-DB also
+	       if file exists, but is (still) zero-sized. */
+	    struct stat sb;
+	    long size = -1;
+	    if (stat(dbf, &sb) == 0)
+		size = (long)sb.st_size;
+	    if (access(dbf, F_OK) == -1 || size == 0) {
+#else
 	    if (access(dbf, F_OK) == -1) {
+#endif
 		/* ... non-existent (or unwritable) DBENV, will create ... */
 		dbi->dbi_oeflags |= DB_CREATE;
 		dbi->dbi_eflags &= ~DB_JOINENV;
@@ -1227,7 +1258,7 @@ static int db3open(rpmdb rpmdb, rpmTag rpmtag, dbiIndex * dbip)
      */
     if ((oflags & DB_CREATE) && (oflags & DB_RDONLY)) {
 	/* dbhome is writable, and DB->open flags may conflict. */
-	const char * dbfn = (dbfile ? dbfile : tagName(dbi->dbi_rpmtag));
+	const char * dbfn = (dbfile ? dbfile : dbiBN);
 	/*@-mods@*/
 	const char * dbf = rpmGetPath(dbhome, "/", dbfn, NULL);
 	/*@=mods@*/
@@ -1261,7 +1292,6 @@ static int db3open(rpmdb rpmdb, rpmTag rpmtag, dbiIndex * dbip)
     if (oflags & DB_RDONLY)
 	dbi->dbi_verify_on_close = 0;
 
-/*@-branchstate@*/
     if (dbi->dbi_use_dbenv) {
 	/*@-mods@*/
 	if (rpmdb->db_dbenv == NULL) {
@@ -1271,7 +1301,7 @@ static int db3open(rpmdb rpmdb, rpmTag rpmtag, dbiIndex * dbip)
 		break;
 
 	    case DB_RUNRECOVERY:
-		rpmError(RPMERR_DBERR, _("Runnning db->verify ...\n"));
+		rpmlog(RPMLOG_ERR, _("Runnning db->verify ...\n"));
 		rpmdb = rpmdbLink(rpmdb, "DB_RUNRECOVERY");
 		rpmdb->db_remove_env = 1;
 		rpmdb->db_verifying = 1;
@@ -1329,14 +1359,18 @@ assert(dbenv);
 	} else {
 assert(rpmdb && rpmdb->db_dbenv);
 	    dbenv = rpmdb->db_dbenv;
+#define	PLD_CHROOT
+#ifdef	PLD_CHROOT
+	    if (rpmdb->db_chrootDone)
+		xx = dbenv->set_data_dir(dbenv, dbhome);
+#endif
 	    rpmdb->db_opens++;
 	}
 	/*@=mods@*/
     }
-/*@=branchstate@*/
 
-    rpmMessage(RPMMESS_DEBUG, _("opening  db index       %s/%s %s mode=0x%x\n"),
-		dbhome, (dbfile ? dbfile : tagName(dbi->dbi_rpmtag)),
+    rpmlog(RPMLOG_DEBUG, D_("opening  db index       %s/%s %s mode=0x%x\n"),
+		dbhome, (dbfile ? dbfile : dbiBN),
 		prDbiOpenFlags(oflags, 0), dbi->dbi_mode);
 
     if (rc == 0) {
@@ -1491,24 +1525,28 @@ assert(rpmdb && rpmdb->db_dbenv);
 		if (dbfile)	nb += 1 + strlen(dbfile);
 		dbfullpath = t = alloca(nb + 1);
 
-/*@-boundswrite@*/
 		t = stpcpy(t, dbhome);
 		if (dbfile)
 		    t = stpcpy( stpcpy( t, "/"), dbfile);
-/*@=boundswrite@*/
 #ifdef	HACK	/* XXX necessary to support dbsubfile */
 		dbpath = (!dbi->dbi_use_dbenv && !dbi->dbi_temporary)
 			? dbfullpath : dbfile;
+#else
+#ifdef	PLD_CHROOT
+		/* XXX Make dbpath relative. */
+		dbpath = (!dbi->dbi_use_dbenv)
+			? dbfullpath : dbfile;
 #else
 		dbpath = (!dbi->dbi_temporary)
 			? dbfullpath : dbfile;
-#endif
+#endif	/* PLD_CHROOT */
+#endif	/* HACK */
 
 #if (DB_VERSION_MAJOR == 4 && DB_VERSION_MINOR >= 1)
-		rc = db->open(db, txnid, dbpath, dbsubfile,
+		rc = (db->open)(db, txnid, dbpath, dbsubfile,
 		    dbi_type, oflags, dbi->dbi_perms);
 #else
-		rc = db->open(db, dbpath, dbsubfile,
+		rc = (db->open)(db, dbpath, dbsubfile,
 		    dbi_type, oflags, dbi->dbi_perms);
 #endif
 
@@ -1558,9 +1596,7 @@ assert(rpmdb && rpmdb->db_dbenv);
 		    rc = 1;
 		} else {
 		    struct flock l;
-/*@-boundswrite@*/
 		    memset(&l, 0, sizeof(l));
-/*@=boundswrite@*/
 		    l.l_whence = 0;
 		    l.l_start = 0;
 		    l.l_len = 0;
@@ -1575,14 +1611,14 @@ assert(rpmdb && rpmdb->db_dbenv);
 				(dbi->dbi_eflags & DB_INIT_CDB) &&
 				!(dbi->dbi_eflags & DB_PRIVATE))
 			    ? 0 : 1);
-			rpmError( (rc ? RPMERR_FLOCK : RPMWARN_FLOCK),
+			rpmlog( (rc ? RPMLOG_ERR : RPMLOG_WARNING),
 				_("cannot get %s lock on %s/%s\n"),
 				((dbi->dbi_mode & (O_RDWR|O_WRONLY))
 					? _("exclusive") : _("shared")),
 				dbhome, (dbfile ? dbfile : ""));
 		    } else if (dbfile) {
-			rpmMessage(RPMMESS_DEBUG,
-				_("locked   db index       %s/%s\n"),
+			rpmlog(RPMLOG_DEBUG,
+				D_("locked   db index       %s/%s\n"),
 				dbhome, dbfile);
 		    }
 		}
@@ -1594,9 +1630,7 @@ assert(rpmdb && rpmdb->db_dbenv);
 
     if (rc == 0 && dbi->dbi_db != NULL && dbip != NULL) {
 	dbi->dbi_vec = &db3vec;
-/*@-boundswrite@*/
 	*dbip = dbi;
-/*@=boundswrite@*/
     } else {
 	dbi->dbi_verify_on_close = 0;
 	(void) db3close(dbi, 0);
