@@ -4,8 +4,8 @@
 
 #include "system.h"
 
-#include <rpmdb.h>
 #include <rpmmacro.h>	/* XXX for rpmCleanPath */
+#include <rpmdb.h>
 
 #include "fprint.h"
 #include "debug.h"
@@ -41,9 +41,7 @@ static /*@null@*/ const struct fprintCacheEntry_s * cacheContainsDirectory(
 
     if (htGetEntry(cache->ht, dirName, &data, NULL, NULL))
 	return NULL;
-/*@-boundsread@*/
     return data[0];
-/*@=boundsread@*/
 }
 
 /**
@@ -54,7 +52,6 @@ static /*@null@*/ const struct fprintCacheEntry_s * cacheContainsDirectory(
  * @param scareMem
  * @return pointer to the finger print associated with a file path.
  */
-/*@-bounds@*/ /* LCL: segfault */
 static fingerPrint doLookup(fingerPrintCache cache,
 		const char * dirName, const char * baseName, int scareMem)
 	/*@modifies cache @*/
@@ -75,11 +72,9 @@ static fingerPrint doLookup(fingerPrintCache cache,
     cdnl = strlen(cleanDirName);
 
     if (*cleanDirName == '/') {
-	/*@-branchstate@*/
 	if (!scareMem)
 	    cleanDirName =
 		rpmCleanPath(strcpy(alloca(cdnl+1), dirName));
-	/*@=branchstate@*/
     } else {
 	scareMem = 0;	/* XXX causes memory leak */
 
@@ -92,7 +87,6 @@ static fingerPrint doLookup(fingerPrintCache cache,
 	/* if the current directory doesn't exist, we might fail. 
 	   oh well. likewise if it's too long.  */
 	dir[0] = '\0';
-	/*@-branchstate@*/
 	if (realpath(".", dir) != NULL) {
 	    end = dir + strlen(dir);
 	    if (end[-1] != '/')	*end++ = '/';
@@ -105,7 +99,6 @@ static fingerPrint doLookup(fingerPrintCache cache,
 	    cleanDirName = dir;
 	    cdnl = end - dir;
 	}
-	/*@=branchstate@*/
     }
     fp.entry = NULL;
     fp.subDir = NULL;
@@ -183,7 +176,6 @@ static fingerPrint doLookup(fingerPrintCache cache,
     /*@-nullret@*/ return fp; /*@=nullret@*/	/* LCL: can't happen. */
     /*@=compdef@*/
 }
-/*@=bounds@*/
 
 fingerPrint fpLookup(fingerPrintCache cache, const char * dirName, 
 			const char * baseName, int scareMem)
@@ -195,11 +187,9 @@ uint32_t fpHashFunction(uint32_t h, const void * data, /*@unused@*/ size_t size)
 {
     const fingerPrint * fp = data;
     const char * chptr = fp->baseName;
-    unsigned char ch = 0;
+    unsigned char ch = '\0';
 
-/*@-boundsread@*/
     while (*chptr != '\0') ch ^= *chptr++;
-/*@=boundsread@*/
 
     h |= ((unsigned)ch) << 24;
     h |= (((((unsigned)fp->entry->dev) >> 8) ^ fp->entry->dev) & 0xFF) << 16;
@@ -208,7 +198,6 @@ uint32_t fpHashFunction(uint32_t h, const void * data, /*@unused@*/ size_t size)
     return h;
 }
 
-/*@-boundsread@*/
 int fpEqual(const void * key1, const void * key2)
 {
     const fingerPrint *k1 = key1;
@@ -226,16 +215,14 @@ int fpEqual(const void * key1, const void * key2)
     return 1;
 
 }
-/*@=boundsread@*/
 
-/*@-bounds@*/
 void fpLookupList(fingerPrintCache cache, const char ** dirNames, 
-		  const char ** baseNames, const uint_32 * dirIndexes, 
-		  int fileCount, fingerPrint * fpList)
+		  const char ** baseNames, const uint32_t * dirIndexes, 
+		  uint32_t fileCount, fingerPrint * fpList)
 {
-    int i;
+    unsigned i;
 
-    for (i = 0; i < fileCount; i++) {
+    for (i = 0; i < (unsigned) fileCount; i++) {
 	/* If this is in the same directory as the last file, don't bother
 	   redoing all of this work */
 	if (i > 0 && dirIndexes[i - 1] == dirIndexes[i]) {
@@ -248,7 +235,6 @@ void fpLookupList(fingerPrintCache cache, const char ** dirNames,
 	}
     }
 }
-/*@=bounds@*/
 
 #ifdef	UNUSED
 /**
@@ -262,21 +248,33 @@ static
 void fpLookupHeader(fingerPrintCache cache, Header h, fingerPrint * fpList)
 	/*@modifies h, cache, *fpList @*/;
 {
-    HGE_t hge = (HGE_t)headerGetEntryMinMemory;
-    HFD_t hfd = headerFreeData;
-    const char ** baseNames, ** dirNames;
-    rpmTagType bnt, dnt;
-    int_32 * dirIndexes;
-    int fileCount;
+    rpmTagData he_p = { .ptr = NULL };
+    HE_s he_s = { .tag = 0, .t = 0, .p = &he_p, .c = 0, .freeData = 0 };
+    HE_t he = &he_s;
+    const char ** baseNames;
+    const char ** dirNames;
+    uint32_t * dirIndexes;
+    rpmTagCount fileCount;
     int xx;
 
-    if (!hge(h, RPMTAG_BASENAMES, &bnt, (void **) &baseNames, &fileCount))
+    he->tag = RPMTAG_BASENAMES;
+    xx = headerGet(h, he->tag, &he->t, he->p, &he->c);
+    baseNames = he_p.argv;
+    fileCount = he->c;
+    if (!xx)
 	return;
 
-    xx = hge(h, RPMTAG_DIRNAMES, &dnt, (void **) &dirNames, NULL);
-    xx = hge(h, RPMTAG_DIRINDEXES, NULL, (void **) &dirIndexes, NULL);
+    he->tag = RPMTAG_DIRNAMES;
+    xx = headerGet(h, he, 0);
+    dirNames = he_p.argv;
+    he->tag = RPMTAG_DIRINDEXES;
+    xx = headerGet(h, he, 0);
+    dirIndexes = he_p.ui32p;
+
     fpLookupList(cache, dirNames, baseNames, dirIndexes, fileCount, fpList);
-    dirNames = hfd(dirNames, dnt);
-    baseNames = hfd(baseNames, bnt);
+
+    dirIndexes = _free(dirIndexes);
+    dirNames = _free(dirNames);
+    baseNames = _free(baseNames);
 }
 #endif
