@@ -4,7 +4,7 @@
 
 #include "system.h"
 
-#if HAVE_GELF_H
+#if defined(HAVE_GELF_H)
 #if LIBELF_H_LFS_CONFLICT
 /* some gelf.h/libelf.h implementations (Solaris) are
  * incompatible with the Large File API
@@ -26,7 +26,7 @@
 #endif
 
 #include "rpmio_internal.h"
-#include <rpmlib.h>
+#include <rpmlib.h>	/* XXX rpmMkdirPath */
 #include <rpmmacro.h>
 #include "misc.h"
 #include "legacy.h"
@@ -41,6 +41,7 @@
  * @retval fsizep	file size
  * @return		-1 on error, otherwise, an open file descriptor
  */ 
+/*@-compdef -moduncon -noeffectuncon @*/
 static int open_dso(const char * path, /*@null@*/ pid_t * pidp, /*@null@*/ size_t *fsizep)
 	/*@globals rpmGlobalMacroContext, h_errno, fileSystem, internalState @*/
 	/*@modifies *pidp, *fsizep, rpmGlobalMacroContext,
@@ -48,15 +49,14 @@ static int open_dso(const char * path, /*@null@*/ pid_t * pidp, /*@null@*/ size_
 {
 /*@only@*/
     static const char * cmd = NULL;
-    static int initted = 0;
+    static int oneshot = 0;
     int fdno;
 
-    if (!initted) {
+    if (!oneshot) {
 	cmd = rpmExpand("%{?__prelink_undo_cmd}", NULL);
-	initted++;
+	oneshot++;
     }
 
-/*@-boundswrite@*/
     if (pidp) *pidp = 0;
 
     if (fsizep) {
@@ -65,18 +65,15 @@ static int open_dso(const char * path, /*@null@*/ pid_t * pidp, /*@null@*/ size_
 	    return -1;
 	*fsizep = st->st_size;
     }
-/*@=boundswrite@*/
 
     fdno = open(path, O_RDONLY);
     if (fdno < 0)
 	return fdno;
 
-/*@-boundsread@*/
     if (!(cmd && *cmd))
 	return fdno;
-/*@=boundsread@*/
 
-#if HAVE_GELF_H && HAVE_LIBELF
+#if defined(HAVE_GELF_H) && defined(HAVE_LIBELF)
  {  Elf *elf = NULL;
     Elf_Scn *scn = NULL;
     Elf_Data *data = NULL;
@@ -96,17 +93,18 @@ static int open_dso(const char * path, /*@null@*/ pid_t * pidp, /*@null@*/ size_
 /*@=evalorder@*/
 
     bingo = 0;
-    /*@-branchstate -uniondef @*/
     while (!bingo && (scn = elf_nextscn(elf, scn)) != NULL) {
 	(void) gelf_getshdr(scn, &shdr);
 	if (shdr.sh_type != SHT_DYNAMIC)
 	    continue;
 	while (!bingo && (data = elf_getdata (scn, data)) != NULL) {
-	    int maxndx = data->d_size / shdr.sh_entsize;
-	    int ndx;
+	    unsigned maxndx = (unsigned) (data->d_size / shdr.sh_entsize);
+	    unsigned ndx;
 
             for (ndx = 0; ndx < maxndx; ++ndx) {
+/*@-uniondef@*/
 		(void) gelf_getdyn (data, ndx, &dyn);
+/*@=uniondef@*/
 		if (!(dyn.d_tag == DT_GNU_PRELINKED || dyn.d_tag == DT_GNU_LIBLIST))
 		    /*@innercontinue@*/ continue;
 		bingo = 1;
@@ -114,9 +112,7 @@ static int open_dso(const char * path, /*@null@*/ pid_t * pidp, /*@null@*/ size_
 	    }
 	}
     }
-    /*@=branchstate =uniondef @*/
 
-/*@-boundswrite@*/
     if (pidp != NULL && bingo) {
 	int pipes[2];
 	pid_t pid;
@@ -143,7 +139,6 @@ static int open_dso(const char * path, /*@null@*/ pid_t * pidp, /*@null@*/ size_
 	fdno = pipes[0];
 	xx = close(pipes[1]);
     }
-/*@=boundswrite@*/
 
 exit:
     if (elf) (void) elf_end(elf);
@@ -152,6 +147,7 @@ exit:
 
     return fdno;
 }
+/*@=compdef =moduncon =noeffectuncon @*/
 
 int dodigest(int digestalgo, const char * fn, unsigned char * digest, int asAscii, size_t *fsizep)
 {
@@ -166,7 +162,9 @@ int dodigest(int digestalgo, const char * fn, unsigned char * digest, int asAsci
     int use_mmap;
     int rc = 0;
     int fdno;
+#if defined(HAVE_MMAP)
     int xx;
+#endif
 
 /*@-globs -internalglobs -mods @*/
     fdno = open_dso(path, &pid, &fsize);
@@ -182,7 +180,7 @@ int dodigest(int digestalgo, const char * fn, unsigned char * digest, int asAsci
     switch(ut) {
     case URL_IS_PATH:
     case URL_IS_UNKNOWN:
-#if HAVE_MMAP
+#if defined(HAVE_MMAP)
       if (use_mmap) {
 	DIGEST_CTX ctx;
 	void * mapped = NULL;
@@ -195,7 +193,7 @@ int dodigest(int digestalgo, const char * fn, unsigned char * digest, int asAsci
 		break;
 	    }
 
-#ifdef	MADV_SEQUENTIAL
+#if defined(HAVE_MADVISE) && defined(MADV_SEQUENTIAL)
 	    xx = madvise(mapped, fsize, MADV_SEQUENTIAL);
 #endif
 	}
@@ -203,7 +201,7 @@ int dodigest(int digestalgo, const char * fn, unsigned char * digest, int asAsci
 	ctx = rpmDigestInit(digestalgo, RPMDIGEST_NONE);
 	if (fsize)
 	    xx = rpmDigestUpdate(ctx, mapped, fsize);
-	xx = rpmDigestFinal(ctx, (void **)&dsum, &dlen, asAscii);
+	xx = rpmDigestFinal(ctx, &dsum, &dlen, asAscii);
 	if (fsize)
 	    xx = munmap(mapped, fsize);
 	xx = close(fdno);
@@ -217,7 +215,7 @@ int dodigest(int digestalgo, const char * fn, unsigned char * digest, int asAsci
     case URL_IS_DASH:
     default:
 	/* Either use the pipe to prelink -y or open the URL. */
-	fd = (pid != 0) ? fdDup(fdno) : Fopen(fn, "r");
+	fd = (pid != 0) ? fdDup(fdno) : Fopen(fn, "r.fdio");
 	(void) close(fdno);
 	if (fd == NULL || Ferror(fd)) {
 	    rc = 1;
@@ -228,9 +226,9 @@ int dodigest(int digestalgo, const char * fn, unsigned char * digest, int asAsci
 	
 	fdInitDigest(fd, digestalgo, 0);
 	fsize = 0;
-	while ((rc = Fread(buf, sizeof(buf[0]), sizeof(buf), fd)) > 0)
+	while ((rc = (int) Fread(buf, sizeof(buf[0]), sizeof(buf), fd)) > 0)
 	    fsize += rc;
-	fdFiniDigest(fd, digestalgo, (void **)&dsum, &dlen, asAscii);
+	fdFiniDigest(fd, digestalgo, &dsum, &dlen, asAscii);
 	if (Ferror(fd))
 	    rc = 1;
 
@@ -247,12 +245,10 @@ int dodigest(int digestalgo, const char * fn, unsigned char * digest, int asAsci
     }
 
 exit:
-/*@-boundswrite@*/
     if (fsizep)
 	*fsizep = fsize;
     if (!rc)
 	memcpy(digest, dsum, dlen);
-/*@=boundswrite@*/
     dsum = _free(dsum);
 
     return rc;
@@ -262,67 +258,3 @@ int domd5(const char * fn, unsigned char * digest, int asAscii, size_t *fsizep)
 {
     return dodigest(PGPHASHALGO_MD5, fn, digest, asAscii, fsizep);
 }
-
-void rpmfiBuildFNames(Header h, rpmTag tagN,
-	/*@out@*/ const char *** fnp, /*@out@*/ int * fcp)
-{
-    HGE_t hge = (HGE_t)headerGetEntryMinMemory;
-    HFD_t hfd = headerFreeData;
-    const char ** baseNames;
-    const char ** dirNames;
-    int * dirIndexes;
-    int count;
-    const char ** fileNames;
-    int size;
-    rpmTag dirNameTag = 0;
-    rpmTag dirIndexesTag = 0;
-    rpmTagType bnt, dnt;
-    char * t;
-    int i, xx;
-
-    if (tagN == RPMTAG_BASENAMES) {
-	dirNameTag = RPMTAG_DIRNAMES;
-	dirIndexesTag = RPMTAG_DIRINDEXES;
-    } else if (tagN == RPMTAG_ORIGBASENAMES) {
-	dirNameTag = RPMTAG_ORIGDIRNAMES;
-	dirIndexesTag = RPMTAG_ORIGDIRINDEXES;
-    }
-
-    if (!hge(h, tagN, &bnt, (void **) &baseNames, &count)) {
-	if (fnp) *fnp = NULL;
-	if (fcp) *fcp = 0;
-	return;		/* no file list */
-    }
-
-    xx = hge(h, dirNameTag, &dnt, (void **) &dirNames, NULL);
-    xx = hge(h, dirIndexesTag, NULL, (void **) &dirIndexes, &count);
-
-    size = sizeof(*fileNames) * count;
-    for (i = 0; i < count; i++) {
-	const char * dn = NULL;
-	(void) urlPath(dirNames[dirIndexes[i]], &dn);
-	size += strlen(baseNames[i]) + strlen(dn) + 1;
-    }
-
-    fileNames = xmalloc(size);
-    t = ((char *) fileNames) + (sizeof(*fileNames) * count);
-    /*@-branchstate@*/
-    for (i = 0; i < count; i++) {
-	const char * dn = NULL;
-	fileNames[i] = t;
-	(void) urlPath(dirNames[dirIndexes[i]], &dn);
-	t = stpcpy( stpcpy(t, dn), baseNames[i]);
-	*t++ = '\0';
-    }
-    /*@=branchstate@*/
-    baseNames = hfd(baseNames, bnt);
-    dirNames = hfd(dirNames, dnt);
-
-    /*@-branchstate@*/
-    if (fnp)
-	*fnp = fileNames;
-    else
-	fileNames = _free(fileNames);
-    /*@=branchstate@*/
-    if (fcp) *fcp = count;
-}
