@@ -10,26 +10,17 @@
 #include <assert.h>
 #include <rpmlib.h>
 #include <mire.h>
+#if defined(WITH_DB)
 #include "db.h"
+#else
+#include "db_emu.h"
+#endif
 
 /*@-exportlocal@*/
 /*@unchecked@*/
 extern int _rpmdb_debug;
 /*@=exportlocal@*/
 
-/*@unchecked@*/
-extern int _rsegfault;
-/*@unchecked@*/
-extern int _wsegfault;
-
-#if defined(__LCLINT__)
-#define	RSEGFAULT
-#define	WSEGFAULT
-#else
-#define	RSEGFAULT	{ if (_rsegfault > 0) assert(--_rsegfault); }
-#define	WSEGFAULT	{ if (_wsegfault > 0) assert(--_wsegfault); }
-#endif
-
 #ifdef	NOTYET
 /** \ingroup rpmdb
  * Database of headers and tag value indices.
@@ -39,7 +30,7 @@ typedef /*@abstract@*/ /*@refcounted@*/ struct rpmdb_s * rpmdb;
 /** \ingroup rpmdb
  * Database iterator.
  */
-typedef /*@abstract@*/ struct _rpmdbMatchIterator * rpmdbMatchIterator;
+typedef /*@abstract@*/ struct rpmdbMatchIterator_s * rpmdbMatchIterator;
 #endif
 
 /**
@@ -56,7 +47,7 @@ typedef /*@abstract@*/ struct _dbiIndexSet * dbiIndexSet;
 typedef /*@abstract@*/ struct _dbiIndex * dbiIndex;
 
 #if defined(_RPMDB_INTERNAL)
-#include "rpmsw.h"
+#include <rpmsw.h>
 #if !defined(SWIG)	/* XXX inline dbiFoo() need */
 /** \ingroup dbi
  * A single item from an index database (i.e. the "data returned").
@@ -64,9 +55,9 @@ typedef /*@abstract@*/ struct _dbiIndex * dbiIndex;
  * and was identical to the "data saved" structure below.
  */
 struct _dbiIndexItem {
-    unsigned int hdrNum;		/*!< header instance in db */
-    unsigned int tagNum;		/*!< tag index in header */
-    unsigned int fpNum;			/*!< finger print index */
+    uint32_t hdrNum;			/*!< header instance in db */
+    uint32_t tagNum;			/*!< tag index in header */
+    uint32_t fpNum;			/*!< finger print index */
 };
 
 /** \ingroup dbi
@@ -304,7 +295,8 @@ struct _dbiIndex {
     int	dbi_no_fsync;		/*!< no-op fsync for db */
     int	dbi_no_dbsync;		/*!< don't call dbiSync */
     int	dbi_lockdbfd;		/*!< do fcntl lock on db fd */
-    int	dbi_temporary;		/*!< non-persistent */
+    int	dbi_temporary;		/*!< non-persistent index/table */
+    int dbi_noload;		/*!< standalone index/table */
     int	dbi_debug;
     int	dbi_byteswapped;
 
@@ -402,7 +394,7 @@ struct _dbiIndex {
 /*@refcounted@*/
     rpmdb dbi_rpmdb;		/*!< the parent rpm database */
     rpmTag dbi_rpmtag;		/*!< rpm tag used for index */
-    int	dbi_jlen;		/*!< size of join key */
+    size_t dbi_jlen;		/*!< size of join key */
 
 /*@only@*//*@relnull@*/
     DB * dbi_db;		/*!< Berkeley DB * handle */
@@ -459,8 +451,8 @@ struct rpmdb_s {
     int		db_opens;	/*!< No. of opens for this rpmdb. */
 /*@only@*/ /*@null@*/
     void *	db_dbenv;	/*!< Berkeley DB_ENV handle. */
-    int *	db_tagn;	/*!< Tag index mappings. */
-    int		db_ndbi;	/*!< No. of tag indices. */
+    tagStore_t	db_tags;	/*!< Tag name/value mappings. */
+    size_t	db_ndbi;	/*!< No. of tag indices. */
 /*@only@*/ /*@null@*/
     dbiIndex * _dbi;		/*!< Tag indices. */
 
@@ -491,6 +483,7 @@ extern "C" {
 
 #if defined(_RPMDB_INTERNAL)
 /*@-exportlocal@*/
+#if defined(WITH_DB) || defined(WITH_SQLITE)
 /** \ingroup db3
  * Return new configured index database handle instance.
  * @param rpmdb		rpm database
@@ -522,6 +515,7 @@ dbiIndex db3Free( /*@only@*/ /*@null@*/ dbiIndex dbi)
 extern const char * prDbiOpenFlags(int dbflags, int print_dbenv_flags)
 	/*@*/;
 /*@=redecl@*/
+#endif
 
 /** \ingroup dbi
  * Return handle for an index database.
@@ -609,13 +603,12 @@ int dbiDel(dbiIndex dbi, /*@null@*/ DBC * dbcursor, DBT * key, DBT * data,
 	/*@globals fileSystem, internalState @*/
 	/*@modifies dbi, *dbcursor, fileSystem, internalState @*/
 {
-    void * sw = dbiStatsAccumulator(dbi, 16);	/* RPMTS_OP_DBDEL */
+    rpmop sw = (rpmop)dbiStatsAccumulator(dbi, 16);	/* RPMTS_OP_DBDEL */
     int rc;
     assert(key->data != NULL && key->size > 0);
     (void) rpmswEnter(sw, 0);
     rc = (dbi->dbi_vec->cdel) (dbi, dbcursor, key, data, flags);
     (void) rpmswExit(sw, data->size);
-WSEGFAULT;
     return rc;
 }
 
@@ -634,13 +627,12 @@ int dbiGet(dbiIndex dbi, /*@null@*/ DBC * dbcursor, DBT * key, DBT * data,
 	/*@globals fileSystem, internalState @*/
 	/*@modifies dbi, *dbcursor, *key, *data, fileSystem, internalState @*/
 {
-    void * sw = dbiStatsAccumulator(dbi, 14);	/* RPMTS_OP_DBGET */
+    rpmop sw = (rpmop)dbiStatsAccumulator(dbi, 14);	/* RPMTS_OP_DBGET */
     int rc;
     assert((flags == DB_NEXT) || (key->data != NULL && key->size > 0));
     (void) rpmswEnter(sw, 0);
     rc = (dbi->dbi_vec->cget) (dbi, dbcursor, key, data, flags);
     (void) rpmswExit(sw, data->size);
-RSEGFAULT;
     return rc;
 }
 
@@ -660,7 +652,7 @@ int dbiPget(dbiIndex dbi, /*@null@*/ DBC * dbcursor,
 	/*@globals fileSystem, internalState @*/
 	/*@modifies dbi, *dbcursor, *key, *pkey, *data, fileSystem, internalState @*/
 {
-    void * sw = dbiStatsAccumulator(dbi, 14);	/* RPMTS_OP_DBGET */
+    rpmop sw = (rpmop)dbiStatsAccumulator(dbi, 14);	/* RPMTS_OP_DBGET */
     int rc;
     assert((flags == DB_NEXT) || (key->data != NULL && key->size > 0));
     (void) rpmswEnter(sw, 0);
@@ -684,13 +676,12 @@ int dbiPut(dbiIndex dbi, /*@null@*/ DBC * dbcursor, DBT * key, DBT * data,
 	/*@globals fileSystem, internalState @*/
 	/*@modifies dbi, *dbcursor, *key, fileSystem, internalState @*/
 {
-    void * sw = dbiStatsAccumulator(dbi, 15);	/* RPMTS_OP_DBPUT */
+    rpmop sw = (rpmop)dbiStatsAccumulator(dbi, 15);	/* RPMTS_OP_DBPUT */
     int rc;
     assert(key->data != NULL && key->size > 0 && data->data != NULL && data->size > 0);
     (void) rpmswEnter(sw, 0);
     rc = (dbi->dbi_vec->cput) (dbi, dbcursor, key, data, flags);
     (void) rpmswExit(sw, data->size);
-WSEGFAULT;
     return rc;
 }
 
@@ -832,11 +823,6 @@ DB_TXN * dbiTxnid(dbiIndex dbi)
 
 /*@=exportlocal@*/
 
-/** \ingroup rpmdb
- */
-unsigned int rpmdbGetIteratorFileNum(rpmdbMatchIterator mi)
-	/*@*/;
-
 /** \ingroup dbi
  * Destroy set of index database items.
  * @param set	set of index database items
@@ -910,6 +896,24 @@ rpmdb XrpmdbLink (rpmdb db, const char * msg,
 /*@=exportlocal@*/
 #define	rpmdbLink(_db, _msg)	XrpmdbLink(_db, _msg, __FILE__, __LINE__)
 
+/** @todo document rpmdbNew
+ */
+/*@only@*/ /*@null@*/
+rpmdb rpmdbNew(/*@kept@*/ /*@null@*/ const char * root,
+		/*@kept@*/ /*@null@*/ const char * home,
+		int mode, int perms, int flags);
+
+/** @todo document rpmdbOpenDatabase
+ */
+int rpmdbOpenDatabase(/*@null@*/ const char * prefix,
+		/*@null@*/ const char * dbpath,
+		int _dbapi, /*@null@*/ /*@out@*/ rpmdb *dbp,
+		int mode, int perms, int flags)
+	/*@globals rpmGlobalMacroContext, h_errno,
+		fileSystem, internalState @*/
+	/*@modifies *dbp, rpmGlobalMacroContext,
+		fileSystem, internalState @*/;
+
 /** \ingroup rpmdb
  * Open rpm database.
  * @param prefix	path to top of install tree
@@ -1010,13 +1014,20 @@ int rpmdbCountPackages(/*@null@*/ rpmdb db, const char * name)
 	/*@modifies db, rpmGlobalMacroContext, fileSystem, internalState @*/;
 
 /** \ingroup rpmdb
- * Return header join key for current position of rpm database iterator.
+ * Return header instance join key for current position of rpmdb iterator.
  * @param mi		rpm database iterator
  * @return		current header join key
  */
 unsigned int rpmdbGetIteratorOffset(/*@null@*/ rpmdbMatchIterator mi)
 	/*@*/;
 
+/** \ingroup rpmdb
+ * Return header tag index join key for current position of rpmdb iterator.
+ * @param mi		rpm database iterator
+ */
+unsigned int rpmdbGetIteratorFileNum(rpmdbMatchIterator mi)
+	/*@*/;
+
 /** \ingroup rpmdb
  * Return number of elements in rpm database iterator.
  * @param mi		rpm database iterator
@@ -1085,11 +1096,9 @@ int rpmdbSetIteratorModified(/*@null@*/ rpmdbMatchIterator mi, int modified)
  * Modify iterator to verify retrieved header blobs.
  * @param mi		rpm database iterator
  * @param ts		transaction set
- * @param (*hdrchk)	headerCheck() vector
  * @return		0 always
  */
-int rpmdbSetHdrChk(/*@null@*/ rpmdbMatchIterator mi, /*@null@*/ rpmts ts,
-		/*@null@*/ rpmRC (*hdrchk) (rpmts ts, const void * uh, size_t uc, const char ** msg))
+int rpmdbSetHdrChk(/*@null@*/ rpmdbMatchIterator mi, /*@null@*/ rpmts ts)
 	/*@modifies mi @*/;
 
 /** \ingroup rpmdb
@@ -1117,7 +1126,21 @@ Header rpmdbNextIterator(/*@null@*/ rpmdbMatchIterator mi)
 	/*@modifies mi, rpmGlobalMacroContext, fileSystem, internalState @*/;
 
 /** \ingroup rpmdb
- * Check rpmdb signal handler for trapped signal exit.
+ * Check rpmdb signal handler for trapped signal and/or requested exit.
+ * Clean up any open iterators and databases on termination condition.
+ * On non-zero exit any open references to rpmdb are invalid and cannot
+ * be accessed anymore, calling process should terminate immediately.
+ *
+ * @param terminate	0 to only check for signals, 1 to terminate anyway
+ * @return		0 to continue, 1 if termination cleanup was done.
+ */
+/*@mayexit@*/
+int rpmdbCheckTerminate(int terminate)
+	/*@globals fileSystem, internalState @*/
+	/*@modifies fileSystem, internalState @*/;
+
+/** \ingroup rpmdb
+ * Check for and exit on termination signals.
  */
 /*@mayexit@*/
 int rpmdbCheckSignals(void)
@@ -1140,13 +1163,12 @@ rpmdbMatchIterator rpmdbFreeIterator(/*@only@*/ /*@null@*/rpmdbMatchIterator mi)
  * @param iid		install transaction id (iid = 0 or -1 to skip)
  * @param h		header
  * @param ts		(unused) transaction set (or NULL)
- * @param (*hdrchk)	(unused) headerCheck() vector (or NULL)
  * @return		0 on success
  */
-int rpmdbAdd(/*@null@*/ rpmdb db, int iid, Header h, /*@null@*/ rpmts ts,
-		/*@null@*/ rpmRC (*hdrchk) (rpmts ts, const void *uh, size_t uc, const char ** msg))
+int rpmdbAdd(/*@null@*/ rpmdb db, int iid, Header h, /*@null@*/ rpmts ts)
 	/*@globals rpmGlobalMacroContext, h_errno, fileSystem, internalState @*/
-	/*@modifies db, h, rpmGlobalMacroContext, fileSystem, internalState @*/;
+	/*@modifies db, h, ts,
+		rpmGlobalMacroContext, fileSystem, internalState @*/;
 
 /** \ingroup rpmdb
  * Remove package header from rpm database and indices.
@@ -1154,39 +1176,32 @@ int rpmdbAdd(/*@null@*/ rpmdb db, int iid, Header h, /*@null@*/ rpmts ts,
  * @param rid		(unused) remove transaction id (rid = 0 or -1 to skip)
  * @param hdrNum	package instance number in database
  * @param ts		(unused) transaction set (or NULL)
- * @param (*hdrchk)	(unused) headerCheck() vector (or NULL)
  * @return		0 on success
  */
 int rpmdbRemove(/*@null@*/ rpmdb db, /*@unused@*/ int rid, unsigned int hdrNum,
-		/*@null@*/ rpmts ts,
-		/*@null@*/ rpmRC (*hdrchk) (rpmts ts, const void *uh, size_t uc, const char ** msg))
+		/*@null@*/ rpmts ts)
 	/*@globals rpmGlobalMacroContext, h_errno, fileSystem, internalState @*/
-	/*@modifies db, rpmGlobalMacroContext, fileSystem, internalState @*/;
+	/*@modifies db, ts,
+		rpmGlobalMacroContext, fileSystem, internalState @*/;
 
 /** \ingroup rpmdb
  * Rebuild database indices from package headers.
  * @param prefix	path to top of install tree
  * @param ts		transaction set (or NULL)
- * @param (*hdrchk)	headerCheck() vector (or NULL)
  * @return		0 on success
  */
-int rpmdbRebuild(/*@null@*/ const char * prefix, /*@null@*/ rpmts ts,
-		/*@null@*/ rpmRC (*hdrchk) (rpmts ts, const void *uh, size_t uc, const char ** msg))
+int rpmdbRebuild(/*@null@*/ const char * prefix, /*@null@*/ rpmts ts)
 	/*@globals rpmGlobalMacroContext, h_errno, fileSystem, internalState @*/
-	/*@modifies rpmGlobalMacroContext, fileSystem, internalState @*/;
+	/*@modifies ts, rpmGlobalMacroContext, fileSystem, internalState @*/;
 
-#ifndef __APPLE__
 /**
  * Mergesort, same arguments as qsort(2).
  */
 /*@unused@*/
-int mergesort(void *base, size_t nmemb, size_t size,
+int rpm_mergesort(void *base, size_t nmemb, size_t size,
                 int (*cmp) (const void *, const void *))
 	/*@globals errno @*/
 	/*@modifies base, errno @*/;
-#else
-/* mergesort is defined in stdlib.h on Mac OS X */
-#endif /* __APPLE__ */
 
 #ifdef __cplusplus
 }
