@@ -1,4 +1,3 @@
-/*@-bounds@*/
 /*@-mustmod@*/
 /*@-paramuse@*/
 /*@-globuse@*/
@@ -6,7 +5,6 @@
 /*@-noeffectuncon@*/
 /*@-compdef@*/
 /*@-compmempass@*/
-/*@-branchstate@*/
 /*@-modfilesystem@*/
 /*@-evalorderuncon@*/
 
@@ -37,10 +35,12 @@
 
 #include "system.h"
 
-#include <rpmlib.h>
+#include <rpmio.h>
+#include <rpmlog.h>
 #include <rpmmacro.h>
 #include <rpmurl.h>     /* XXX urlPath proto */
 
+#include <rpmtag.h>
 #define	_RPMDB_INTERNAL
 #include <rpmdb.h>
 
@@ -77,7 +77,7 @@ struct _sql_dbcursor_s {
 /*@only@*/ /*@relnull@*/
     char ** av;			/* item ptrs */
 /*@only@*/ /*@relnull@*/
-    int * avlen;		/* item sizes */
+    size_t * avlen;		/* item sizes */
     int nalloc;
     int ac;			/* no. of items */
     int rx;			/* Which row are we on? 1, 2, 3 ... */
@@ -97,10 +97,12 @@ struct _sql_dbcursor_s {
     int used;
 };
 
+/*@-redef@*/
 union _dbswap {
-    unsigned int ui;
+    uint32_t ui;
     unsigned char uc[4];
 };
+/*@=redef@*/
 
 #define _DBSWAP(_a) \
   { unsigned char _b, *_c = (_a).uc; \
@@ -144,7 +146,9 @@ fprintf(stderr, "sql:chroot(%s)\n", dbi->dbi_root);
 /*@-globs@*/
     xx = Chdir("/");
 /*@=globs@*/
+/*@-modobserver@*/
     xx = Chroot(dbi->dbi_root);
+/*@=modobserver@*/
 assert(xx == 0);
     sqlInRoot=1;
 }
@@ -162,9 +166,9 @@ static void leaveChroot(dbiIndex dbi)
 if (_debug)
 fprintf(stderr, "sql:chroot(.)\n");
 
-/*@-superuser@*/
+/*@-modobserver@*/
     xx = Chroot(".");
-/*@=superuser@*/
+/*@=modobserver@*/
 assert(xx == 0);
     if (sqlCwd != NULL) {
 /*@-globs@*/
@@ -218,7 +222,7 @@ static SCP_t scpResetKeys(/*@only@*/ SCP_t scp)
     int ix;
 
 if (_debug)
-fprintf(stderr, "*** %s(%p)\n", __FUNCTION__, scp);
+fprintf(stderr, "*** scpResetKeys(%p)\n", scp);
 dbg_scp(scp);
 
     for ( ix =0 ; ix < scp->nkeys ; ix++ ) {
@@ -240,7 +244,7 @@ static SCP_t scpResetAv(/*@only@*/ SCP_t scp)
     int xx;
 
 if (_debug)
-fprintf(stderr, "*** %s(%p)\n", __FUNCTION__, scp);
+fprintf(stderr, "*** scpResetAv(%p)\n", scp);
 dbg_scp(scp);
 
     if (scp->av != NULL) {
@@ -279,7 +283,7 @@ static SCP_t scpReset(/*@only@*/ SCP_t scp)
     int xx;
 
 if (_debug)
-fprintf(stderr, "*** %s(%p)\n", __FUNCTION__, scp);
+fprintf(stderr, "*** scpReset(%p)\n", scp);
 dbg_scp(scp);
 
     if (scp->cmd) {
@@ -288,9 +292,9 @@ dbg_scp(scp);
     }
     if (scp->pStmt) {
 	xx = sqlite3_reset(scp->pStmt);
-	if (xx) rpmMessage(RPMMESS_WARNING, "reset %d\n", xx);
+	if (xx) rpmlog(RPMLOG_WARNING, "reset %d\n", xx);
 	xx = sqlite3_finalize(scp->pStmt);
-	if (xx) rpmMessage(RPMMESS_WARNING, "finalize %d\n", xx);
+	if (xx) rpmlog(RPMLOG_WARNING, "finalize %d\n", xx);
 	scp->pStmt = NULL;
     }
 
@@ -310,7 +314,7 @@ static SCP_t scpFree(/*@only@*/ SCP_t scp)
     scp->avlen = _free(scp->avlen);
 
 if (_debug)
-fprintf(stderr, "*** %s(%p)\n", __FUNCTION__, scp);
+fprintf(stderr, "*** scpFree(%p)\n", scp);
     scp = _free(scp);
     return NULL;
 }
@@ -329,7 +333,7 @@ static SCP_t scpNew(DB * dbp)
     scp->ldata = NULL;
 
 if (_debug)
-fprintf(stderr, "*** %s(%p)\n", __FUNCTION__, scp);
+fprintf(stderr, "*** scpNew(%p)\n", scp);
     return scp;
 }
 
@@ -427,7 +431,7 @@ assert(swapped == 0); /* Byte swap?! */
 		    int32_t v = sqlite3_column_int(scp->pStmt, i);
 		    nb = sizeof(v);
 if (_debug)
-fprintf(stderr, "\t%d %s %s %d\n", i, cname, vtype, v);
+fprintf(stderr, "\t%d %s %s %d\n", i, cname, vtype, (int) v);
 		    if (nb > 0) {
 			scp->av[scp->ac] = memcpy(xmalloc(nb), &v, nb);
 			scp->avlen[scp->ac] = nb;
@@ -453,7 +457,7 @@ assert(swapped == 0); /* Byte swap?! */
 		    }
 		} else
 		if (!strcmp(vtype, "text")) {
-		    const char * v = sqlite3_column_text(scp->pStmt, i);
+		    const char * v = (const char *)sqlite3_column_text(scp->pStmt, i);
 		    nb = strlen(v) + 1;
 if (_debug)
 fprintf(stderr, "\t%d %s %s \"%s\"\n", i, cname, vtype, v);
@@ -508,7 +512,7 @@ assert(key->data != NULL);
     switch (dbi->dbi_rpmtag) {
     case RPMDBI_PACKAGES:
 	{   unsigned int hnum;
-/*@i@*/ assert(key->size == sizeof(int_32));
+/*@i@*/ assert(key->size == sizeof(uint32_t));
 	    memcpy(&hnum, key->data, sizeof(hnum));
 
 if (swapped == 1) {
@@ -520,32 +524,32 @@ if (swapped == 1) {
 	} break;
     default:
 	switch (tagType(dbi->dbi_rpmtag) & RPM_MASK_TYPE) {
-	case RPM_NULL_TYPE:   
 	case RPM_BIN_TYPE:
 /*@-castfcnptr -nullpass@*/ /* FIX: annotate sqlite. */
 	    rc = sqlite3_bind_blob(scp->pStmt, pos, key->data, key->size, SQLITE_STATIC);
 /*@=castfcnptr =nullpass@*/
 	    /*@innerbreak@*/ break;
-	case RPM_CHAR_TYPE:
-	case RPM_INT8_TYPE:
+	case RPM_UINT8_TYPE:
 	{   unsigned char i;
 /*@i@*/ assert(key->size == sizeof(unsigned char));
 assert(swapped == 0); /* Byte swap?! */
 	    memcpy(&i, key->data, sizeof(i));
-	    rc = sqlite3_bind_int(scp->pStmt, pos, i);
+	    rc = sqlite3_bind_int(scp->pStmt, pos, (int) i);
 	} /*@innerbreak@*/ break;
-	case RPM_INT16_TYPE:
+	case RPM_UINT16_TYPE:
 	{	unsigned short i;
-/*@i@*/ assert(key->size == sizeof(int_16));
+/*@i@*/ assert(key->size == sizeof(uint16_t));
 assert(swapped == 0); /* Byte swap?! */
 	    memcpy(&i, key->data, sizeof(i));
-	    rc = sqlite3_bind_int(scp->pStmt, pos, i);
+	    rc = sqlite3_bind_int(scp->pStmt, pos, (int) i);
 	} /*@innerbreak@*/ break;
-        case RPM_INT32_TYPE:
-/*      case RPM_INT64_TYPE: */   
+	case RPM_UINT64_TYPE:
+assert(0);	/* borken */
+	/*@innerbreak@*/ break;
+        case RPM_UINT32_TYPE:
 	default:
 	{   unsigned int i;
-/*@i@*/ assert(key->size == sizeof(int_32));
+/*@i@*/ assert(key->size == sizeof(uint32_t));
 	    memcpy(&i, key->data, sizeof(i));
 
 if (swapped == 1)
@@ -663,7 +667,7 @@ static int sql_busy_handler(void * dbi_void, int time)
     dbiIndex dbi = (dbiIndex) dbi_void;
 /*@=castexpose@*/
 
-    rpmMessage(RPMMESS_WARNING, _("Unable to get lock on db %s, retrying... (%d)\n"),
+    rpmlog(RPMLOG_WARNING, _("Unable to get lock on db %s, retrying... (%d)\n"),
 		dbi->dbi_file, time);
 
     (void) sleep(1);
@@ -679,13 +683,43 @@ static int sql_busy_handler(void * dbi_void, int time)
  * Create the table.. create the db_info
  */
 static int sql_initDB(dbiIndex dbi)
-	/*@*/
+	/*@globals rpmGlobalMacroContext, h_errno @*/
 {
     SQL_DB * sqldb = (SQL_DB *) dbi->dbi_db;
     SCP_t scp = scpNew(dbi->dbi_db);
     char cmd[BUFSIZ];
     int rc = 0;
 
+    if (dbi->dbi_tmpdir) {
+        const char *root;
+        const char *tmpdir;
+        int xx;
+        root = (dbi->dbi_root ? dbi->dbi_root : dbi->dbi_rpmdb->db_root);
+        if ((root[0] == '/' && root[1] == '\0') || dbi->dbi_rpmdb->db_chrootDone)
+            root = NULL;
+        /*@-mods@*/
+        tmpdir = rpmGenPath(root, dbi->dbi_tmpdir, NULL);
+        /*@=mods@*/
+        sprintf(cmd, "PRAGMA temp_store_directory = '%s';", tmpdir);
+        xx = sqlite3_exec(sqldb->db, cmd, NULL, NULL, (char **)&scp->pzErrmsg);
+        tmpdir = _free(tmpdir);
+    }
+    if (dbi->dbi_eflags & DB_EXCL) {
+        int xx;
+        sprintf(cmd, "PRAGMA locking_mode = EXCLUSIVE;");
+        xx = sqlite3_exec(sqldb->db, cmd, NULL, NULL, (char **)&scp->pzErrmsg);
+    }
+    if (dbi->dbi_pagesize > 0) {
+        int xx;
+        sprintf(cmd, "PRAGMA cache_size = %d;", dbi->dbi_cachesize);
+        xx = sqlite3_exec(sqldb->db, cmd, NULL, NULL, (char **)&scp->pzErrmsg);
+    }
+    if (dbi->dbi_cachesize > 0) {
+        int xx;
+        sprintf(cmd, "PRAGMA page_size = %d;", dbi->dbi_pagesize);
+        xx = sqlite3_exec(sqldb->db, cmd, NULL, NULL, (char **)&scp->pzErrmsg);
+    }
+
     /* Check if the table exists... */
     sprintf(cmd,
 	"SELECT name FROM 'sqlite_master' WHERE type='table' and name='%s';",
@@ -708,16 +742,14 @@ static int sql_initDB(dbiIndex dbi)
 	    break;
 	default:
 	    switch (tagType(dbi->dbi_rpmtag) & RPM_MASK_TYPE) {
-	    case RPM_NULL_TYPE:
 	    case RPM_BIN_TYPE:
 	    default:
 		keytype = "blob UNIQUE";
 		/*@innerbreak@*/ break;
-	    case RPM_CHAR_TYPE:
-	    case RPM_INT8_TYPE:
-	    case RPM_INT16_TYPE:
-	    case RPM_INT32_TYPE:
-/*	    case RPM_INT64_TYPE: */
+	    case RPM_UINT8_TYPE:
+	    case RPM_UINT16_TYPE:
+	    case RPM_UINT32_TYPE:
+	    case RPM_UINT64_TYPE:
 		keytype = "int UNIQUE";
 		/*@innerbreak@*/ break;
 	    case RPM_STRING_TYPE:
@@ -729,18 +761,20 @@ static int sql_initDB(dbiIndex dbi)
 	}
 if (_debug)
 fprintf(stderr, "\t%s(%d) type(%d) keytype %s\n", tagName(dbi->dbi_rpmtag), dbi->dbi_rpmtag, (tagType(dbi->dbi_rpmtag) & RPM_MASK_TYPE), keytype);
-	sprintf(cmd, "CREATE TABLE '%s' (key %s, value %s)",
+	sprintf(cmd, "CREATE %sTABLE '%s' (key %s, value %s)",
+			dbi->dbi_temporary ? "TEMPORARY " : "",
 			dbi->dbi_subfile, keytype, valtype);
 	rc = sqlite3_exec(sqldb->db, cmd, NULL, NULL, (char **)&scp->pzErrmsg);
 	if (rc)
 	    goto exit;
 
-	sprintf(cmd, "CREATE TABLE 'db_info' (endian TEXT)");
+	sprintf(cmd, "CREATE %sTABLE 'db_info' (endian TEXT)",
+			dbi->dbi_temporary ? "TEMPORARY " : "");
 	rc = sqlite3_exec(sqldb->db, cmd, NULL, NULL, (char **)&scp->pzErrmsg);
 	if (rc)
 	    goto exit;
 
-	sprintf(cmd, "INSERT INTO 'db_info' values('%d')", ((union _dbswap *)&endian)->uc[0]);
+	sprintf(cmd, "INSERT INTO 'db_info' values('%u')", (unsigned)((union _dbswap *)&endian)->uc[0]);
 	rc = sqlite3_exec(sqldb->db, cmd, NULL, NULL, (char **)&scp->pzErrmsg);
 	if (rc)
 	    goto exit;
@@ -754,7 +788,7 @@ fprintf(stderr, "\t%s(%d) type(%d) keytype %s\n", tagName(dbi->dbi_rpmtag), dbi-
 
 exit:
     if (rc)
-	rpmMessage(RPMMESS_WARNING, "Unable to initDB %s (%d)\n",
+	rpmlog(RPMLOG_WARNING, "Unable to initDB %s (%d)\n",
 		scp->pzErrmsg, rc);
 
     scp = scpFree(scp);
@@ -778,7 +812,7 @@ static int sql_cclose (dbiIndex dbi, /*@only@*/ DBC * dbcursor,
     int rc;
 
 if (_debug)
-fprintf(stderr, "==> %s(%p)\n", __FUNCTION__, scp);
+fprintf(stderr, "==> sql_cclose(%p)\n", scp);
 
     if (scp->lkey)
 	scp->lkey = _free(scp->lkey);
@@ -823,9 +857,20 @@ enterChroot(dbi);
 
 	(void) sqlite3_close(sqldb->db);
 
-	rpmMessage(RPMMESS_DEBUG, _("closed   sql db         %s\n"),
+	rpmlog(RPMLOG_DEBUG, D_("closed   sql db         %s\n"),
 		dbi->dbi_subfile);
 
+#if defined(MAYBE) /* XXX should SQLite and BDB have different semantics? */
+	if (dbi->dbi_temporary && !(dbi->dbi_eflags & DB_PRIVATE)) {
+	    const char * dbhome = NULL;
+	    urltype ut = urlPath(dbi->dbi_home, &dbhome);
+	    const char * dbfname = rpmGenPath(dbhome, dbi->dbi_file, NULL);
+	    int xx = (dbfname ? Unlink(dbfname) : 0);
+	    ut = ut; xx = xx;	/* XXX tell gcc to be quiet. */
+	    dbfname = _free(dbfname);
+	}
+#endif
+
 	dbi->dbi_stats = _free(dbi->dbi_stats);
 	dbi->dbi_file = _free(dbi->dbi_file);
 	dbi->dbi_db = _free(dbi->dbi_db);
@@ -859,6 +904,7 @@ static int sql_open(rpmdb rpmdb, rpmTag rpmtag, /*@out@*/ dbiIndex * dbip)
     const char * dbfile;  
     const char * dbfname;
     const char * sql_errcode;
+    mode_t umask_safed = 0002;
     dbiIndex dbi;
     SQL_DB * sqldb;
     size_t len;
@@ -922,16 +968,29 @@ enterChroot(dbi);
      */
     (void) rpmioMkpath(dbhome, 0755, getuid(), getgid());
        
-    dbfname = rpmGenPath(dbhome, dbi->dbi_file, NULL);
+    if (dbi->dbi_eflags & DB_PRIVATE)
+        dbfname = xstrdup(":memory:");
+    else
+        dbfname = rpmGenPath(dbhome, dbi->dbi_file, NULL);
 
-    rpmMessage(RPMMESS_DEBUG, _("opening  sql db         %s (%s) mode=0x%x\n"),
+    rpmlog(RPMLOG_DEBUG, D_("opening  sql db         %s (%s) mode=0x%x\n"),
 		dbfname, dbi->dbi_subfile, dbi->dbi_mode);
 
     /* Open the Database */
     sqldb = xcalloc(1, sizeof(*sqldb));
        
     sql_errcode = NULL;
+    if (dbi->dbi_perms)
+        /* mask-out permission bits which are not requested (security) */
+        umask_safed = umask(~((mode_t)(dbi->dbi_perms)));
     xx = sqlite3_open(dbfname, &sqldb->db);
+    if (dbi->dbi_perms) {
+        if ((0644 /* = SQLite hard-coded default */ & dbi->dbi_perms) != dbi->dbi_perms) {
+            /* add requested permission bits which are still missing (semantic) */
+            (void) Chmod(dbfname, dbi->dbi_perms);
+        }
+        (void) umask(umask_safed);
+    }
     if (xx != SQLITE_OK)
 	sql_errcode = sqlite3_errmsg(sqldb->db);
 
@@ -943,7 +1002,7 @@ enterChroot(dbi);
     dbi->dbi_db = (DB *)sqldb;
 
     if (sql_errcode != NULL) {
-      rpmMessage(RPMMESS_DEBUG, "Unable to open database: %s\n", sql_errcode);
+      rpmlog(RPMLOG_DEBUG, D_("Unable to open database: %s\n"), sql_errcode);
       rc = EINVAL;
     }
 
@@ -989,11 +1048,11 @@ leaveChroot(dbi);
 
 /**
  * Open database cursor.
- * @param dbi           index database handle
- * @param txnid         database transaction handle
- * @retval dbcp         address of new database cursor
- * @param dbiflags      DB_WRITECURSOR or 0
- * @return              0 on success
+ * @param dbi		index database handle
+ * @param txnid		database transaction handle
+ * @retval dbcp		address of new database cursor
+ * @param flags		DB_WRITECURSOR or 0
+ * @return		0 on success
  */
 static int sql_copen (dbiIndex dbi, /*@null@*/ DB_TXN * txnid,
 		/*@out@*/ DBC ** dbcp, unsigned int flags)
@@ -1005,7 +1064,7 @@ static int sql_copen (dbiIndex dbi, /*@null@*/ DB_TXN * txnid,
     int rc = 0;
 
 if (_debug)
-fprintf(stderr, "==> %s(%s) tag %d type %d scp %p\n", __FUNCTION__, tagName(dbi->dbi_rpmtag), dbi->dbi_rpmtag, (tagType(dbi->dbi_rpmtag) & RPM_MASK_TYPE), scp);
+fprintf(stderr, "==> sql_copen(%s) tag %d type %d scp %p\n", tagName(dbi->dbi_rpmtag), dbi->dbi_rpmtag, (tagType(dbi->dbi_rpmtag) & RPM_MASK_TYPE), scp);
 
 enterChroot(dbi);
 
@@ -1041,21 +1100,21 @@ static int sql_cdel (dbiIndex dbi, /*@null@*/ DBC * dbcursor, DBT * key,
     SCP_t scp = scpNew(dbi->dbi_db);
     int rc = 0;
 
-dbg_keyval(__FUNCTION__, dbi, dbcursor, key, data, flags);
+dbg_keyval("sql_cdel", dbi, dbcursor, key, data, flags);
 enterChroot(dbi);
 
     scp->cmd = sqlite3_mprintf("DELETE FROM '%q' WHERE key=? AND value=?;",
 	dbi->dbi_subfile);
 
-    rc = sqlite3_prepare(sqldb->db, scp->cmd, strlen(scp->cmd), &scp->pStmt, &scp->pzErrmsg);
-    if (rc) rpmMessage(RPMMESS_WARNING, "cdel(%s) prepare %s (%d)\n", dbi->dbi_subfile, sqlite3_errmsg(sqldb->db), rc);
+    rc = sqlite3_prepare(sqldb->db, scp->cmd, (int)strlen(scp->cmd), &scp->pStmt, &scp->pzErrmsg);
+    if (rc) rpmlog(RPMLOG_WARNING, "cdel(%s) prepare %s (%d)\n", dbi->dbi_subfile, sqlite3_errmsg(sqldb->db), rc);
     rc = sql_bind_key(dbi, scp, 1, key);
-    if (rc) rpmMessage(RPMMESS_WARNING, "cdel(%s) bind key %s (%d)\n", dbi->dbi_subfile, sqlite3_errmsg(sqldb->db), rc);
+    if (rc) rpmlog(RPMLOG_WARNING, "cdel(%s) bind key %s (%d)\n", dbi->dbi_subfile, sqlite3_errmsg(sqldb->db), rc);
     rc = sql_bind_data(dbi, scp, 2, data);
-    if (rc) rpmMessage(RPMMESS_WARNING, "cdel(%s) bind data %s (%d)\n", dbi->dbi_subfile, sqlite3_errmsg(sqldb->db), rc);
+    if (rc) rpmlog(RPMLOG_WARNING, "cdel(%s) bind data %s (%d)\n", dbi->dbi_subfile, sqlite3_errmsg(sqldb->db), rc);
 
     rc = sql_step(dbi, scp);
-    if (rc) rpmMessage(RPMMESS_WARNING, "cdel(%s) sql_step rc %d\n", dbi->dbi_subfile, rc);
+    if (rc) rpmlog(RPMLOG_WARNING, "cdel(%s) sql_step rc %d\n", dbi->dbi_subfile, rc);
 
     scp = scpFree(scp);
 
@@ -1084,7 +1143,7 @@ static int sql_cget (dbiIndex dbi, /*@null@*/ DBC * dbcursor, DBT * key,
     int ix;
 
 assert(dbcursor != NULL);
-dbg_keyval(__FUNCTION__, dbi, dbcursor, key, data, flags);
+dbg_keyval("sql_cget", dbi, dbcursor, key, data, flags);
 
 enterChroot(dbi);
 
@@ -1125,18 +1184,18 @@ assert(dbi->dbi_rpmtag == RPMDBI_PACKAGES);
 		    dbi->dbi_subfile);
 	        break;
 	    }
-	    rc = sqlite3_prepare(sqldb->db, scp->cmd, strlen(scp->cmd), &scp->pStmt, &scp->pzErrmsg);
-	    if (rc) rpmMessage(RPMMESS_WARNING, "cget(%s) sequential prepare %s (%d)\n", dbi->dbi_subfile, sqlite3_errmsg(sqldb->db), rc);
+	    rc = sqlite3_prepare(sqldb->db, scp->cmd, (int)strlen(scp->cmd), &scp->pStmt, &scp->pzErrmsg);
+	    if (rc) rpmlog(RPMLOG_WARNING, "cget(%s) sequential prepare %s (%d)\n", dbi->dbi_subfile, sqlite3_errmsg(sqldb->db), rc);
 
 	    rc = sql_step(dbi, scp);
-	    if (rc) rpmMessage(RPMMESS_WARNING, "cget(%s) sequential sql_step rc %d\n", dbi->dbi_subfile, rc);
+	    if (rc) rpmlog(RPMLOG_WARNING, "cget(%s) sequential sql_step rc %d\n", dbi->dbi_subfile, rc);
 
 	    scp = scpResetKeys(scp);
 	    scp->nkeys = scp->nr;
 	    scp->keys = xcalloc(scp->nkeys, sizeof(*scp->keys));
 	    for (ix = 0; ix < scp->nkeys; ix++) {
 		scp->keys[ix] = xmalloc(sizeof(*scp->keys[0]));
-		scp->keys[ix]->size = scp->avlen[ix+1];
+		scp->keys[ix]->size = (u_int32_t) scp->avlen[ix+1];
 		scp->keys[ix]->data = xmalloc(scp->keys[ix]->size);
 		memcpy(scp->keys[ix]->data, scp->av[ix+1], scp->avlen[ix+1]);
 	    }
@@ -1157,9 +1216,9 @@ assert(dbi->dbi_rpmtag == RPMDBI_PACKAGES);
 
         /* Prepare SQL statement to retrieve the value for the current key */
         scp->cmd = sqlite3_mprintf("SELECT value FROM '%q' WHERE key=?;", dbi->dbi_subfile);
-        rc = sqlite3_prepare(sqldb->db, scp->cmd, strlen(scp->cmd), &scp->pStmt, &scp->pzErrmsg);
+        rc = sqlite3_prepare(sqldb->db, scp->cmd, (int)strlen(scp->cmd), &scp->pStmt, &scp->pzErrmsg);
 
-        if (rc) rpmMessage(RPMMESS_WARNING, "cget(%s) prepare %s (%d)\n", dbi->dbi_subfile, sqlite3_errmsg(sqldb->db), rc);
+        if (rc) rpmlog(RPMLOG_WARNING, "cget(%s) prepare %s (%d)\n", dbi->dbi_subfile, sqlite3_errmsg(sqldb->db), rc);
     }
 
 /*@i@*/ scp = scpResetAv(scp);	/* Free av and avlen, reset counters.*/
@@ -1173,13 +1232,13 @@ assert(dbi->dbi_rpmtag == RPMDBI_PACKAGES);
 
     /* Bind key to prepared statement */
     rc = sql_bind_key(dbi, scp, 1, scp->keys[scp->rx]);
-    if (rc) rpmMessage(RPMMESS_WARNING, "cget(%s)  key bind %s (%d)\n", dbi->dbi_subfile, sqlite3_errmsg(sqldb->db), rc);
+    if (rc) rpmlog(RPMLOG_WARNING, "cget(%s)  key bind %s (%d)\n", dbi->dbi_subfile, sqlite3_errmsg(sqldb->db), rc);
 
     rc = sql_step(dbi, scp);
-    if (rc) rpmMessage(RPMMESS_WARNING, "cget(%s) sql_step rc %d\n", dbi->dbi_subfile, rc);
+    if (rc) rpmlog(RPMLOG_WARNING, "cget(%s) sql_step rc %d\n", dbi->dbi_subfile, rc);
 
     rc = sqlite3_reset(scp->pStmt);
-    if (rc) rpmMessage(RPMMESS_WARNING, "reset %d\n", rc);
+    if (rc) rpmlog(RPMLOG_WARNING, "reset %d\n", rc);
 
 /* 1 key should return 0 or 1 row/value */
 assert(scp->nr < 2);
@@ -1215,7 +1274,7 @@ assert(scp->nr == 1);
 	    scp->ldata = _free(scp->ldata);
 	}
 
-	data->size = scp->avlen[1];
+	data->size = (u_int32_t) scp->avlen[1];
         data->data = xmalloc(data->size);
 	if (! (data->flags & DB_DBT_MALLOC) )
 	    scp->ldata = data->data;
@@ -1262,7 +1321,7 @@ static int sql_cput (dbiIndex dbi, /*@null@*/ DBC * dbcursor, DBT * key,
     SCP_t scp = scpNew(dbi->dbi_db);
     int rc = 0;
 
-dbg_keyval(__FUNCTION__, dbi, dbcursor, key, data, flags);
+dbg_keyval("sql_cput", dbi, dbcursor, key, data, flags);
 
 enterChroot(dbi);
 
@@ -1270,15 +1329,15 @@ enterChroot(dbi);
     default:
 	scp->cmd = sqlite3_mprintf("INSERT OR REPLACE INTO '%q' VALUES(?, ?);",
 		dbi->dbi_subfile);
-	rc = sqlite3_prepare(sqldb->db, scp->cmd, strlen(scp->cmd), &scp->pStmt, &scp->pzErrmsg);
-	if (rc) rpmMessage(RPMMESS_WARNING, "cput(%s) prepare %s (%d)\n",dbi->dbi_subfile,  sqlite3_errmsg(sqldb->db), rc);
+	rc = sqlite3_prepare(sqldb->db, scp->cmd, (int)strlen(scp->cmd), &scp->pStmt, &scp->pzErrmsg);
+	if (rc) rpmlog(RPMLOG_WARNING, "cput(%s) prepare %s (%d)\n",dbi->dbi_subfile,  sqlite3_errmsg(sqldb->db), rc);
 	rc = sql_bind_key(dbi, scp, 1, key);
-	if (rc) rpmMessage(RPMMESS_WARNING, "cput(%s)  key bind %s (%d)\n", dbi->dbi_subfile, sqlite3_errmsg(sqldb->db), rc);
+	if (rc) rpmlog(RPMLOG_WARNING, "cput(%s)  key bind %s (%d)\n", dbi->dbi_subfile, sqlite3_errmsg(sqldb->db), rc);
 	rc = sql_bind_data(dbi, scp, 2, data);
-	if (rc) rpmMessage(RPMMESS_WARNING, "cput(%s) data bind %s (%d)\n", dbi->dbi_subfile, sqlite3_errmsg(sqldb->db), rc);
+	if (rc) rpmlog(RPMLOG_WARNING, "cput(%s) data bind %s (%d)\n", dbi->dbi_subfile, sqlite3_errmsg(sqldb->db), rc);
 
 	rc = sql_step(dbi, scp);
-	if (rc) rpmMessage(RPMMESS_WARNING, "cput(%s) sql_step rc %d\n", dbi->dbi_subfile, rc);
+	if (rc) rpmlog(RPMLOG_WARNING, "cput(%s) sql_step rc %d\n", dbi->dbi_subfile, rc);
 
 	break;
     }
@@ -1313,7 +1372,7 @@ enterChroot(dbi);
 
     if (sql_rc == 0 && scp->nr > 0) {
 assert(scp->av != NULL);
-	db_endian.uc[0] = strtol(scp->av[1], NULL, 10);
+	db_endian.uc[0] = (unsigned char) strtol(scp->av[1], NULL, 10);
 
 	if ( db_endian.uc[0] == ((union _dbswap *)&endian)->uc[0] )
 	    rc = 0; /* Native endian */
@@ -1322,10 +1381,10 @@ assert(scp->av != NULL);
 
     } else {
 	if ( sql_rc ) {
-	    rpmMessage(RPMMESS_DEBUG, "db_info failed %s (%d)\n",
+	    rpmlog(RPMLOG_DEBUG, D_("db_info failed %s (%d)\n"),
 		scp->pzErrmsg, sql_rc);
 	}
-	rpmMessage(RPMMESS_WARNING, "Unable to determine DB endian.\n");
+	rpmlog(RPMLOG_WARNING, "Unable to determine DB endian.\n");
     }
 
     scp = scpFree(scp);
@@ -1356,7 +1415,7 @@ static int sql_associate (dbiIndex dbi, dbiIndex dbisecondary,
 	/*@*/
 {
 if (_debug)
-fprintf(stderr, "*** %s:\n", __FUNCTION__);
+fprintf(stderr, "*** sql_associate:\n");
     return EINVAL;
 }
 
@@ -1374,7 +1433,7 @@ static int sql_join (dbiIndex dbi, DBC ** curslist, /*@out@*/ DBC ** dbcp,
 	/*@modifies dbi, *dbcp, fileSystem @*/
 {
 if (_debug)
-fprintf(stderr, "*** %s:\n", __FUNCTION__);
+fprintf(stderr, "*** sql_join:\n");
     return EINVAL;
 }
 
@@ -1392,7 +1451,7 @@ static int sql_cdup (dbiIndex dbi, DBC * dbcursor, /*@out@*/ DBC ** dbcp,
 	/*@modifies dbi, *dbcp, fileSystem @*/
 {
 if (_debug)
-fprintf(stderr, "*** %s:\n", __FUNCTION__);
+fprintf(stderr, "*** sql_cdup:\n");
     return EINVAL;
 }
 
@@ -1412,7 +1471,7 @@ static int sql_cpget (dbiIndex dbi, /*@null@*/ DBC * dbcursor,
 	/*@modifies *dbcursor, *key, *pkey, *data, fileSystem @*/
 {
 if (_debug)
-fprintf(stderr, "*** %s:\n", __FUNCTION__);
+fprintf(stderr, "*** sql_cpget:\n");
     return EINVAL;
 }
 
@@ -1431,7 +1490,7 @@ static int sql_ccount (dbiIndex dbi, /*@unused@*/ DBC * dbcursor,
 	/*@modifies *dbcursor, fileSystem @*/
 {
 if (_debug)
-fprintf(stderr, "*** %s:\n", __FUNCTION__);
+fprintf(stderr, "*** sql_ccount:\n");
     return EINVAL;
 }
 
@@ -1468,11 +1527,11 @@ enterChroot(dbi);
 assert(scp->av != NULL);
 	nkeys = strtol(scp->av[1], NULL, 10);
 
-	rpmMessage(RPMMESS_DEBUG, "  stat on %s nkeys %ld\n",
+	rpmlog(RPMLOG_DEBUG, D_("  stat on %s nkeys %ld\n"),
 		dbi->dbi_subfile, nkeys);
     } else {
 	if ( rc ) {
-	    rpmMessage(RPMMESS_DEBUG, "stat failed %s (%d)\n",
+	    rpmlog(RPMLOG_DEBUG, D_("stat failed %s (%d)\n"),
 		scp->pzErrmsg, rc);
 	}
     }
@@ -1516,7 +1575,6 @@ struct _dbiVec sqlitevec = {
 
 /*@=evalorderuncon@*/
 /*@=modfilesystem@*/
-/*@=branchstate@*/
 /*@=compmempass@*/
 /*@=compdef@*/
 /*@=moduncon@*/
@@ -1524,4 +1582,3 @@ struct _dbiVec sqlitevec = {
 /*@=globuse@*/
 /*@=paramuse@*/
 /*@=mustmod@*/
-/*@=bounds@*/
