@@ -50,8 +50,8 @@ static const union _dbswap {
     const uint32_t ui;
     const unsigned char uc[4];
 } endian = { .ui = 0x11223344 };
-# define HASH_LITTLE_ENDIAN	(endian.uc[0] == 0x44)
-# define HASH_BIG_ENDIAN	(endian.uc[0] == 0x11)
+# define HASH_LITTLE_ENDIAN	(endian.uc[0] == (unsigned char) 0x44)
+# define HASH_BIG_ENDIAN	(endian.uc[0] == (unsigned char) 0x11)
 /*@=redef@*/
 
 #ifndef ROTL32
