@@ -5,9 +5,7 @@
 
 #include "system.h"
 
-#if defined(HAVE_PTHREAD_H) && !defined(__LCLINT__)
-#include <pthread.h>
-#endif
+#ifdef WITH_NEON
 
 #include "ne_alloc.h"
 #include "ne_auth.h"
@@ -25,15 +23,41 @@
 #include "ne_request.h"
 #include "ne_socket.h"
 #include "ne_string.h"
-#include "ne_utils.h"
 
-/* XXX API changes for neon-0.26.0 */
-#if !defined(NE_FREE)
+#include "ne_utils.h"
+#if !defined(HEADER_ERR_H)
+/* cheats to avoid having to explicitly build against OpenSSL */
+/*@-exportheader@*/
+extern void ERR_remove_state(int foo);
+extern void ENGINE_cleanup(void);
+extern void CONF_modules_unload(int foo);
+extern void ERR_free_strings(void);
+extern void EVP_cleanup(void);
+extern void CRYPTO_cleanup_all_ex_data(void);
+extern void CRYPTO_mem_leaks(void * ptr);
+/*@=exportheader@*/
+#endif
+
+#include "ne_md5.h" /* for version detection only */
+
+/* poor-man's NEON version determination */
+#if defined(NE_MD5_H)
+#define WITH_NEON_MIN_VERSION 0x002700
+#elif defined(NE_FEATURE_I18N)
+#define WITH_NEON_MIN_VERSION 0x002600
+#else
+#define WITH_NEON_MIN_VERSION 0x002500
+#endif
+
+/* XXX API changes for NEON 0.26 */
+#if WITH_NEON_MIN_VERSION >= 0x002600
 #define	ne_set_persist(_sess, _flag)
 #define	ne_propfind_set_private(_pfh, _create_item, NULL) \
 	ne_propfind_set_private(_pfh, _create_item, NULL, NULL)
 #endif
 
+#endif /* WITH_NEON */
+
 #include <rpmio_internal.h>
 
 #define _RPMDAV_INTERNAL
@@ -55,20 +79,25 @@
 /*@unchecked@*/
 static int httpTimeoutSecs = TIMEOUT_SECS;
 
-/**
- * Wrapper to free(3), hides const compilation noise, permit NULL, return NULL.
- * @param p		memory to free
- * @retval		NULL always
- */
-/*@unused@*/ static inline /*@null@*/ void *
-_free(/*@only@*/ /*@null@*/ /*@out@*/ const void * p)
-	/*@modifies p@*/
-{
-    if (p != NULL)	free((void *)p);
-    return NULL;
-}
+#ifdef WITH_NEON
 
 /* =============================================================== */
+void davDestroy(void)
+{
+#ifdef NE_FEATURE_SSL
+    if (ne_has_support(NE_FEATURE_SSL)) {
+/* XXX http://www.nabble.com/Memory-Leaks-in-SSL_Library_init()-t3431875.html */
+	ENGINE_cleanup();
+	CRYPTO_cleanup_all_ex_data();
+	ERR_free_strings();
+	ERR_remove_state(0);
+	EVP_cleanup();
+	CRYPTO_mem_leaks(NULL);
+	CONF_modules_unload(1);
+    }
+#endif
+}
+
 int davFree(urlinfo u)
 	/*@globals internalState @*/
 	/*@modifies u, internalState @*/
@@ -104,7 +133,9 @@ static void davProgress(void * userdata, off_t current, off_t total)
 assert(u != NULL);
     sess = u->sess;
 assert(sess != NULL);
+/*@-sefuncon@*/
 assert(u == ne_get_session_private(sess, "urlinfo"));
+/*@=sefuncon@*/
 
     u->current = current;
     u->total = total;
@@ -113,8 +144,13 @@ if (_dav_debug < 0)
 fprintf(stderr, "*** davProgress(%p,0x%x:0x%x) sess %p u %p\n", userdata, (unsigned int)current, (unsigned int)total, sess, u);
 }
 
+#if WITH_NEON_MIN_VERSION >= 0x002700
+static void davNotify(void * userdata,
+		ne_session_status connstatus, const ne_session_status_info *info)
+#else
 static void davNotify(void * userdata,
 		ne_conn_status connstatus, const char * info)
+#endif
 	/*@*/
 {
     urlinfo u = userdata;
@@ -131,7 +167,9 @@ static void davNotify(void * userdata,
 assert(u != NULL);
     sess = u->sess;
 assert(sess != NULL);
+/*@-sefuncon@*/
 assert(u == ne_get_session_private(sess, "urlinfo"));
+/*@=sefuncon@*/
 
 #ifdef	REFERENCE
 typedef enum {
@@ -142,12 +180,12 @@ typedef enum {
 } ne_conn_status;
 #endif
 
+#if WITH_NEON_MIN_VERSION < 0x002700
     u->connstatus = connstatus;
+#endif
 
-/*@-boundsread@*/
 if (_dav_debug < 0)
 fprintf(stderr, "*** davNotify(%p,%d,%p) sess %p u %p %s\n", userdata, connstatus, info, sess, u, connstates[ (connstatus < 4 ? connstatus : 4)]);
-/*@=boundsread@*/
 
 }
 
@@ -165,7 +203,9 @@ assert(u->sess != NULL);
 assert(req != NULL);
     sess = ne_get_session(req);
 assert(sess == u->sess);
+/*@-sefuncon@*/
 assert(u == ne_get_session_private(sess, "urlinfo"));
+/*@=sefuncon@*/
 
 assert(sess != NULL);
     private = ne_get_session_private(sess, id);
@@ -182,14 +222,18 @@ static void davPreSend(ne_request * req, void * userdata, ne_buffer * buf)
     const char * id = "fd";
     FD_t fd = NULL;
 
+/*@-modunconnomods@*/
 assert(u != NULL);
 assert(u->sess != NULL);
 assert(req != NULL);
     sess = ne_get_session(req);
 assert(sess == u->sess);
+/*@-sefuncon@*/
 assert(u == ne_get_session_private(sess, "urlinfo"));
+/*@=sefuncon@*/
 
     fd = ne_get_request_private(req, id);
+/*@=modunconnomods@*/
 
 if (_dav_debug < 0)
 fprintf(stderr, "*** davPreSend(%p,%p,%p) sess %p %s %p\n", req, userdata, buf, sess, id, fd);
@@ -211,7 +255,9 @@ assert(u->sess != NULL);
 assert(req != NULL);
     sess = ne_get_session(req);
 assert(sess == u->sess);
+/*@-sefuncon@*/
 assert(u == ne_get_session_private(sess, "urlinfo"));
+/*@=sefuncon@*/
 
     fd = ne_get_request_private(req, id);
 
@@ -235,7 +281,9 @@ assert(u->sess != NULL);
 assert(req != NULL);
     sess = ne_get_session(req);
 assert(sess == u->sess);
+/*@-sefuncon@*/
 assert(u == ne_get_session_private(sess, "urlinfo"));
+/*@=sefuncon@*/
 
     fd = ne_get_request_private(req, id);
 
@@ -254,7 +302,9 @@ static void davDestroySession(void * userdata)
 assert(u != NULL);
 assert(u->sess != NULL);
     sess = u->sess;
+/*@-sefuncon@*/
 assert(u == ne_get_session_private(sess, "urlinfo"));
+/*@=sefuncon@*/
 
 assert(sess != NULL);
     private = ne_get_session_private(sess, id);
@@ -277,8 +327,8 @@ fprintf(stderr, "*** davVerifyCert(%p,%d,%p) %s\n", userdata, failures, cert, ho
 }
 
 static int davConnect(urlinfo u)
-	/*@globals internalState @*/
-	/*@modifies u, internalState @*/
+	/*@globals errno, internalState @*/
+	/*@modifies u, errno, internalState @*/
 {
     const char * path = NULL;
     int rc;
@@ -320,9 +370,11 @@ static int davConnect(urlinfo u)
     case NE_CONNECT:		/* HACK: errno set already? */
     default:
 bottom:
+/*@-evalorderuncon@*/
 if (_dav_debug)
 fprintf(stderr, "*** Connect to %s:%d failed(%d):\n\t%s\n",
 		   u->host, u->port, rc, ne_get_error(u->sess));
+/*@=evalorderuncon@*/
 	break;
     }
 
@@ -379,7 +431,11 @@ static int davInit(const char * url, urlinfo * uret)
 #endif
 
 	ne_set_progress(u->sess, davProgress, u);
+#if WITH_NEON_MIN_VERSION >= 0x002700
+	ne_set_notifier(u->sess, davNotify, u);
+#else
 	ne_set_status(u->sess, davNotify, u);
+#endif
 
 	ne_set_persist(u->sess, 1);
 	ne_set_read_timeout(u->sess, httpTimeoutSecs);
@@ -406,10 +462,8 @@ static int davInit(const char * url, urlinfo * uret)
     }
 
 exit:
-/*@-boundswrite@*/
     if (uret != NULL)
-	*uret = urlLink(u, __FUNCTION__);
-/*@=boundswrite@*/
+	*uret = urlLink(u, "davInit");
     u = urlFree(u, "urlSplit (davInit)");
 
     return rc;
@@ -454,17 +508,19 @@ static void *fetch_destroy_list(/*@only@*/ struct fetch_resource_s *res)
 	/*@modifies res @*/
 {
     struct fetch_resource_s *next;
-/*@-branchstate@*/
     for (; res != NULL; res = next) {
 	next = res->next;
 	res = fetch_destroy_item(res);
     }
-/*@=branchstate@*/
     return NULL;
 }
 #endif
 
+#if WITH_NEON_MIN_VERSION >= 0x002600
+static void *fetch_create_item(/*@unused@*/ void *userdata, /*@unused@*/ const ne_uri *uri)
+#else
 static void *fetch_create_item(/*@unused@*/ void *userdata, /*@unused@*/ const char *uri)
+#endif
         /*@*/
 {
     struct fetch_resource_s * res = ne_calloc(sizeof(*res));
@@ -501,11 +557,9 @@ static void *fetch_destroy_context(/*@only@*/ /*@null@*/ struct fetch_context_s
     ctx->modes = _free(ctx->modes);
     ctx->sizes = _free(ctx->sizes);
     ctx->mtimes = _free(ctx->mtimes);
-    ctx->u = urlFree(ctx->u, __FUNCTION__);
+    ctx->u = urlFree(ctx->u, "fetch_destroy_context");
     ctx->uri = _free(ctx->uri);
-/*@-boundswrite@*/
     memset(ctx, 0, sizeof(*ctx));
-/*@=boundswrite@*/
     ctx = _free(ctx);
     return NULL;
 }
@@ -513,7 +567,7 @@ static void *fetch_destroy_context(/*@only@*/ /*@null@*/ struct fetch_context_s
 /*@null@*/
 static void *fetch_create_context(const char *uri, /*@null@*/ struct stat *st)
 	/*@globals internalState @*/
-	/*@modifies internalState @*/
+	/*@modifies *st, internalState @*/
 {
     struct fetch_context_s * ctx;
     urlinfo u;
@@ -525,12 +579,15 @@ static void *fetch_create_context(const char *uri, /*@null@*/ struct stat *st)
 
     ctx = ne_calloc(sizeof(*ctx));
     ctx->uri = xstrdup(uri);
-    ctx->u = urlLink(u, __FUNCTION__);
+    ctx->u = urlLink(u, "fetch_create_context");
+/*@-temptrans@*/	/* XXX note the assignment */
     if ((ctx->st = st) != NULL)
 	memset(ctx->st, 0, sizeof(*ctx->st));
+/*@=temptrans@*/
     return ctx;
 }
 
+/*@-readonlytrans@*/
 /*@unchecked@*/ /*@observer@*/
 static const ne_propname fetch_props[] = {
     { "DAV:", "getcontentlength" },
@@ -541,15 +598,18 @@ static const ne_propname fetch_props[] = {
     { "DAV:", "checked-out" },
     { NULL, NULL }
 };
+/*@=readonlytrans@*/
 
 #define ELM_resourcetype (NE_PROPS_STATE_TOP + 1)
 #define ELM_collection (NE_PROPS_STATE_TOP + 2)
 
+/*@-readonlytrans@*/
 /*@unchecked@*/ /*@observer@*/
 static const struct ne_xml_idmap fetch_idmap[] = {
     { "DAV:", "resourcetype", ELM_resourcetype },
     { "DAV:", "collection", ELM_collection }
 };
+/*@=readonlytrans@*/
 
 static int fetch_startelm(void *userdata, int parent,
 		const char *nspace, const char *name,
@@ -558,8 +618,10 @@ static int fetch_startelm(void *userdata, int parent,
 {
     ne_propfind_handler *pfh = userdata;
     struct fetch_resource_s *r = ne_propfind_current_private(pfh);
+/*@-sizeoftype@*/
     int state = ne_xml_mapid(fetch_idmap, NE_XML_MAPLEN(fetch_idmap),
                              nspace, name);
+/*@=sizeoftype@*/
 
     if (r == NULL ||
         !((parent == NE_207_STATE_PROP && state == ELM_resourcetype) ||
@@ -597,8 +659,13 @@ static int fetch_compare(const struct fetch_resource_s *r1,
     }
 }
 
+#if WITH_NEON_MIN_VERSION >= 0x002600
+static void fetch_results(void *userdata, const ne_uri *uarg,
+		    const ne_prop_result_set *set)
+#else
 static void fetch_results(void *userdata, void *uarg,
 		    const ne_prop_result_set *set)
+#endif
 	/*@*/
 {
     struct fetch_context_s *ctx = userdata;
@@ -608,7 +675,7 @@ static void fetch_results(void *userdata, void *uarg,
     const ne_status *status = NULL;
     const char * path = NULL;
 
-#if !defined(NE_FREE)
+#if WITH_NEON_MIN_VERSION >= 0x002600
     const ne_uri * uri = uarg;
     (void) urlPath(uri->path, &path);
 #else
@@ -636,20 +703,16 @@ fprintf(stderr, "==> %s skipping target resource.\n", path);
 
     newres->uri = ne_strdup(path);
 
-/*@-boundsread@*/
     clength = ne_propset_value(set, &fetch_props[0]);
     modtime = ne_propset_value(set, &fetch_props[1]);
     isexec = ne_propset_value(set, &fetch_props[2]);
     checkin = ne_propset_value(set, &fetch_props[4]);
     checkout = ne_propset_value(set, &fetch_props[5]);
-/*@=boundsread@*/
 
-/*@-branchstate@*/
     if (clength == NULL)
 	status = ne_propset_status(set, &fetch_props[0]);
     if (modtime == NULL)
 	status = ne_propset_status(set, &fetch_props[1]);
-/*@=branchstate@*/
 
     if (newres->type == resr_normal && status != NULL) {
 	/* It's an error! */
@@ -705,9 +768,9 @@ fprintf(stderr, "==> %s skipping target resource.\n", path);
     if (previous) {
 	previous->next = newres;
     } else {
-/*@-boundswrite -dependenttrans @*/
+/*@-dependenttrans @*/
 	*ctx->resrock = newres;
-/*@=boundswrite =dependenttrans @*/
+/*@=dependenttrans @*/
     }
     newres->next = current;
 }
@@ -800,11 +863,9 @@ fprintf(stderr, "*** argvAdd(%p,\"%s\")\n", &ctx->av, val);
 	    st_mode = 0;
 	    /*@switchbreak@*/ break;
 	}
-/*@-boundswrite@*/
 	ctx->modes[ctx->ac] = st_mode;
 	ctx->sizes[ctx->ac] = current->size;
 	ctx->mtimes[ctx->ac] = current->modtime;
-/*@=boundswrite@*/
 	ctx->ac++;
 
 	current = fetch_destroy_item(current);
@@ -837,7 +898,7 @@ static int davHEAD(urlinfo u, struct stat *st)
     switch (rc) {
     default:
 	goto exit;
-	/*@notreached@*/
+	/*@notreached@*/ break;
     case NE_OK:
 	if (ne_get_status(req)->klass != 2) {
 	    rc = NE_ERROR;
@@ -859,7 +920,9 @@ static int davHEAD(urlinfo u, struct stat *st)
     value = ne_get_response_header(req, htag); 
 #endif
     if (value) {
+/*@-unrecog@*/	/* XXX LCLINT needs stdlib.h update. */
 	st->st_size = strtoll(value, NULL, 10);
+/*@=unrecog@*/
 	st->st_blocks = (st->st_size + 511)/512;
     }
 
@@ -892,8 +955,11 @@ static int davNLST(struct fetch_context_s * ctx)
 /* HACK do PROPFIND through davFetch iff enabled, otherwise HEAD Content-length/ETag/Last-Modified */
     if (u->allow & RPMURL_SERVER_HASDAV)
 	   rc = davFetch(u, ctx);	/* use PROPFIND to get contentLength */
-    else
+    else {
+/*@-nullpass@*/	/* XXX annotate ctx->st correctly */
 	   rc = davHEAD(u, ctx->st);	/* use HEAD to get contentLength */
+/*@=nullpass@*/
+    }
 
     switch (rc) {
     case NE_OK:
@@ -907,9 +973,11 @@ static int davNLST(struct fetch_context_s * ctx)
 	    break;
 	/*@fallthrough@*/
     default:
+/*@-evalorderuncon@*/
 if (_dav_debug)
 fprintf(stderr, "*** Fetch from %s:%d failed:\n\t%s\n",
 		   u->host, u->port, ne_get_error(u->sess));
+/*@=evalorderuncon@*/
         break;
     }
 
@@ -1137,7 +1205,7 @@ FD_t davOpen(const char * url, /*@unused@*/ int flags,
 #endif
 
 if (_dav_debug < 0)
-fprintf(stderr, "*** davOpen(%s,0x%x,0%o,%p)\n", url, flags, mode, uret);
+fprintf(stderr, "*** davOpen(%s,0x%x,0%o,%p)\n", url, flags, (unsigned)mode, uret);
     rc = davInit(url, &u);
     if (rc || u == NULL || u->sess == NULL)
 	goto exit;
@@ -1168,10 +1236,8 @@ assert(urlType == URL_IS_HTTPS || urlType == URL_IS_HTTP || urlType == URL_IS_HK
     }
 
 exit:
-/*@-boundswrite@*/
     if (uret)
 	*uret = u;
-/*@=boundswrite@*/
     /*@-refcounttrans@*/
     return fd;
     /*@=refcounttrans@*/
@@ -1199,9 +1265,11 @@ hexdump(buf, rc);
 
 ssize_t davWrite(void * cookie, const char * buf, size_t count)
 {
+#if !defined(NEONBLOWSCHUNKS) || defined(HAVE_NEON_NE_SEND_REQUEST_CHUNK) || defined(__LCLINT__)
     FD_t fd = cookie;
+#endif
     ssize_t rc;
-    int xx;
+    int xx = -1;
 
 #if !defined(NEONBLOWSCHUNKS)
     ne_session * sess;
@@ -1215,7 +1283,9 @@ assert(sess != NULL);
 #else
 #if defined(HAVE_NEON_NE_SEND_REQUEST_CHUNK) || defined(__LCLINT__)
 assert(fd->req != NULL);
+/*@-unrecog@*/
     xx = ne_send_request_chunk(fd->req, buf, count);
+/*@=unrecog@*/
 #else
     errno = EIO;       /* HACK */
     return -1;
@@ -1284,7 +1354,7 @@ int davMkdir(const char * path, mode_t mode)
 
 exit:
 if (_dav_debug)
-fprintf(stderr, "*** davMkdir(%s,0%o) rc %d\n", path, mode, rc);
+fprintf(stderr, "*** davMkdir(%s,0%o) rc %d\n", path, (unsigned)mode, rc);
     return rc;
 }
 
@@ -1382,19 +1452,18 @@ static const char * statstr(const struct stat * st,
 	"*** dev %x ino %x mode %0o nlink %d uid %d gid %d rdev %x size %x\n",
 	(unsigned)st->st_dev,
 	(unsigned)st->st_ino,
-	st->st_mode,
+	(unsigned)st->st_mode,
 	(unsigned)st->st_nlink,
-	st->st_uid,
-	st->st_gid,
+	(unsigned)st->st_uid,
+	(unsigned)st->st_gid,
 	(unsigned)st->st_rdev,
 	(unsigned)st->st_size);
     return buf;
 }
 
 /*@unchecked@*/
-static int dav_st_ino = 0xdead0000;
+static unsigned int dav_st_ino = 0xdead0000;
 
-/*@-boundswrite@*/
 int davStat(const char * path, /*@out@*/ struct stat *st)
 	/*@globals dav_st_ino, fileSystem, internalState @*/
 	/*@modifies *st, dav_st_ino, fileSystem, internalState @*/
@@ -1406,7 +1475,7 @@ int davStat(const char * path, /*@out@*/ struct stat *st)
 /* HACK: neon really wants collections with trailing '/' */
     ctx = fetch_create_context(path, st);
     if (ctx == NULL) {
-fprintf(stderr, "==> %s fetch_create_context ctx %p\n", __FUNCTION__, ctx);
+fprintf(stderr, "==> %s fetch_create_context ctx %p\n", "davStat", ctx);
 /* HACK: errno = ??? */
 	goto exit;
     }
@@ -1440,9 +1509,7 @@ fprintf(stderr, "*** davStat(%s) rc %d\n%s", path, rc, statstr(st, buf));
     ctx = fetch_destroy_context(ctx);
     return rc;
 }
-/*@=boundswrite@*/
 
-/*@-boundswrite@*/
 int davLstat(const char * path, /*@out@*/ struct stat *st)
 	/*@globals dav_st_ino, fileSystem, internalState @*/
 	/*@modifies *st, dav_st_ino, fileSystem, internalState @*/
@@ -1486,7 +1553,6 @@ exit:
     ctx = fetch_destroy_context(ctx);
     return rc;
 }
-/*@=boundswrite@*/
 
 #ifdef	NOTYET
 static int davReadlink(const char * path, /*@out@*/ char * buf, size_t bufsiz)
@@ -1501,10 +1567,60 @@ fprintf(stderr, "*** davReadlink(%s) rc %d\n", path, rc);
 }
 #endif	/* NOTYET */
 
+#endif /* WITH_NEON */
+
 /* =============================================================== */
 /*@unchecked@*/
 int avmagicdir = 0x3607113;
 
+#ifndef WITH_NEON
+/*@-nullstate@*/        /* FIX: u->{ctrl,data}->url undef after XurlLink. */
+FD_t httpOpen(const char * url, /*@unused@*/ int flags,
+                /*@unused@*/ mode_t mode, /*@out@*/ urlinfo * uret)
+        /*@globals internalState @*/
+        /*@modifies *uret, internalState @*/
+{
+    urlinfo u = NULL;
+    FD_t fd = NULL;
+
+#if 0   /* XXX makeTempFile() heartburn */
+    assert(!(flags & O_RDWR));
+#endif
+    if (urlSplit(url, &u))
+        goto exit;
+
+    if (u->ctrl == NULL)
+        u->ctrl = fdNew("persist ctrl (httpOpen)");
+    if (u->ctrl->nrefs > 2 && u->data == NULL)
+        u->data = fdNew("persist data (httpOpen)");
+
+    if (u->ctrl->url == NULL)
+        fd = fdLink(u->ctrl, "grab ctrl (httpOpen persist ctrl)");
+    else if (u->data->url == NULL)
+        fd = fdLink(u->data, "grab ctrl (httpOpen persist data)");
+    else
+        fd = fdNew("grab ctrl (httpOpen)");
+
+    if (fd) {
+        fdSetIo(fd, ufdio);
+        fd->ftpFileDoneNeeded = 0;
+        fd->rd_timeoutsecs = httpTimeoutSecs;
+        fd->contentLength = fd->bytesRemain = -1;
+        fd->url = urlLink(u, "url (httpOpen)");
+        fd = fdLink(fd, "grab data (httpOpen)");
+        fd->urlType = URL_IS_HTTP;
+    }
+
+exit:
+    if (uret)
+        *uret = u;
+    /*@-refcounttrans@*/
+    return fd;
+    /*@=refcounttrans@*/
+}
+/*@=nullstate@*/
+#endif
+
 int avClosedir(/*@only@*/ DIR * dir)
 {
     AVDIR avdir = (AVDIR)dir;
@@ -1538,29 +1654,27 @@ struct dirent * avReaddir(DIR * dir)
 
     dp = (struct dirent *) avdir->data;
     av = (const char **) (dp + 1);
-    ac = avdir->size;
-    dt = (char *) (av + (ac + 1));
+    ac = (int)avdir->size;
+    dt = (unsigned char *) (av + (ac + 1));
     i = avdir->offset + 1;
 
-/*@-boundsread@*/
     if (i < 0 || i >= ac || av[i] == NULL)
 	return NULL;
-/*@=boundsread@*/
 
     avdir->offset = i;
 
     /* XXX glob(3) uses REAL_DIR_ENTRY(dp) test on d_ino */
 /*@-type@*/
     dp->d_ino = i + 1;		/* W2DO? */
+#if !defined(__DragonFly__) && !defined(__CYGWIN__)
     dp->d_reclen = 0;		/* W2DO? */
+#endif
 
-#if !(defined(hpux) || defined(__hpux) || defined(sun))
-#if !defined(__APPLE__) && !defined(__FreeBSD_kernel__) && !defined(__FreeBSD__)
+#if !(defined(hpux) || defined(__hpux) || defined(sun) || defined(RPM_OS_AIX) || defined(__CYGWIN__) || defined(__QNXNTO__))
+#if !defined(__APPLE__) && !defined(__FreeBSD_kernel__) && !defined(__FreeBSD__) && !defined(__NetBSD__) && !defined(__DragonFly__)
     dp->d_off = 0;		/* W2DO? */
 #endif
-/*@-boundsread@*/
     dp->d_type = dt[i];
-/*@=boundsread@*/
 #endif
 /*@=type@*/
 
@@ -1571,7 +1685,6 @@ fprintf(stderr, "*** avReaddir(%p) %p \"%s\"\n", (void *)avdir, dp, dp->d_name);
     return dp;
 }
 
-/*@-boundswrite@*/
 DIR * avOpendir(const char * path)
 {
     AVDIR avdir;
@@ -1592,7 +1705,7 @@ fprintf(stderr, "*** avOpendir(%s)\n", path);
 /*@-abstract@*/
     dp = (struct dirent *) (avdir + 1);
     av = (const char **) (dp + 1);
-    dt = (char *) (av + (ac + 1));
+    dt = (unsigned char *) (av + (ac + 1));
     t = (char *) (dt + ac + 1);
 /*@=abstract@*/
 
@@ -1613,8 +1726,8 @@ fprintf(stderr, "*** avOpendir(%s)\n", path);
 
     ac = 0;
     /*@-dependenttrans -unrecog@*/
-    dt[ac] = DT_DIR;	av[ac++] = t;	t = stpcpy(t, ".");	t++;
-    dt[ac] = DT_DIR;	av[ac++] = t;	t = stpcpy(t, "..");	t++;
+    dt[ac] = (unsigned char)DT_DIR; av[ac++] = t; t = stpcpy(t, ".");	t++;
+    dt[ac] = (unsigned char)DT_DIR; av[ac++] = t; t = stpcpy(t, "..");	t++;
     /*@=dependenttrans =unrecog@*/
 
     av[ac] = NULL;
@@ -1623,7 +1736,8 @@ fprintf(stderr, "*** avOpendir(%s)\n", path);
     return (DIR *) avdir;
 /*@=kepttrans@*/
 }
-/*@=boundswrite@*/
+
+#ifdef WITH_NEON
 
 /* =============================================================== */
 /*@unchecked@*/
@@ -1662,29 +1776,27 @@ struct dirent * davReaddir(DIR * dir)
 
     dp = (struct dirent *) avdir->data;
     av = (const char **) (dp + 1);
-    ac = avdir->size;
-    dt = (char *) (av + (ac + 1));
+    ac = (int)avdir->size;
+    dt = (unsigned char *) (av + (ac + 1));
     i = avdir->offset + 1;
 
-/*@-boundsread@*/
     if (i < 0 || i >= ac || av[i] == NULL)
 	return NULL;
-/*@=boundsread@*/
 
     avdir->offset = i;
 
     /* XXX glob(3) uses REAL_DIR_ENTRY(dp) test on d_ino */
 /*@-type@*/
     dp->d_ino = i + 1;		/* W2DO? */
+#if !defined(__DragonFly__) && !defined(__CYGWIN__)
     dp->d_reclen = 0;		/* W2DO? */
+#endif
 
-#if !(defined(hpux) || defined(__hpux) || defined(sun))
-#if !defined(__APPLE__) && !defined(__FreeBSD_kernel__)
+#if !(defined(hpux) || defined(__hpux) || defined(sun) || defined(RPM_OS_AIX) || defined(__CYGWIN__))
+#if !defined(__APPLE__) && !defined(__FreeBSD_kernel__) && !defined(__FreeBSD__) && !defined(__NetBSD__) && !defined(__DragonFly__)
     dp->d_off = 0;		/* W2DO? */
 #endif
-/*@-boundsread@*/
     dp->d_type = dt[i];
-/*@=boundsread@*/
 #endif
 /*@=type@*/
 
@@ -1695,7 +1807,6 @@ fprintf(stderr, "*** davReaddir(%p) %p \"%s\"\n", (void *)avdir, dp, dp->d_name)
     return dp;
 }
 
-/*@-boundswrite@*/
 DIR * davOpendir(const char * path)
 {
     struct fetch_context_s * ctx;
@@ -1710,14 +1821,12 @@ DIR * davOpendir(const char * path)
 
     /* HACK: glob does not pass dirs with trailing '/' */
     nb = strlen(path)+1;
-/*@-branchstate@*/
     if (path[nb-1] != '/') {
 	char * npath = alloca(nb+1);
 	*npath = '\0';
 	(void) stpcpy( stpcpy(npath, path), "/");
 	path = npath;
     }
-/*@=branchstate@*/
 
 if (_dav_debug < 0)
 fprintf(stderr, "*** davOpendir(%s)\n", path);
@@ -1748,7 +1857,7 @@ fprintf(stderr, "*** davOpendir(%s)\n", path);
     /*@-abstract@*/
     dp = (struct dirent *) (avdir + 1);
     nav = (const char **) (dp + 1);
-    dt = (char *) (nav + (ac + 1));
+    dt = (unsigned char *) (nav + (ac + 1));
     t = (char *) (dt + ac + 1);
     /*@=abstract@*/
 
@@ -1769,8 +1878,8 @@ fprintf(stderr, "*** davOpendir(%s)\n", path);
 
     nac = 0;
 /*@-dependenttrans -unrecog@*/
-    dt[nac] = DT_DIR;	nav[nac++] = t;	t = stpcpy(t, ".");	t++;
-    dt[nac] = DT_DIR;	nav[nac++] = t;	t = stpcpy(t, "..");	t++;
+    dt[nac] = (unsigned char)DT_DIR; nav[nac++] = t; t = stpcpy(t, ".");  t++;
+    dt[nac] = (unsigned char)DT_DIR; nav[nac++] = t; t = stpcpy(t, ".."); t++;
 /*@=dependenttrans =unrecog@*/
 
     /* Copy DAV items into DIR elments. */
@@ -1778,7 +1887,7 @@ fprintf(stderr, "*** davOpendir(%s)\n", path);
     if (av != NULL)
     while (av[ac] != NULL) {
 	nav[nac] = t;
-	dt[nac] = (S_ISDIR(ctx->modes[ac]) ? DT_DIR : DT_REG);
+	dt[nac] = (unsigned char) (S_ISDIR(ctx->modes[ac]) ? DT_DIR : DT_REG);
 	t = stpcpy(t, av[ac]);
 	ac++;
 	t++;
@@ -1793,3 +1902,5 @@ fprintf(stderr, "*** davOpendir(%s)\n", path);
 /*@=kepttrans@*/
 }
 /*@=modfilesys@*/
+
+#endif /* WITH_NEON */
