@@ -5,8 +5,7 @@
 #include <string.h>
 #include <stdarg.h>
 
-#include "rpmio/rpmhook.h"
-
+#include <rpmhook.h>
 #include "debug.h"
 
 #define RPMHOOK_TABLE_INITSIZE  256
@@ -20,6 +19,7 @@ typedef struct rpmhookItem_s {
 
 typedef struct rpmhookBucket_s {
     unsigned long hash;
+/*@relnull@*/
     char *name;
     rpmhookItem item;
 } * rpmhookBucket;
@@ -46,7 +46,9 @@ rpmhookArgs rpmhookArgsFree(rpmhookArgs args)
     return NULL;
 }
 
+/*@only@*/
 static rpmhookTable rpmhookTableNew(int size)
+	/*@*/
 {
     rpmhookTable table = (rpmhookTable) xcalloc(1,
 		sizeof(*table) + sizeof(table->bucket) * (size-1));
@@ -56,6 +58,7 @@ static rpmhookTable rpmhookTableNew(int size)
 
 #if 0
 static rpmhookTable rpmhookTableFree(rpmhookTable table)
+	/*@*/
 {
     rpmhookItem item, nextItem;
     int i;
@@ -75,9 +78,11 @@ static rpmhookTable rpmhookTableFree(rpmhookTable table)
 }
 #endif
 
-static void rpmhookTableRehash(rpmhookTable *table);
+static void rpmhookTableRehash(rpmhookTable *table)
+	/*@modifies *table @*/;
 
 static int rpmhookTableFindBucket(rpmhookTable *table, const char *name)
+	/*@modifies *table @*/
 {
     /* Hash based on http://www.isthe.com/chongo/tech/comp/fnv/ */
     unsigned long perturb;
@@ -99,7 +104,9 @@ static int rpmhookTableFindBucket(rpmhookTable *table, const char *name)
     while (bucket->name &&
 	    (bucket->hash != hash || strcmp(bucket->name, name) != 0)) {
 	/* Collision resolution based on Python's perturb scheme. */
+/*@-shiftimplementation@*/
 	ret = ((ret << 2) + ret + perturb + 1) % (*table)->size;
+/*@=shiftimplementation@*/
 	perturb >>= 5;
 	bucket = &(*table)->bucket[ret];
     }
@@ -109,6 +116,7 @@ static int rpmhookTableFindBucket(rpmhookTable *table, const char *name)
 }
 
 static void rpmhookTableRehash(rpmhookTable *table)
+	/*@modifies *table @*/
 {
     rpmhookTable newtable = rpmhookTableNew((*table)->size*2);
     int n, i = 0;
@@ -121,29 +129,35 @@ static void rpmhookTableRehash(rpmhookTable *table)
 	newtable->bucket[n].item = (*table)->bucket[i].item;
     }
     newtable->used = (*table)->used;
+/*@-unqualifiedtrans@*/
     free(*table);
+/*@=unqualifiedtrans@*/
     *table = newtable;
 }
 
 static void rpmhookTableAddItem(rpmhookTable *table, const char *name,
 				rpmhookFunc func, void *data)
+	/*@modifies *table @*/
 {
     int n = rpmhookTableFindBucket(table, name);
     rpmhookBucket bucket = &(*table)->bucket[n];
     rpmhookItem *item = &bucket->item;
     if (!bucket->name) {
-	bucket->name = xstrdup(name);
+	bucket->name = strdup(name);
 	(*table)->used++;
     }
     while (*item) item = &(*item)->next;
     *item = xcalloc(1, sizeof(**item));
     (*item)->func = func;
+/*@-temptrans@*/
     (*item)->data = data;
+/*@=temptrans@*/
 }
 
 static void rpmhookTableDelItem(rpmhookTable *table, const char *name,
-		rpmhookFunc func, void *data,
+		/*@null@*/ rpmhookFunc func, /*@null@*/ void *data,
 		int matchfunc, int matchdata)
+	/*@modifies *table @*/
 {
     int n = rpmhookTableFindBucket(table, name);
     rpmhookBucket bucket = &(*table)->bucket[n];
@@ -162,6 +176,7 @@ static void rpmhookTableDelItem(rpmhookTable *table, const char *name,
 	} else {
 	    lastItem = item;
 	}
+/*@-usereleased@*/
 	item = nextItem;
     }
     if (!bucket->item) {
@@ -169,32 +184,38 @@ static void rpmhookTableDelItem(rpmhookTable *table, const char *name,
 	bucket->name = NULL;
 	(*table)->used--;
     }
+/*@=usereleased@*/
 }
 
 static rpmhookArgs rpmhookArgsParse(const char *argt, va_list ap)
+	/*@*/
 {
-    rpmhookArgs args = rpmhookArgsNew(strlen(argt));
+    rpmhookArgs args = rpmhookArgsNew((int)strlen(argt));
     int i;
 
+/*@-temptrans@*/
     args->argt = argt;
+/*@=temptrans@*/
     for (i = 0; i != args->argc; i++) {
 	switch (argt[i]) {
 	    case 's':
 		args->argv[i].s = va_arg(ap, char *);
-		break;
+		/*@switchbreak@*/ break;
 	    case 'i':
 		args->argv[i].i = va_arg(ap, int);
-		break;
+		/*@switchbreak@*/ break;
 	    case 'f':
 		args->argv[i].f = (float)va_arg(ap, double);
-		break;
+		/*@switchbreak@*/ break;
 	    case 'p':
 		args->argv[i].p = va_arg(ap, void *);
-		break;
+		/*@switchbreak@*/ break;
 	    default:
+/*@-modfilesys @*/
 		fprintf(stderr, "error: unsupported type '%c' as "
 				"a hook argument\n", argt[i]);
-		break;
+/*@=modfilesys @*/
+		/*@switchbreak@*/ break;
 	}
     }
     return args;
@@ -202,21 +223,23 @@ static rpmhookArgs rpmhookArgsParse(const char *argt, va_list ap)
 
 static void rpmhookTableCallArgs(rpmhookTable *table, const char *name,
 			rpmhookArgs args)
+	/*@modifies *table @*/
 {
     int n = rpmhookTableFindBucket(table, name);
     rpmhookItem item = (*table)->bucket[n].item;
-    rpmhookItem next;
     while (item) {
-	next = item->next;
 	if (item->func(args, item->data) != 0)
 	    break;
-	item = next;
+	item = item->next;
     }
 }
 
+/*@unchecked@*/ /*@only@*/ /*@null@*/
 static rpmhookTable globalTable = NULL;
 
 void rpmhookRegister(const char *name, rpmhookFunc func, void *data)
+	/*@globals globalTable @*/
+	/*@modifies globalTable @*/
 {
     if (globalTable == NULL)
 	globalTable = rpmhookTableNew(RPMHOOK_TABLE_INITSIZE);
@@ -248,7 +271,9 @@ void rpmhookCall(const char *name, const char *argt, ...)
 	va_list ap;
 	va_start(ap, argt);
 	args = rpmhookArgsParse(argt, ap);
+/*@-noeffect@*/
 	rpmhookTableCallArgs(&globalTable, name, args);
+/*@=noeffect@*/
 	(void) rpmhookArgsFree(args);
 	va_end(ap);
     }
@@ -256,6 +281,8 @@ void rpmhookCall(const char *name, const char *argt, ...)
 
 void rpmhookCallArgs(const char *name, rpmhookArgs args)
 {
+/*@-noeffect@*/
     if (globalTable != NULL)
 	rpmhookTableCallArgs(&globalTable, name, args);
+/*@=noeffect@*/
 }
