@@ -16,14 +16,6 @@
 #include <stdlib.h>
 #include <unistd.h>
 
-/**
- */
-typedef /*@abstract@*/ struct pgpDig_s * pgpDig;
-
-/**
- */
-typedef /*@abstract@*/ struct pgpDigParams_s * pgpDigParams;
-
 /** \ingroup rpmio
  * Hide libio API lossage.
  * The libio interface changed after glibc-2.1.3 to pass the seek offset
@@ -342,11 +334,22 @@ int Rmdir(const char * path)
 	/*@globals errno, h_errno, fileSystem, internalState @*/
 	/*@modifies errno, fileSystem, internalState @*/;
 
+/*@unchecked@*/ /*@observer@*/ /*@null@*/
+extern const char * _chroot_prefix;
+
 /**
  * chroot(2) clone.
  * @todo Implement remotely.
  */
 int Chroot(const char * path)
+	/*@globals _chroot_prefix, errno, fileSystem, internalState @*/
+	/*@modifies _chroot_prefix, errno, fileSystem, internalState @*/;
+
+/**
+ * open(2) clone.
+ * @todo Implement remotely.
+ */
+int Open(const char * path, int flags, mode_t mode)
 	/*@globals errno, fileSystem, internalState @*/
 	/*@modifies errno, fileSystem, internalState @*/;
 
@@ -429,6 +432,7 @@ int Mknod(const char * path, mode_t mode, dev_t dev)
  * utime(2) clone.
  * @todo Implement remotely.
  */
+struct utimbuf;
 int Utime(const char * path, const struct utimbuf * buf)
 	/*@globals errno, fileSystem, internalState @*/
 	/*@modifies errno, fileSystem, internalState @*/;
@@ -469,6 +473,31 @@ int Access(const char * path, int amode)
 	/*@globals errno, fileSystem @*/
 	/*@modifies errno, fileSystem @*/;
 
+#if defined(__linux__)
+/**
+ * mount(2) clone.
+ */
+int Mount(const char *source, const char *target,
+		const char *filesystemtype, unsigned long mountflags,
+		const void *data)
+	/*@globals errno, fileSystem @*/
+	/*@modifies errno, fileSystem @*/;
+
+/**
+ * umount(2) clone.
+ */
+int Umount(const char *target)
+	/*@globals errno, fileSystem @*/
+	/*@modifies errno, fileSystem @*/;
+
+/**
+ * umount2(2) clone.
+ */
+int Umount2(const char *target, int flags)
+	/*@globals errno, fileSystem @*/
+	/*@modifies errno, fileSystem @*/;
+#endif
+
 /**
  * glob_pattern_p(3) clone.
  */
@@ -659,6 +688,18 @@ int rpmioAccess(const char *FN, /*@null@*/ const char * path, int mode)
 	/*@globals fileSystem, internalState @*/
 	/*@modifies fileSystem, internalState @*/;
 
+/**
+ * Return a password.
+ * @param prompt	prompt string
+ * @return		password
+ */
+extern char * (*Getpass) (const char * prompt)
+	/*@*/;
+char * _GetPass (const char * prompt)
+	/*@*/;
+char * _RequestPass (const char * prompt)
+	/*@*/;
+
 /**
  * FTP and HTTP error codes.
  */
@@ -725,13 +766,6 @@ int ufdGetFile( /*@killref@*/ FD_t sfd, FD_t tfd)
 	/*@globals fileSystem, internalState @*/
 	/*@modifies sfd, tfd, fileSystem, internalState @*/;
 
-/**
- */
-/*@unused@*/ int timedRead(FD_t fd, /*@out@*/ void * bufptr, int length)
-	/*@globals fileSystem @*/
-	/*@modifies fd, *bufptr, fileSystem @*/;
-#define	timedRead	(ufdio->read)
-
 /*@-exportlocal@*/
 /**
  */
@@ -764,37 +798,37 @@ int ufdGetFile( /*@killref@*/ FD_t sfd, FD_t tfd)
 /*@}*/
 
 /*@unused@*/ static inline int xislower(int c) /*@*/ {
-    return (c >= 'a' && c <= 'z');
+    return (c >= (int)'a' && c <= (int)'z');
 }
 /*@unused@*/ static inline int xisupper(int c) /*@*/ {
-    return (c >= 'A' && c <= 'Z');
+    return (c >= (int)'A' && c <= (int)'Z');
 }
 /*@unused@*/ static inline int xisalpha(int c) /*@*/ {
     return (xislower(c) || xisupper(c));
 }
 /*@unused@*/ static inline int xisdigit(int c) /*@*/ {
-    return (c >= '0' && c <= '9');
+    return (c >= (int)'0' && c <= (int)'9');
 }
 /*@unused@*/ static inline int xisalnum(int c) /*@*/ {
     return (xisalpha(c) || xisdigit(c));
 }
 /*@unused@*/ static inline int xisblank(int c) /*@*/ {
-    return (c == ' ' || c == '\t');
+    return (c == (int)' ' || c == (int)'\t');
 }
 /*@unused@*/ static inline int xisspace(int c) /*@*/ {
-    return (xisblank(c) || c == '\n' || c == '\r' || c == '\f' || c == '\v');
+    return (xisblank(c) || c == (int)'\n' || c == (int)'\r' || c == (int)'\f' || c == (int)'\v');
 }
 /*@unused@*/ static inline int xiscntrl(int c) /*@*/ {
-    return (c < ' ');
+    return (c < (int)' ');
 }
 /*@unused@*/ static inline int xisascii(int c) /*@*/ {
     return ((c & 0x80) != 0x80);
 }
 /*@unused@*/ static inline int xisprint(int c) /*@*/ {
-    return (c >= ' ' && xisascii(c));
+    return (c >= (int)' ' && xisascii(c));
 }
 /*@unused@*/ static inline int xisgraph(int c) /*@*/ {
-    return (c > ' ' && xisascii(c));
+    return (c > (int)' ' && xisascii(c));
 }
 /*@unused@*/ static inline int xispunct(int c) /*@*/ {
     return (xisgraph(c) && !xisalnum(c));
@@ -823,6 +857,33 @@ int xstrncasecmp(const char *s1, const char * s2, size_t n)	/*@*/;
 /*@only@*/ /*@null@*/
 const char * xstrtolocale(/*@only@*/ const char *str)
 	/*@modifies *str @*/;
+
+#if !defined(SWIG)
+/**
+ * Wrapper to free(3), hides const compilation noise, permit NULL, return NULL.
+ * @param p		memory to free
+ * @return		NULL always
+ */
+#if defined(WITH_DMALLOC)
+#define _free(p) ((p) != NULL ? free((void *)(p)) : (void)0, NULL)
+#else
+/*@unused@*/ static inline /*@null@*/
+void * _free(/*@only@*/ /*@null@*/ /*@out@*/ const void * p)
+	/*@modifies p @*/
+{
+    if (p != NULL)	free((void *)p);
+    return NULL;
+}
+#endif
+#endif
+
+/**
+ * Free all memory allocated by rpmio usage.
+ */
+void rpmioClean(void)
+	/*@globals internalState, fileSystem @*/
+	/*@modifies internalState, fileSystem @*/;
+
 #ifdef __cplusplus
 }
 #endif
