@@ -1,133 +1,138 @@
+/*@-realcompare -sizeoftype -protoparammatch @*/
 #include "system.h"
 
 #ifdef	WITH_LUA
+#include <rpmio.h>
+#include <rpmmacro.h>
+#include <rpmlog.h>
+#include <rpmurl.h>
+#include <rpmhook.h>
+#include <rpmcb.h>
+#include <argv.h>
+
 #include <lua.h>
 #include <lualib.h>
 #include <lauxlib.h>
+#ifdef WITH_LUA_INTERNAL
+#include <llocal.h>
 #include <lposix.h>
 #include <lrexlib.h>
-
-/* replaced in 5.1 */
-#ifndef lua_open
-#define lua_open()	luaL_newstate()
-#endif
-
-/* defined as lua_objlen in 5.1 */
-#ifndef lua_strlen
-#define lua_strlen(L,i)	lua_rawlen(L, (i))
-#endif
-
-/* deprecated in 5.1, defined as lua_objlen in 5.1 */
-#ifndef luaL_getn
-#define luaL_getn(L,i)	((int)lua_rawlen(L, i))
-#endif
-
-/* define added in 5.2 */
-#ifndef lua_pushglobaltable
-#define lua_pushglobaltable(L)	lua_pushvalue(L, LUA_GLOBALSINDEX)
 #endif
 
 #include <unistd.h>
 #include <assert.h>
 
-#include <rpm/rpmio.h>
-#include <rpm/rpmmacro.h>
-#include <rpm/rpmlog.h>
-#include <rpm/rpmurl.h>
-#include <rpm/rpmfileutil.h>
-#include <rpm/rpmbase64.h>
-#include "rpmio/rpmhook.h"
-
 #define _RPMLUA_INTERNAL
-#include "rpmio/rpmlua.h"
+#include "rpmlua.h"
 
 #include "debug.h"
 
+#if !defined(HAVE_VSNPRINTF)
+static inline int vsnprintf(char * buf, /*@unused@*/ int nb,
+			    const char * fmt, va_list ap)
+{
+    return vsprintf(buf, fmt, ap);
+}
+#endif
+
 #define INITSTATE(_lua, lua) \
     rpmlua lua = _lua ? _lua : \
 	    (globalLuaState ? globalLuaState : \
-			\
+			/*@-mods@*/ \
 			(globalLuaState = rpmluaNew()) \
-			\
+			/*@=mods@*/ \
 	    )
 
-struct rpmluapb_s {
-    size_t alloced;
-    size_t used;
-    char *buf;
-    rpmluapb next;
-};
-
+/*@only@*/ /*@unchecked@*/ /*@null@*/
 static rpmlua globalLuaState = NULL;
 
-static int luaopen_rpm(lua_State *L);
-static int rpm_print(lua_State *L);
+static int luaopen_rpm(lua_State *L)
+	/*@modifies L @*/;
+static int rpm_print(lua_State *L)
+	/*@globals fileSystem @*/
+	/*@modifies L, fileSystem @*/;
 
 rpmlua rpmluaGetGlobalState(void)
 {
-    INITSTATE(NULL, lua);
-    return lua;
+    return globalLuaState;
 }
 
+/*@-mods@*/	/* XXX hide rpmGlobalMacroContext mods for now. */
 rpmlua rpmluaNew()
 {
     rpmlua lua = (rpmlua) xcalloc(1, sizeof(*lua));
-    struct stat st;
-    const luaL_Reg *lib;
-    char *initlua = rpmGenPath(rpmConfigDir(), "init.lua", NULL);
-   
-    static const luaL_Reg extlibs[] = {
+    lua_State *L = lua_open();
+    /*@-readonlytrans@*/
+    /*@observer@*/ /*@unchecked@*/
+    static const luaL_reg lualibs[] = {
+	/* standard LUA libraries */
+	{"", luaopen_base},
+	{LUA_LOADLIBNAME, luaopen_package},
+	{LUA_TABLIBNAME, luaopen_table},
+	{LUA_IOLIBNAME, luaopen_io},
+	{LUA_OSLIBNAME, luaopen_os},
+	{LUA_STRLIBNAME, luaopen_string},
+	{LUA_MATHLIBNAME, luaopen_math},
+	{LUA_DBLIBNAME, luaopen_debug},
+	/* local LUA libraries (RPM only) */
+#ifdef WITH_LUA_INTERNAL
 	{"posix", luaopen_posix},
 	{"rex", luaopen_rex},
+	{"local", luaopen_local},
+#endif
 	{"rpm", luaopen_rpm},
-	{"os",	luaopen_rpm_os},
 	{NULL, NULL},
     };
-    
-    lua_State *L = lua_open();
-    luaL_openlibs(L);
-    lua->L = L;
+    /*@observer@*/ /*@unchecked@*/
+    const luaL_reg *lib = lualibs;
+    /*@=readonlytrans@*/
 
-    for (lib = extlibs; lib->name; lib++) {
+    lua->L = L;
+    for (; lib->name; lib++) {
+/*@-noeffectuncon@*/
 	lua_pushcfunction(L, lib->func);
 	lua_pushstring(L, lib->name);
 	lua_call(L, 1, 0);
-	lua_settop(L, 0);
+/*@=noeffectuncon@*/
+    }
+#define	_LUADOTDIR	"%{?_rpmhome}%{!?_rpmhome:" USRLIBRPM "}"
+    {	const char * _lua_path = rpmGetPath(_LUADOTDIR, "/lua/?.lua", NULL);
+	if (_lua_path != NULL) {
+	    lua_pushliteral(L, "LUA_PATH");
+	    lua_pushstring(L, _lua_path);
+	    _lua_path = _free(_lua_path);
+	}
     }
-#ifndef LUA_GLOBALSINDEX
-    lua_pushglobaltable(L);
-#endif
-    lua_pushliteral(L, "LUA_PATH");
-    lua_pushfstring(L, "%s/%s", rpmConfigDir(), "/lua/?.lua");
-#ifdef LUA_GLOBALSINDEX
     lua_rawset(L, LUA_GLOBALSINDEX);
-#else
-    lua_settable(L, -3);
-#endif
     lua_pushliteral(L, "print");
     lua_pushcfunction(L, rpm_print);
-#ifdef LUA_GLOBALSINDEX
     lua_rawset(L, LUA_GLOBALSINDEX);
-#else
-    lua_settable(L, -3);
-#endif
-#ifndef LUA_GLOBALSINDEX
-    lua_pop(L, 1);
-#endif
     rpmluaSetData(lua, "lua", lua);
-    if (stat(initlua, &st) != -1)
-	(void)rpmluaRunScriptFile(lua, initlua);
-    free(initlua);
+    {	const char * _lua_init = rpmGetPath(_LUADOTDIR, "/init.lua", NULL);
+	if (_lua_init != NULL) {
+	    struct stat st;
+	    if (Stat(_lua_init, &st) != -1)
+		(void)rpmluaRunScriptFile(lua, _lua_init);
+	    _lua_init = _free(_lua_init);
+	}
+    }
+#undef	_LUADOTDIR
     return lua;
 }
+/*@=mods@*/
 
-rpmlua rpmluaFree(rpmlua lua)
+void *rpmluaFree(rpmlua lua)
+	/*@globals globalLuaState @*/
+	/*@modifies globalLuaState @*/
 {
+    if (lua == NULL) {
+	lua = globalLuaState;
+	globalLuaState = NULL;
+    }
     if (lua) {
 	if (lua->L) lua_close(lua->L);
-	free(lua->printbuf);
-	free(lua);
-	if (lua == globalLuaState) globalLuaState = NULL;
+	lua->printbuf = _free(lua->printbuf);
+	lua = _free(lua);
     }
     return NULL;
 }
@@ -147,6 +152,7 @@ void rpmluaSetData(rpmlua _lua, const char *key, const void *data)
 }
 
 static void *getdata(lua_State *L, const char *key)
+	/*@modifies L @*/
 {
     void *ret = NULL;
     lua_pushliteral(L, "rpm_");
@@ -165,34 +171,23 @@ void *rpmluaGetData(rpmlua _lua, const char *key)
     return getdata(lua->L, key);
 }
 
-void rpmluaPushPrintBuffer(rpmlua _lua)
+void rpmluaSetPrintBuffer(rpmlua _lua, int flag)
 {
     INITSTATE(_lua, lua);
-    rpmluapb prbuf = xcalloc(1, sizeof(*prbuf));
-    prbuf->buf = NULL;
-    prbuf->alloced = 0;
-    prbuf->used = 0;
-    prbuf->next = lua->printbuf;
-
-    lua->printbuf = prbuf;
+    lua->storeprint = flag;
+    lua->printbuf = _free(lua->printbuf);
+    lua->printbufsize = 0;
+    lua->printbufused = 0;
 }
 
-char *rpmluaPopPrintBuffer(rpmlua _lua)
+const char *rpmluaGetPrintBuffer(rpmlua _lua)
 {
     INITSTATE(_lua, lua);
-    rpmluapb prbuf = lua->printbuf;
-    char *ret = NULL;
-
-    if (prbuf) {
-	ret = prbuf->buf;
-	lua->printbuf = prbuf->next;
-	free(prbuf);
-    }
-    
-    return ret;
+    return lua->printbuf;
 }
 
 static int pushvar(lua_State *L, rpmluavType type, void *value)
+	/*@modifies L @*/
 {
     int ret = 0;
     switch (type) {
@@ -225,7 +220,7 @@ void rpmluaSetVar(rpmlua _lua, rpmluav var)
     }
     if (!var->listmode || lua->pushsize > 0) {
 	if (lua->pushsize == 0)
-	    lua_pushglobaltable(L);
+	    lua_pushvalue(L, LUA_GLOBALSINDEX);
 	if (pushvar(L, var->keyType, &var->key) != -1) {
 	    if (pushvar(L, var->valueType, &var->value) != -1)
 		lua_rawset(L, -3);
@@ -238,11 +233,14 @@ void rpmluaSetVar(rpmlua _lua, rpmluav var)
 }
 
 static void popvar(lua_State *L, rpmluavType *type, void *value)
+	/*@modifies L, *type, *value @*/
 {
     switch (lua_type(L, -1)) {
     case LUA_TSTRING:
 	*type = RPMLUAV_STRING;
+/*@-observertrans -dependenttrans @*/
 	*((const char **)value) = lua_tostring(L, -1);
+/*@=observertrans =dependenttrans @*/
 	break;
     case LUA_TNUMBER:
 	*type = RPMLUAV_NUMBER;
@@ -262,7 +260,7 @@ void rpmluaGetVar(rpmlua _lua, rpmluav var)
     lua_State *L = lua->L;
     if (!var->listmode) {
 	if (lua->pushsize == 0)
-	    lua_pushglobaltable(L);
+	    lua_pushvalue(L, LUA_GLOBALSINDEX);
 	if (pushvar(L, var->keyType, &var->key) != -1) {
 	    lua_rawget(L, -2);
 	    popvar(L, &var->valueType, &var->value);
@@ -280,22 +278,14 @@ void rpmluaGetVar(rpmlua _lua, rpmluav var)
 #define FINDKEY_CREATE 1
 #define FINDKEY_REMOVE 2
 static int findkey(lua_State *L, int oper, const char *key, va_list va)
+	/*@modifies L @*/
 {
-    char *buf;
+    char buf[BUFSIZ];
     const char *s, *e;
     int ret = 0;
-    int blen;
-
-    blen = vsnprintf(NULL, 0, key, va);
-    if (blen <= 0) {
-	return -1;
-    }
-
-    buf = xmalloc(blen + 1);
-    vsnprintf(buf, blen + 1, key, va);
-
+    (void) vsnprintf(buf, BUFSIZ, key, va);
     s = e = buf;
-    lua_pushglobaltable(L);
+    lua_pushvalue(L, LUA_GLOBALSINDEX);
     for (;;) {
 	if (*e == '\0' || *e == '.') {
 	    if (e != s) {
@@ -306,12 +296,13 @@ static int findkey(lua_State *L, int oper, const char *key, va_list va)
 			lua_pushnil(L);
 			lua_rawset(L, -3);
 			lua_pop(L, 1);
-			break;
+			/*@switchbreak@*/ break;
 		    }
+		    /*@fallthrough@*/
 		case FINDKEY_RETURN:
 		    lua_rawget(L, -2);
 		    lua_remove(L, -2);
-		    break;
+		    /*@switchbreak@*/ break;
 		case FINDKEY_CREATE:
 		    lua_rawget(L, -2);
 		    if (!lua_istable(L, -1)) {
@@ -322,7 +313,7 @@ static int findkey(lua_State *L, int oper, const char *key, va_list va)
 			lua_rawset(L, -4);
 		    }
 		    lua_remove(L, -2);
-		    break;
+		    /*@switchbreak@*/ break;
 		}
 	    }
 	    if (*e == '\0')
@@ -336,7 +327,6 @@ static int findkey(lua_State *L, int oper, const char *key, va_list va)
 	}
 	e++;
     }
-    free(buf);
 
     return ret;
 }
@@ -390,9 +380,9 @@ rpmluav rpmluavNew(void)
     return var;
 }
 
-rpmluav rpmluavFree(rpmluav var)
+void *rpmluavFree(rpmluav var)
 {
-    free(var);
+    var = _free(var);
     return NULL;
 }
 
@@ -405,6 +395,7 @@ void rpmluavSetListMode(rpmluav var, int flag)
 void rpmluavSetKey(rpmluav var, rpmluavType type, const void *value)
 {
     var->keyType = type;
+/*@-assignexpose -temptrans @*/
     switch (type) {
 	case RPMLUAV_NUMBER:
 	    var->key.num = *((double *)value);
@@ -415,11 +406,13 @@ void rpmluavSetKey(rpmluav var, rpmluavType type, const void *value)
 	default:
 	    break;
     }
+/*@=assignexpose =temptrans @*/
 }
 
 void rpmluavSetValue(rpmluav var, rpmluavType type, const void *value)
 {
     var->valueType = type;
+/*@-assignexpose -temptrans @*/
     switch (type) {
 	case RPMLUAV_NUMBER:
 	    var->value.num = *((const double *)value);
@@ -430,11 +423,13 @@ void rpmluavSetValue(rpmluav var, rpmluavType type, const void *value)
 	default:
 	    break;
     }
+/*@=assignexpose =temptrans @*/
 }
 
 void rpmluavGetKey(rpmluav var, rpmluavType *type, void **value)
 {
     *type = var->keyType;
+/*@-onlytrans@*/
     switch (var->keyType) {
 	case RPMLUAV_NUMBER:
 	    *((double **)value) = &var->key.num;
@@ -445,11 +440,13 @@ void rpmluavGetKey(rpmluav var, rpmluavType *type, void **value)
 	default:
 	    break;
     }
+/*@=onlytrans@*/
 }
 
 void rpmluavGetValue(rpmluav var, rpmluavType *type, void **value)
 {
     *type = var->valueType;
+/*@-onlytrans@*/
     switch (var->valueType) {
 	case RPMLUAV_NUMBER:
 	    *((double **)value) = &var->value.num;
@@ -460,6 +457,7 @@ void rpmluavGetValue(rpmluav var, rpmluavType *type, void **value)
 	default:
 	    break;
     }
+/*@=onlytrans@*/
 }
 
 void rpmluavSetKeyNum(rpmluav var, double value)
@@ -511,7 +509,7 @@ int rpmluaCheckScript(rpmlua _lua, const char *script, const char *name)
 	name = "<lua>";
     if (luaL_loadbuffer(L, script, strlen(script), name) != 0) {
 	rpmlog(RPMLOG_ERR,
-		_("invalid syntax in lua scriptlet: %s\n"),
+		_("invalid syntax in Lua scriptlet: %s\n"),
 		  lua_tostring(L, -1));
 	ret = -1;
     }
@@ -526,15 +524,13 @@ int rpmluaRunScript(rpmlua _lua, const char *script, const char *name)
     int ret = 0;
     if (name == NULL)
 	name = "<lua>";
-    if (script == NULL)
-	script = "";
     if (luaL_loadbuffer(L, script, strlen(script), name) != 0) {
-	rpmlog(RPMLOG_ERR, _("invalid syntax in lua script: %s\n"),
+	rpmlog(RPMLOG_ERR, _("invalid syntax in Lua script: %s\n"),
 		 lua_tostring(L, -1));
 	lua_pop(L, 1);
 	ret = -1;
     } else if (lua_pcall(L, 0, 0, 0) != 0) {
-	rpmlog(RPMLOG_ERR, _("lua script failed: %s\n"),
+	rpmlog(RPMLOG_ERR, _("Lua script failed: %s\n"),
 		 lua_tostring(L, -1));
 	lua_pop(L, 1);
 	ret = -1;
@@ -548,12 +544,12 @@ int rpmluaRunScriptFile(rpmlua _lua, const char *filename)
     lua_State *L = lua->L;
     int ret = 0;
     if (luaL_loadfile(L, filename) != 0) {
-	rpmlog(RPMLOG_ERR, _("invalid syntax in lua file: %s\n"),
+	rpmlog(RPMLOG_ERR, _("invalid syntax in Lua file: %s\n"),
 		 lua_tostring(L, -1));
 	lua_pop(L, 1);
 	ret = -1;
     } else if (lua_pcall(L, 0, 0, 0) != 0) {
-	rpmlog(RPMLOG_ERR, _("lua script failed: %s\n"),
+	rpmlog(RPMLOG_ERR, _("Lua script failed: %s\n"),
 		 lua_tostring(L, -1));
 	lua_pop(L, 1);
 	ret = -1;
@@ -563,13 +559,15 @@ int rpmluaRunScriptFile(rpmlua _lua, const char *filename)
 
 /* From lua.c */
 static int rpmluaReadline(lua_State *L, const char *prompt)
+	/*@globals fileSystem @*/
+	/*@modifies L, fileSystem @*/
 {
    static char buffer[1024];
    if (prompt) {
       (void) fputs(prompt, stdout);
       (void) fflush(stdout);
    }
-   if (fgets(buffer, sizeof(buffer), stdin) == NULL) {
+   if (fgets(buffer, (int)sizeof(buffer), stdin) == NULL) {
       return 0;  /* read fails */
    } else {
       lua_pushstring(L, buffer);
@@ -579,6 +577,8 @@ static int rpmluaReadline(lua_State *L, const char *prompt)
 
 /* Based on lua.c */
 static void _rpmluaInteractive(lua_State *L)
+	/*@globals fileSystem @*/
+	/*@modifies L, fileSystem @*/
 {
    (void) fputs("\n", stdout);
    printf("RPM Interactive %s Interpreter\n", LUA_VERSION);
@@ -588,26 +588,32 @@ static void _rpmluaInteractive(lua_State *L)
       if (rpmluaReadline(L, "> ") == 0)
 	 break;
       if (lua_tostring(L, -1)[0] == '=') {
+/*@-evalorder@*/
 	 (void) lua_pushfstring(L, "print(%s)", lua_tostring(L, -1)+1);
+/*@=evalorder@*/
 	 lua_remove(L, -2);
       }
       for (;;) {
+/*@-evalorder@*/
 	 rc = luaL_loadbuffer(L, lua_tostring(L, -1),
 			      lua_strlen(L, -1), "<lua>");
+/*@=evalorder@*/
 	 if (rc == LUA_ERRSYNTAX &&
 	     strstr(lua_tostring(L, -1), "near `<eof>'") != NULL) {
 	    if (rpmluaReadline(L, ">> ") == 0)
-	       break;
+	       /*@innerbreak@*/ break;
 	    lua_remove(L, -2); /* Remove error */
 	    lua_concat(L, 2);
-	    continue;
+	    /*@innercontinue@*/ continue;
 	 }
-	 break;
+	 /*@innerbreak@*/ break;
       }
       if (rc == 0)
 	 rc = lua_pcall(L, 0, 0, 0);
       if (rc != 0) {
+/*@-evalorderuncon@*/
 	 fprintf(stderr, "%s\n", lua_tostring(L, -1));
+/*@=evalorderuncon@*/
 	 lua_pop(L, 1);
       }
       lua_pop(L, 1); /* Remove line */
@@ -615,83 +621,112 @@ static void _rpmluaInteractive(lua_State *L)
    (void) fputs("\n", stdout);
 }
 
+/*@-mods@*/
 void rpmluaInteractive(rpmlua _lua)
 {
     INITSTATE(_lua, lua);
     _rpmluaInteractive(lua->L);
 }
+/*@=mods@*/
 
 /* ------------------------------------------------------------------ */
 /* Lua API */
 
-static int rpm_b64encode(lua_State *L)
+static int rpm_macros(lua_State *L)
 {
-    const char *str = luaL_checkstring(L, 1);
-    size_t len = lua_strlen(L, 1);
-    int linelen = -1;
-    if (lua_gettop(L) == 2)
-	linelen = luaL_checkinteger(L, 2);
-    if (str && len) {
-	char *data = rpmBase64Encode(str, len, linelen);
-	lua_pushstring(L, data);
-	free(data);
-    }
-    return 1;
-}
+    const char ** av = NULL;
+    int ac = 0;
+    int i;
 
-static int rpm_b64decode(lua_State *L)
-{
-    const char *str = luaL_checkstring(L, 1);
-    if (str) {
-	void *data = NULL;
-	size_t len = 0;
-	if (rpmBase64Decode(str, &data, &len) == 0) {
-	    lua_pushlstring(L, data, len);
-	} else {
-	    lua_pushnil(L);
+/*@-modunconnomods@*/
+    lua_newtable(L);
+/*@=modunconnomods@*/
+
+    ac = rpmGetMacroEntries(NULL, NULL, -1, &av);
+
+    if (av != NULL)
+    for (i = 0; i < ac; i++) {
+	char *n, *o, *b;
+
+	/* Parse out "%name(opts)\tbody" into n/o/b strings. */
+	n = (char *) av[i];
+	b = strchr(n, '\t');
+assert(b != NULL);
+	o = ((b > n && b[-1] == ')') ? strchr(n, '(') : NULL);
+	if (*n == '%')	n++;
+	if (o != NULL && *o == '(') {
+	    b[-1] = '\0';
+	    o++;
+            o[-1] = '\0';
 	}
-	free(data);
+        else
+            b[0] = '\0';
+	b++;
+
+/*@-modunconnomods@*/
+        lua_pushstring(L, n);
+        lua_newtable(L);
+        if (o) {
+            lua_pushstring(L, "opts");
+            lua_pushstring(L, o);
+            lua_settable(L, -3);
+        }
+        if (b) {
+            lua_pushstring(L, "body");
+            lua_pushstring(L, b);
+            lua_settable(L, -3);
+        }
+        lua_settable(L, -3);
+/*@=modunconnomods@*/
     }
+    av = argvFree(av);
     return 1;
 }
 
 static int rpm_expand(lua_State *L)
+	/*@globals rpmGlobalMacroContext, h_errno, internalState @*/
+	/*@modifies L, rpmGlobalMacroContext, internalState @*/
 {
     const char *str = luaL_checkstring(L, 1);
-    char *val = rpmExpand(str, NULL);
-    lua_pushstring(L, val);
-    free(val);
+    lua_pushstring(L, rpmExpand(str, NULL));
     return 1;
 }
 
 static int rpm_define(lua_State *L)
+	/*@globals rpmGlobalMacroContext, h_errno, internalState @*/
+	/*@modifies L, rpmGlobalMacroContext, internalState @*/
 {
     const char *str = luaL_checkstring(L, 1);
     (void) rpmDefineMacro(NULL, str, 0);
     return 0;
 }
 
-static int rpm_load(lua_State *L)
+static int rpm_undefine(lua_State *L)
+	/*@globals rpmGlobalMacroContext, h_errno, internalState @*/
+	/*@modifies L, rpmGlobalMacroContext, internalState @*/
 {
     const char *str = luaL_checkstring(L, 1);
-    int rc = rpmLoadMacroFile(NULL, str);
-    lua_pushnumber(L, rc);
-    return 1;
+    (void) rpmUndefineMacro(NULL, str);
+    return 0;
 }
 
 static int rpm_interactive(lua_State *L)
+	/*@globals fileSystem @*/
+	/*@modifies L, fileSystem @*/
 {
     _rpmluaInteractive(L);
     return 0;
 }
 
 typedef struct rpmluaHookData_s {
+/*@shared@*/
     lua_State *L;
     int funcRef;
     int dataRef;
 } * rpmluaHookData;
 
 static int rpmluaHookWrapper(rpmhookArgs args, void *data)
+    /*@*/
 {
     rpmluaHookData hookdata = (rpmluaHookData)data;
     lua_State *L = hookdata->L;
@@ -704,23 +739,23 @@ static int rpmluaHookWrapper(rpmhookArgs args, void *data)
 	    case 's':
 		lua_pushstring(L, args->argv[i].s);
 		lua_rawseti(L, -2, i+1);
-		break;
+		/*@switchbreak@*/ break;
 	    case 'i':
 		lua_pushnumber(L, (lua_Number)args->argv[i].i);
 		lua_rawseti(L, -2, i+1);
-		break;
+		/*@switchbreak@*/ break;
 	    case 'f':
 		lua_pushnumber(L, (lua_Number)args->argv[i].f);
 		lua_rawseti(L, -2, i+1);
-		break;
+		/*@switchbreak@*/ break;
 	    case 'p':
 		lua_pushlightuserdata(L, args->argv[i].p);
 		lua_rawseti(L, -2, i+1);
-		break;
+		/*@switchbreak@*/ break;
 	    default:
                 (void) luaL_error(L, "unsupported type '%c' as "
                               "a hook argument\n", args->argt[i]);
-		break;
+		/*@switchbreak@*/ break;
 	}
     }
     if (lua_pcall(L, 1, 1, 0) != 0) {
@@ -736,6 +771,8 @@ static int rpmluaHookWrapper(rpmhookArgs args, void *data)
 }
 
 static int rpm_register(lua_State *L)
+	/*@globals internalState @*/
+	/*@modifies L, internalState @*/
 {
     if (!lua_isstring(L, 1)) {
 	(void) luaL_argerror(L, 1, "hook name expected");
@@ -748,7 +785,9 @@ static int rpm_register(lua_State *L)
 	hookdata->dataRef = luaL_ref(L, LUA_REGISTRYINDEX);
 	lua_pushvalue(L, 2);
 	hookdata->funcRef = luaL_ref(L, LUA_REGISTRYINDEX);
+/*@-temptrans@*/
 	hookdata->L = L;
+/*@=temptrans@*/
 	rpmhookRegister(lua_tostring(L, 1), rpmluaHookWrapper, hookdata);
 	return 1;
     }
@@ -756,10 +795,11 @@ static int rpm_register(lua_State *L)
 }
 
 static int rpm_unregister(lua_State *L)
+	/*@modifies L @*/
 {
     if (!lua_isstring(L, 1)) {
 	(void) luaL_argerror(L, 1, "hook name expected");
-    } else if (!lua_isuserdata(L, 2)) {
+    } else if (!lua_islightuserdata(L, 2)) {
 	(void) luaL_argerror(L, 2, "hook information expected");
     } else {
 	rpmluaHookData hookdata = (rpmluaHookData)lua_touserdata(L, 2);
@@ -771,6 +811,8 @@ static int rpm_unregister(lua_State *L)
 }
 
 static int rpm_call(lua_State *L)
+	/*@globals internalState @*/
+	/*@modifies L, internalState @*/
 {
     if (!lua_isstring(L, 1)) {
 	(void) luaL_argerror(L, 1, "hook name expected");
@@ -784,9 +826,10 @@ static int rpm_call(lua_State *L)
 		case LUA_TNIL:
 		    argt[i] = 'p';
 		    args->argv[i].p = NULL;
-		    break;
+		    /*@switchbreak@*/ break;
 		case LUA_TNUMBER: {
 		    float f = (float)lua_tonumber(L, i+1);
+/*@+relaxtypes@*/
 		    if (f == (int)f) {
 			argt[i] = 'i';
 			args->argv[i].i = (int)f;
@@ -794,33 +837,38 @@ static int rpm_call(lua_State *L)
 			argt[i] = 'f';
 			args->argv[i].f = f;
 		    }
-		}   break;
+/*@=relaxtypes@*/
+		}   /*@switchbreak@*/ break;
 		case LUA_TSTRING:
 		    argt[i] = 's';
 		    args->argv[i].s = lua_tostring(L, i+1);
-		    break;
+		    /*@switchbreak@*/ break;
 		case LUA_TUSERDATA:
 		case LUA_TLIGHTUSERDATA:
 		    argt[i] = 'p';
 		    args->argv[i].p = lua_touserdata(L, i+1);
-		    break;
+		    /*@switchbreak@*/ break;
 		default:
 		    (void) luaL_error(L, "unsupported Lua type passed to hook");
 		    argt[i] = 'p';
 		    args->argv[i].p = NULL;
-		    break;
+		    /*@switchbreak@*/ break;
 	    }
 	}
+/*@-compdef -kepttrans -usereleased @*/
 	args->argt = argt;
 	rpmhookCallArgs(name, args);
-	free(argt);
+	argt = _free(argt);
 	(void) rpmhookArgsFree(args);
+/*@=compdef =kepttrans =usereleased @*/
     }
     return 0;
 }
 
 /* Based on luaB_print. */
 static int rpm_print (lua_State *L)
+	/*@globals fileSystem @*/
+	/*@modifies L, fileSystem @*/
 {
     rpmlua lua = (rpmlua)getdata(L, "lua");
     int n = lua_gettop(L);  /* number of arguments */
@@ -835,17 +883,16 @@ static int rpm_print (lua_State *L)
 	s = lua_tostring(L, -1);  /* get result */
 	if (s == NULL)
 	    return luaL_error(L, "`tostring' must return a string to `print'");
-	if (lua->printbuf) {
-	    rpmluapb prbuf = lua->printbuf;
-	    int sl = lua_strlen(L, -1);
-	    if (prbuf->used+sl+1 > prbuf->alloced) {
-		prbuf->alloced += sl+512;
-		prbuf->buf = xrealloc(prbuf->buf, prbuf->alloced);
+	if (lua->storeprint) {
+	    size_t sl = lua_strlen(L, -1);
+	    if (lua->printbufused+sl+1 > lua->printbufsize) {
+		lua->printbufsize += sl+512;
+		lua->printbuf = xrealloc(lua->printbuf, lua->printbufsize);
 	    }
 	    if (i > 1)
-		prbuf->buf[prbuf->used++] = '\t';
-	    memcpy(prbuf->buf+prbuf->used, s, sl+1);
-	    prbuf->used += sl;
+		lua->printbuf[lua->printbufused++] = '\t';
+	    memcpy(lua->printbuf+lua->printbufused, s, sl+1);
+	    lua->printbufused += sl;
 	} else {
 	    if (i > 1)
 		(void) fputs("\t", stdout);
@@ -853,37 +900,78 @@ static int rpm_print (lua_State *L)
 	}
 	lua_pop(L, 1);  /* pop result */
     }
-    if (!lua->printbuf) {
+    if (!lua->storeprint) {
 	(void) fputs("\n", stdout);
     } else {
-	rpmluapb prbuf = lua->printbuf;
-	if (prbuf->used+1 > prbuf->alloced) {
-	    prbuf->alloced += 512;
-	    prbuf->buf = xrealloc(prbuf->buf, prbuf->alloced);
+	if (lua->printbufused+1 > lua->printbufsize) {
+	    lua->printbufsize += 512;
+	    lua->printbuf = xrealloc(lua->printbuf, lua->printbufsize);
 	}
-	prbuf->buf[prbuf->used] = '\0';
+	lua->printbuf[lua->printbufused] = '\0';
     }
     return 0;
 }
 
-static const luaL_Reg rpmlib[] = {
-    {"b64encode", rpm_b64encode},
-    {"b64decode", rpm_b64decode},
+static int rpm_source(lua_State *L)
+	/*@globals internalState @*/
+	/*@modifies L, internalState @*/
+{
+    if (!lua_isstring(L, 1)) {
+	(void)luaL_argerror(L, 1, "filename expected");
+    } else {
+        rpmlua lua = (rpmlua)getdata(L, "lua");
+	const char *filename = lua_tostring(L, 1);
+	(void)rpmluaRunScriptFile(lua, filename);
+    }
+    return 0;
+}
+
+static int rpm_load(lua_State *L)
+	/*@globals internalState @*/
+	/*@modifies L, internalState @*/
+{
+    if (!lua_isstring(L, 1)) {
+	(void)luaL_argerror(L, 1, "filename expected");
+    } else {
+	const char *filename = lua_tostring(L, 1);
+	(void)rpmLoadMacroFile(NULL, filename);
+    }
+    return 0;
+}
+
+static int rpm_verbose(lua_State *L)
+	/*@globals internalState @*/
+	/*@modifies L, internalState @*/
+{
+    lua_pushboolean(L, rpmIsVerbose());
+    return 1;
+}
+
+/*@-readonlytrans@*/
+/*@observer@*/ /*@unchecked@*/
+static const luaL_reg rpmlib[] = {
+    {"macros", rpm_macros},
     {"expand", rpm_expand},
     {"define", rpm_define},
-    {"load", rpm_load},
+    {"undefine", rpm_undefine},
     {"register", rpm_register},
     {"unregister", rpm_unregister},
     {"call", rpm_call},
     {"interactive", rpm_interactive},
+    {"source", rpm_source},
+    {"load", rpm_load},
+    {"verbose", rpm_verbose},
     {NULL, NULL}
 };
+/*@=readonlytrans@*/
 
 static int luaopen_rpm(lua_State *L)
+	/*@modifies L @*/
 {
-    lua_pushglobaltable(L);
+    lua_pushvalue(L, LUA_GLOBALSINDEX);
     luaL_openlib(L, "rpm", rpmlib, 0);
     return 0;
 }
 #endif	/* WITH_LUA */
 
+/*@=realcompare =sizeoftype =protoparammatch @*/
