@@ -5,10 +5,18 @@
 #include "system.h"
 #include "debug.h"
 
+#if defined(WITH_DMALLOC)
+#undef xmalloc
+#undef xcalloc
+#undef xrealloc
+#undef xstrdup
+#endif
+
 #if !defined(EXIT_FAILURE)
 #define	EXIT_FAILURE	1
 #endif
 
+/*@-modfilesys@*/
 /*@only@*/ void *vmefail(size_t size)
 {
     fprintf(stderr, _("memory alloc (%u bytes) returned NULL.\n"), (unsigned)size);
@@ -16,19 +24,18 @@
     /*@notreached@*/
     return NULL;
 }
+/*@=modfilesys@*/
 
 #if !(HAVE_MCHECK_H && defined(__GNUC__)) && !defined(__LCLINT__)
 
-/*@only@*/ void * xmalloc (size_t size)
+/*@out@*/ /*@only@*/ void * xmalloc (size_t size)
 {
     register void *value;
     if (size == 0) size++;
     value = malloc (size);
     if (value == 0)
 	value = vmefail(size);
-    /*@-compdef@*/
     return value;
-    /*@=compdef@*/
 }
 
 /*@only@*/ void * xcalloc (size_t nmemb, size_t size)
