@@ -4,14 +4,11 @@
 
 #include "system.h"
 
-#if defined(HAVE_PTHREAD_H) && !defined(__LCLINT__)
-#include <pthread.h>
-#endif
-
-#include <rpmio_internal.h>
+#include "rpmio_internal.h"
+#include "rpmmacro.h"
 
 #define	_RPMDAV_INTERNAL
-#include <rpmdav.h>
+#include "rpmdav.h"
 
 #include "ugid.h"
 #include "debug.h"
@@ -20,19 +17,6 @@
 /*@access FD_t @*/
 /*@access urlinfo @*/
 
-/**
- * Wrapper to free(3), hides const compilation noise, permit NULL, return NULL.
- * @param p		memory to free
- * @retval		NULL always
- */
-/*@unused@*/ static inline /*@null@*/ void *
-_free(/*@only@*/ /*@null@*/ /*@out@*/ const void * p)
-	/*@modifies p@*/
-{
-    if (p != NULL)	free((void *)p);
-    return NULL;
-}
-
 /* =============================================================== */
 static int ftpMkdir(const char * path, /*@unused@*/ mode_t mode)
 	/*@globals h_errno, fileSystem, internalState @*/
@@ -93,7 +77,9 @@ int Mkdir (const char * path, mode_t mode)
 	/*@notreached@*/ break;
     case URL_IS_HTTPS:
     case URL_IS_HTTP:
+#ifdef WITH_NEON
 	return davMkdir(path, mode);
+#endif
 	/*@notreached@*/ break;
     case URL_IS_PATH:
 	path = lpath;
@@ -153,7 +139,9 @@ int Rmdir (const char * path)
 	/*@notreached@*/ break;
     case URL_IS_HTTPS:
     case URL_IS_HTTP:
+#ifdef WITH_NEON
 	return davRmdir(path);
+#endif
 	/*@notreached@*/ break;
     case URL_IS_PATH:
 	path = lpath;
@@ -169,6 +157,10 @@ int Rmdir (const char * path)
     return rmdir(path);
 }
 
+/*@unchecked@*/
+const char * _chroot_prefix = NULL;
+
+/*@-mods@*/	/* XXX hide rpmGlobalMacroContext mods for now. */
 int Chroot(const char * path)
 {
     const char * lpath;
@@ -192,10 +184,66 @@ fprintf(stderr, "*** Chroot(%s)\n", path);
 	return -2;
 	/*@notreached@*/ break;
     }
+
+/*@-dependenttrans -modobserver -observertrans @*/
+    _chroot_prefix = _free(_chroot_prefix);
+/*@=dependenttrans =modobserver =observertrans @*/
+    if (strcmp(path, "."))
+	_chroot_prefix = rpmGetPath(path, NULL);
+
 /*@-superuser@*/
     return chroot(path);
 /*@=superuser@*/
 }
+/*@=mods@*/
+
+int Open(const char * path, int flags, mode_t mode)
+{
+    const char * lpath;
+    int ut = urlPath(path, &lpath);
+    int fdno;
+
+if (_rpmio_debug)
+fprintf(stderr, "*** Open(%s, 0x%x, 0%o)\n", path, flags, (unsigned)mode);
+    switch (ut) {
+    case URL_IS_PATH:
+	path = lpath;
+	/*@fallthrough@*/
+    case URL_IS_UNKNOWN:
+	break;
+    case URL_IS_DASH:
+    case URL_IS_HKP:
+    case URL_IS_FTP:		/* XXX TODO: implement. */
+    case URL_IS_HTTPS:		/* XXX TODO: implement. */
+    case URL_IS_HTTP:		/* XXX TODO: implement. */
+    default:
+	errno = EINVAL;		/* XXX W2DO? */
+	return -2;
+	/*@notreached@*/ break;
+    }
+
+    if (_chroot_prefix && _chroot_prefix[0] == '/' && _chroot_prefix[1] != '\0')
+    {
+	size_t nb = strlen(_chroot_prefix);
+	size_t ob = strlen(path);
+	while (nb > 0 && _chroot_prefix[nb-1] == '/')
+	    nb--;
+	if (ob > nb && !strncmp(path, _chroot_prefix, nb) && path[nb] == '/')
+	    path += nb;
+    }
+#ifdef	NOTYET	/* XXX likely sane default. */
+    if (mode == 0)
+	mode = 0644;
+#endif
+    fdno = open(path, flags, mode);
+    if (fdno >= 0) {
+	if (fcntl(fdno, F_SETFD, FD_CLOEXEC) < 0) {
+	    (void) close(fdno);
+	    fdno = -1;
+	}
+    }
+    return fdno;
+}
 
 /* XXX rpmdb.c: analogue to rename(2). */
 
@@ -212,7 +260,9 @@ int Rename (const char * oldpath, const char * newpath)
     switch (oldut) {
     case URL_IS_HTTPS:
     case URL_IS_HTTP:
+#ifdef WITH_NEON
 	return davRename(oldpath, newpath);
+#endif
 	/*@notreached@*/ break;
     case URL_IS_FTP:		/* XXX WRONG WRONG WRONG */
     case URL_IS_PATH:
@@ -310,7 +360,9 @@ int Unlink(const char * path) {
 	/*@notreached@*/ break;
     case URL_IS_HTTPS:
     case URL_IS_HTTP:
+#ifdef WITH_NEON
 	return davUnlink(path);
+#endif
 	/*@notreached@*/ break;
     case URL_IS_PATH:
 	path = lpath;
@@ -349,7 +401,6 @@ static char *columns [MAXCOLS];	/* Points to the string in column n */
 /*@unchecked@*/
 static int   column_ptr [MAXCOLS]; /* Index from 0 to the starting positions of the columns */
 
-/*@-boundswrite@*/
 static int
 vfs_split_text (char *p)
 	/*@globals columns, column_ptr @*/
@@ -361,7 +412,7 @@ vfs_split_text (char *p)
 
     for (numcols = 0; *p && numcols < MAXCOLS; numcols++){
 	while (*p == ' ' || *p == '\r' || *p == '\n'){
-	    *p = 0;
+	    *p = '\0';
 	    p++;
 	}
 	columns [numcols] = p;
@@ -371,9 +422,7 @@ vfs_split_text (char *p)
     }
     return numcols;
 }
-/*@=boundswrite@*/
 
-/*@-boundsread@*/
 static int
 is_num (int idx)
 	/*@*/
@@ -382,9 +431,7 @@ is_num (int idx)
 	return 0;
     return 1;
 }
-/*@=boundsread@*/
 
-/*@-boundsread@*/
 static int
 is_dos_date(/*@null@*/ const char *str)
 	/*@*/
@@ -394,7 +441,6 @@ is_dos_date(/*@null@*/ const char *str)
 	return 1;
     return 0;
 }
-/*@=boundsread@*/
 
 static int
 is_week (/*@null@*/ const char * str, /*@out@*/ struct tm * tim)
@@ -486,17 +532,17 @@ vfs_parse_filetype (char c)
 	/*@*/
 {
     switch (c) {
-        case 'd': return S_IFDIR;
-        case 'b': return S_IFBLK;
-        case 'c': return S_IFCHR;
-        case 'l': return S_IFLNK;
+        case 'd': return (int)S_IFDIR;
+        case 'b': return (int)S_IFBLK;
+        case 'c': return (int)S_IFCHR;
+        case 'l': return (int)S_IFLNK;
         case 's':
 #ifdef IS_IFSOCK /* And if not, we fall through to IFIFO, which is pretty close */
-	          return S_IFSOCK;
+	          return (int)S_IFSOCK;
 #endif
-        case 'p': return S_IFIFO;
+        case 'p': return (int)S_IFIFO;
         case 'm': case 'n':		/* Don't know what these are :-) */
-        case '-': case '?': return S_IFREG;
+        case '-': case '?': return (int)S_IFREG;
         default: return -1;
     }
 }
@@ -560,7 +606,6 @@ static int vfs_parse_filemode (const char *p)
     return res;
 }
 
-/*@-boundswrite@*/
 static int vfs_parse_filedate(int idx, /*@out@*/ time_t *t)
 	/*@modifies *t @*/
 {	/* This thing parses from idx in columns[] array */
@@ -668,9 +713,7 @@ static int vfs_parse_filedate(int idx, /*@out@*/ time_t *t)
         *t = 0;
     return idx;
 }
-/*@=boundswrite@*/
 
-/*@-boundswrite@*/
 static int
 vfs_parse_ls_lga (char * p, /*@out@*/ struct stat * st,
 		/*@out@*/ const char ** filename,
@@ -680,6 +723,7 @@ vfs_parse_ls_lga (char * p, /*@out@*/ struct stat * st,
     int idx, idx2, num_cols;
     int i;
     char *p_copy;
+    long n;
 
     if (strncmp (p, "total", 5) == 0)
         return 0;
@@ -720,8 +764,9 @@ vfs_parse_ls_lga (char * p, /*@out@*/ struct stat * st,
     p_copy = g_strdup(p);
     num_cols = vfs_split_text (p);
 
-    st->st_nlink = atol (columns [0]);
-    if (st->st_nlink < 0)
+    n = atol(columns[0]);
+    st->st_nlink = n;
+    if (n < 0)
         goto error;
 
     if (!is_num (1))
@@ -805,8 +850,9 @@ vfs_parse_ls_lga (char * p, /*@out@*/ struct stat * st,
 
     if (((S_ISLNK (st->st_mode) ||
         (num_cols == idx + 3 && st->st_nlink > 1))) /* Maybe a hardlink? (in extfs) */
-        && idx2){
-	int tlen;
+        && idx2)
+    {
+	size_t tlen;
 	char *t;
 
 	if (filename){
@@ -823,9 +869,9 @@ vfs_parse_ls_lga (char * p, /*@out@*/ struct stat * st,
 	    t = g_strdup (p_copy + column_ptr [idx2+1]);
 	    tlen = strlen (t);
 	    if (t [tlen-1] == '\r' || t [tlen-1] == '\n')
-		t [tlen-1] = 0;
+		t [tlen-1] = '\0';
 	    if (t [tlen-2] == '\r' || t [tlen-2] == '\n')
-		t [tlen-2] = 0;
+		t [tlen-2] = '\0';
 		
 	    *linkname = t;
 	}
@@ -837,16 +883,16 @@ vfs_parse_ls_lga (char * p, /*@out@*/ struct stat * st,
 	    /*
 	    *filename = g_strdup (columns [idx++]);
 	    */
-	    int tlen;
+	    size_t tlen;
 	    char *t;
 
 	    t = g_strdup (p_copy + column_ptr [idx]); idx++;
 	    tlen = strlen (t);
 	    /* g_strchomp(); */
 	    if (t [tlen-1] == '\r' || t [tlen-1] == '\n')
-	        t [tlen-1] = 0;
+	        t [tlen-1] = '\0';
 	    if (t [tlen-2] == '\r' || t [tlen-2] == '\n')
-		t [tlen-2] = 0;
+		t [tlen-2] = '\0';
 
 	    *filename = t;
 	}
@@ -874,7 +920,6 @@ error:
 	g_free (p_copy);
     return 0;
 }
-/*@=boundswrite@*/
 
 typedef enum {
 	DO_FTP_STAT	= 1,
@@ -896,7 +941,6 @@ static /*@only@*/ char * ftpBuf = NULL;
 	
 #define alloca_strdup(_s)       strcpy(alloca(strlen(_s)+1), (_s))
 
-/*@-boundswrite@*/
 static int ftpNLST(const char * url, ftpSysCall_t ftpSysCall,
 		/*@out@*/ /*@null@*/ struct stat * st,
 		/*@out@*/ /*@null@*/ char * rlbuf, size_t rlbufsiz)
@@ -913,7 +957,7 @@ static int ftpNLST(const char * url, ftpSysCall_t ftpSysCall,
     char * se;
     const char * urldn;
     char * bn = NULL;
-    int nbn = 0;
+    size_t nbn = 0;
     urlinfo u;
     int rc;
 
@@ -932,14 +976,12 @@ static int ftpNLST(const char * url, ftpSysCall_t ftpSysCall,
 	break;
     default:
 	urldn = alloca_strdup(url);
-	/*@-branchstate@*/
 	if ((bn = strrchr(urldn, '/')) == NULL)
 	    return -2;
 	else if (bn == path)
 	    bn = ".";
 	else
 	    *bn++ = '\0';
-	/*@=branchstate@*/
 	nbn = strlen(bn);
 
 	rc = ftpChdir(urldn);		/* XXX don't care about CWD */
@@ -953,7 +995,7 @@ static int ftpNLST(const char * url, ftpSysCall_t ftpSysCall,
 	/* XXX possibly should do "NLST -lais" to get st_ino/st_blocks also */
 	u->openError = ftpReq(fd, "NLST", "-la");
 
-	if (bn == NULL || nbn <= 0) {
+	if (bn == NULL || nbn == 0) {
 	    rc = -2;
 	    goto exit;
 	}
@@ -1069,10 +1111,11 @@ static int ftpNLST(const char * url, ftpSysCall_t ftpSysCall,
 	    rc = -1;
 	} else {
 	    rc = oe - o;
-	    if (rc > rlbufsiz)
-		rc = rlbufsiz;
-	    memcpy(rlbuf, o, rc);
-	    if (rc < rlbufsiz)
+assert(rc >= 0);
+	    if (rc > (int)rlbufsiz)
+		rc = (int)rlbufsiz;
+	    memcpy(rlbuf, o, (size_t)rc);
+	    if (rc < (int)rlbufsiz)
 		rlbuf[rc] = '\0';
 	}
 	break;
@@ -1088,7 +1131,6 @@ exit:
     (void) ufdClose(fd);
     return rc;
 }
-/*@=boundswrite@*/
 
 static const char * statstr(const struct stat * st,
 		/*@returned@*/ /*@out@*/ char * buf)
@@ -1153,7 +1195,6 @@ fprintf(stderr, "*** ftpReadlink(%s) rc %d\n", path, rc);
     return rc;
 }
 
-/*@-boundswrite@*/
 /*@null@*/
 static DIR * ftpOpendir(const char * path)
 	/*@globals h_errno, fileSystem, internalState @*/
@@ -1184,7 +1225,7 @@ fprintf(stderr, "*** ftpOpendir(%s)\n", path);
     ac = 2;
     sb = NULL;
     s = se = ftpBuf;
-    while ((c = *se) != '\0') {
+    while ((c = (int) *se) != (int) '\0') {
 	se++;
 	switch (c) {
 	case '/':
@@ -1212,7 +1253,7 @@ fprintf(stderr, "*** ftpOpendir(%s)\n", path);
     /*@-abstract@*/
     dp = (struct dirent *) (avdir + 1);
     av = (const char **) (dp + 1);
-    dt = (char *) (av + (ac + 1));
+    dt = (unsigned char *) (av + (ac + 1));
     t = (char *) (dt + ac + 1);
     /*@=abstract@*/
 
@@ -1233,12 +1274,12 @@ fprintf(stderr, "*** ftpOpendir(%s)\n", path);
 
     ac = 0;
     /*@-dependenttrans -unrecog@*/
-    dt[ac] = DT_DIR;	av[ac++] = t;	t = stpcpy(t, ".");	t++;
-    dt[ac] = DT_DIR;	av[ac++] = t;	t = stpcpy(t, "..");	t++;
+    dt[ac] = (unsigned char)DT_DIR; av[ac++] = t; t = stpcpy(t, ".");	t++;
+    dt[ac] = (unsigned char)DT_DIR; av[ac++] = t; t = stpcpy(t, "..");	t++;
     /*@=dependenttrans =unrecog@*/
     sb = NULL;
     s = se = ftpBuf;
-    while ((c = *se) != '\0') {
+    while ((c = (int) *se) != (int) '\0') {
 	se++;
 	switch (c) {
 	case '/':
@@ -1252,28 +1293,28 @@ fprintf(stderr, "*** ftpOpendir(%s)\n", path);
 		/*@-unrecog@*/
 		switch(*s) {
 		case 'p':
-		    dt[ac] = DT_FIFO;
+		    dt[ac] = (unsigned char) DT_FIFO;
 		    /*@innerbreak@*/ break;
 		case 'c':
-		    dt[ac] = DT_CHR;
+		    dt[ac] = (unsigned char) DT_CHR;
 		    /*@innerbreak@*/ break;
 		case 'd':
-		    dt[ac] = DT_DIR;
+		    dt[ac] = (unsigned char) DT_DIR;
 		    /*@innerbreak@*/ break;
 		case 'b':
-		    dt[ac] = DT_BLK;
+		    dt[ac] = (unsigned char) DT_BLK;
 		    /*@innerbreak@*/ break;
 		case '-':
-		    dt[ac] = DT_REG;
+		    dt[ac] = (unsigned char) DT_REG;
 		    /*@innerbreak@*/ break;
 		case 'l':
-		    dt[ac] = DT_LNK;
+		    dt[ac] = (unsigned char) DT_LNK;
 		    /*@innerbreak@*/ break;
 		case 's':
-		    dt[ac] = DT_SOCK;
+		    dt[ac] = (unsigned char) DT_SOCK;
 		    /*@innerbreak@*/ break;
 		default:
-		    dt[ac] = DT_UNKNOWN;
+		    dt[ac] = (unsigned char) DT_UNKNOWN;
 		    /*@innerbreak@*/ break;
 		}
 		/*@=unrecog@*/
@@ -1297,7 +1338,6 @@ fprintf(stderr, "*** ftpOpendir(%s)\n", path);
     return (DIR *) avdir;
 /*@=kepttrans@*/
 }
-/*@=boundswrite@*/
 
 int Stat(const char * path, struct stat * st)
 {
@@ -1312,7 +1352,9 @@ fprintf(stderr, "*** Stat(%s,%p)\n", path, st);
 	/*@notreached@*/ break;
     case URL_IS_HTTPS:
     case URL_IS_HTTP:
+#ifdef WITH_NEON
 	return davStat(path, st);
+#endif
 	/*@notreached@*/ break;
     case URL_IS_PATH:
 	path = lpath;
@@ -1342,7 +1384,9 @@ fprintf(stderr, "*** Lstat(%s,%p)\n", path, st);
 	/*@notreached@*/ break;
     case URL_IS_HTTPS:
     case URL_IS_HTTP:
+#ifdef WITH_NEON
 	return davLstat(path, st);
+#endif
 	/*@notreached@*/ break;
     case URL_IS_PATH:
 	path = lpath;
@@ -1552,6 +1596,7 @@ int Symlink(const char * oldpath, const char * newpath)
     const char * npath;
     int nut = urlPath(newpath, &npath);
 
+    nut = 0;	/* XXX keep gcc quiet. */
 if (_rpmio_debug)
 fprintf(stderr, "*** Symlink(%s,%s)\n", oldpath, newpath);
     switch (out) {
@@ -1689,6 +1734,7 @@ int Glob(const char *pattern, int flags,
 {
     const char * lpath;
     int ut = urlPath(pattern, &lpath);
+    const char *home = getenv("HOME");
 
 /*@-castfcnptr@*/
 if (_rpmio_debug)
@@ -1699,9 +1745,9 @@ fprintf(stderr, "*** Glob(%s,0x%x,%p,%p)\n", pattern, (unsigned)flags, (void *)e
     case URL_IS_HTTP:
     case URL_IS_FTP:
 /*@-type@*/
-	pglob->gl_closedir = Closedir;
-	pglob->gl_readdir = Readdir;
-	pglob->gl_opendir = Opendir;
+	pglob->gl_closedir = (void *) Closedir;
+	pglob->gl_readdir = (void *) Readdir;
+	pglob->gl_opendir = (void *) Opendir;
 	pglob->gl_lstat = Lstat;
 	pglob->gl_stat = Stat;
 /*@=type@*/
@@ -1712,6 +1758,10 @@ fprintf(stderr, "*** Glob(%s,0x%x,%p,%p)\n", pattern, (unsigned)flags, (void *)e
 	pattern = lpath;
 	/*@fallthrough@*/
     case URL_IS_UNKNOWN:
+	if (home && home[0])
+	    flags |= GLOB_TILDE;
+	else
+	    flags &= ~GLOB_TILDE;
 	break;
     case URL_IS_DASH:
     case URL_IS_HKP:
@@ -1742,7 +1792,9 @@ fprintf(stderr, "*** Opendir(%s)\n", path);
 	/*@notreached@*/ break;
     case URL_IS_HTTPS:	
     case URL_IS_HTTP:
+#ifdef WITH_NEON
 	return davOpendir(path);
+#endif
 	/*@notreached@*/ break;
     case URL_IS_PATH:
 	path = lpath;
@@ -1768,8 +1820,10 @@ fprintf(stderr, "*** Readdir(%p)\n", (void *)dir);
 	return NULL;
     if (ISAVMAGIC(dir))
 	return avReaddir(dir);
+#ifdef WITH_NEON
     if (ISDAVMAGIC(dir))
 	return davReaddir(dir);
+#endif
     return readdir(dir);
 }
 
@@ -1781,8 +1835,10 @@ fprintf(stderr, "*** Closedir(%p)\n", (void *)dir);
 	return 0;
     if (ISAVMAGIC(dir))
 	return avClosedir(dir);
+#ifdef WITH_NEON
     if (ISDAVMAGIC(dir))
 	return davClosedir(dir);
+#endif
     return closedir(dir);
 }
 
