@@ -3,21 +3,34 @@
  */
 
 #include "system.h"
+#ifdef HAVE_LANGINFO_H
 #include <langinfo.h>
+#endif
+#ifdef HAVE_ICONV
 #include <iconv.h>
+#endif
+#include "rpmio.h"
 #include "debug.h"
 
+#ifdef HAVE_ICONV
 static char *locale_encoding = NULL;
 static int locale_encoding_is_utf8 = 0;
+#endif
 
 const char * xstrtolocale(const char *str)
 {
+#ifdef HAVE_ICONV
     iconv_t cd;
     size_t src_size, dest_size;
-    char *result, *src, *dest;
+    char *result, *dest;
+    const char *src;
 
     if (locale_encoding == NULL) {
+#ifdef HAVE_LANGINFO_H
 	const char *encoding = nl_langinfo(CODESET);
+#else
+	const char *encoding = "char";
+#endif
 	locale_encoding = xmalloc(strlen(encoding) + 11);
 	sprintf(locale_encoding, "%s//TRANSLIT", encoding);
 	locale_encoding_is_utf8 = strcasecmp(encoding, "UTF-8") == 0;
@@ -33,7 +46,7 @@ const char * xstrtolocale(const char *str)
     src_size = strlen(str);
     dest_size = src_size + 1;
     result = xmalloc(dest_size);
-    src = (char *)str;
+    src = str;
     dest = result;
     for(;;) {
 	size_t status = iconv(cd, &src, &src_size, &dest, &dest_size);
@@ -62,4 +75,7 @@ const char * xstrtolocale(const char *str)
     }
     *dest = '\0';
     return result;
+#else
+    return str;
+#endif
 }
