@@ -1,4 +1,4 @@
-/* Copyright (C) 2001, 2002, 2003, 2005, 2007, 2009, 2010, 2011 Red Hat, Inc.
+/* Copyright (C) 2001, 2002, 2003, 2005, 2007 Red Hat, Inc.
    Written by Alexander Larsson <alexl@redhat.com>, 2002
    Based on code by Jakub Jelinek <jakub@redhat.com>, 2001.
 
@@ -16,8 +16,6 @@
    along with this program; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
 
-#include "system.h"
-
 /* Needed for libelf */
 #define _FILE_OFFSET_BITS 64
 
@@ -29,7 +27,6 @@
 #include <limits.h>
 #include <string.h>
 #include <stdlib.h>
-#include <stdint.h>
 #include <unistd.h>
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -37,17 +34,42 @@
 #include <popt.h>
 
 #include <gelf.h>
-#include <dwarf.h>
 
-#include <rpm/rpmio.h>
-#include <rpm/rpmpgp.h>
-#include "tools/hashtab.h"
+/* some defines taken from the dwarf standard */
+
+#define DW_TAG_compile_unit	0x11
+
+#define DW_AT_name		0x03
+#define DW_AT_stmt_list		0x10
+#define DW_AT_comp_dir		0x1b
+
+#define DW_FORM_addr		0x01
+#define DW_FORM_block2		0x03
+#define DW_FORM_block4		0x04
+#define DW_FORM_data2		0x05
+#define DW_FORM_data4		0x06
+#define DW_FORM_data8		0x07
+#define DW_FORM_string		0x08
+#define DW_FORM_block		0x09
+#define DW_FORM_block1		0x0a
+#define DW_FORM_data1		0x0b
+#define DW_FORM_flag		0x0c
+#define DW_FORM_sdata		0x0d
+#define DW_FORM_strp		0x0e
+#define DW_FORM_udata		0x0f
+#define DW_FORM_ref_addr	0x10
+#define DW_FORM_ref1		0x11
+#define DW_FORM_ref2		0x12
+#define DW_FORM_ref4		0x13
+#define DW_FORM_ref8		0x14
+#define DW_FORM_ref_udata	0x15
+#define DW_FORM_indirect	0x16
+
+#include <beecrypt/beecrypt.h>
+
+#include "hashtab.h"
 
 #define DW_TAG_partial_unit 0x3c
-#define DW_FORM_sec_offset 0x17
-#define DW_FORM_exprloc 0x18
-#define DW_FORM_flag_present 0x19
-#define DW_FORM_ref_sig8 0x20
 
 char *base_dir = NULL;
 char *dest_dir = NULL;
@@ -55,6 +77,9 @@ char *list_file = NULL;
 int list_file_fd = -1;
 int do_build_id = 0;
 
+typedef unsigned int uint_32;
+typedef unsigned short uint_16;
+
 typedef struct
 {
   Elf *elf;
@@ -68,7 +93,7 @@ typedef struct
 typedef struct
 {
   unsigned char *ptr;
-  uint32_t addend;
+  uint_32 addend;
 } REL;
 
 #define read_uleb128(ptr) ({		\
@@ -87,32 +112,31 @@ typedef struct
   ret;					\
 })
 
-static uint16_t (*do_read_16) (unsigned char *ptr);
-static uint32_t (*do_read_32) (unsigned char *ptr);
+static uint_16 (*do_read_16) (unsigned char *ptr);
+static uint_32 (*do_read_32) (unsigned char *ptr);
 static void (*write_32) (unsigned char *ptr, GElf_Addr val);
 
 static int ptr_size;
-static int cu_version;
 
-static inline uint16_t
+static inline uint_16
 buf_read_ule16 (unsigned char *data)
 {
   return data[0] | (data[1] << 8);
 }
 
-static inline uint16_t
+static inline uint_16
 buf_read_ube16 (unsigned char *data)
 {
   return data[1] | (data[0] << 8);
 }
 
-static inline uint32_t
+static inline uint_32
 buf_read_ule32 (unsigned char *data)
 {
   return data[0] | (data[1] << 8) | (data[2] << 16) | (data[3] << 24);
 }
 
-static inline uint32_t
+static inline uint_32
 buf_read_ube32 (unsigned char *data)
 {
   return data[3] | (data[2] << 8) | (data[1] << 16) | (data[0] << 24);
@@ -144,13 +168,13 @@ strptr (DSO *dso, int sec, off_t offset)
 #define read_1(ptr) *ptr++
 
 #define read_16(ptr) ({					\
-  uint16_t ret = do_read_16 (ptr);			\
+  uint_16 ret = do_read_16 (ptr);			\
   ptr += 2;						\
   ret;							\
 })
 
 #define read_32(ptr) ({					\
-  uint32_t ret = do_read_32 (ptr);			\
+  uint_32 ret = do_read_32 (ptr);			\
   ptr += 4;						\
   ret;							\
 })
@@ -159,7 +183,7 @@ REL *relptr, *relend;
 int reltype;
 
 #define do_read_32_relocated(ptr) ({			\
-  uint32_t dret = do_read_32 (ptr);			\
+  uint_32 dret = do_read_32 (ptr);			\
   if (relptr)						\
     {							\
       while (relptr < relend && relptr->ptr < ptr)	\
@@ -176,7 +200,7 @@ int reltype;
 })
 
 #define read_32_relocated(ptr) ({			\
-  uint32_t ret = do_read_32_relocated (ptr);		\
+  uint_32 ret = do_read_32_relocated (ptr);		\
   ptr += 4;						\
   ret;							\
 })
@@ -184,7 +208,7 @@ int reltype;
 static void
 dwarf2_write_le32 (unsigned char *p, GElf_Addr val)
 {
-  uint32_t v = (uint32_t) val;
+  uint_32 v = (uint_32) val;
 
   p[0] = v;
   p[1] = v >> 8;
@@ -196,7 +220,7 @@ dwarf2_write_le32 (unsigned char *p, GElf_Addr val)
 static void
 dwarf2_write_be32 (unsigned char *p, GElf_Addr val)
 {
-  uint32_t v = (uint32_t) val;
+  uint_32 v = (uint_32) val;
 
   p[3] = v;
   p[2] = v >> 8;
@@ -218,29 +242,21 @@ static struct
 #define DEBUG_LINE	2
 #define DEBUG_ARANGES	3
 #define DEBUG_PUBNAMES	4
-#define DEBUG_PUBTYPES	5
-#define DEBUG_MACINFO	6
-#define DEBUG_LOC	7
-#define DEBUG_STR	8
-#define DEBUG_FRAME	9
-#define DEBUG_RANGES	10
-#define DEBUG_TYPES	11
-#define DEBUG_MACRO	12
-#define DEBUG_GDB_SCRIPT	13
+#define DEBUG_MACINFO	5
+#define DEBUG_LOC	6
+#define DEBUG_STR	7
+#define DEBUG_FRAME	8
+#define DEBUG_RANGES	9
     { ".debug_info", NULL, NULL, 0, 0, 0 },
     { ".debug_abbrev", NULL, NULL, 0, 0, 0 },
     { ".debug_line", NULL, NULL, 0, 0, 0 },
     { ".debug_aranges", NULL, NULL, 0, 0, 0 },
     { ".debug_pubnames", NULL, NULL, 0, 0, 0 },
-    { ".debug_pubtypes", NULL, NULL, 0, 0, 0 },
     { ".debug_macinfo", NULL, NULL, 0, 0, 0 },
     { ".debug_loc", NULL, NULL, 0, 0, 0 },
     { ".debug_str", NULL, NULL, 0, 0, 0 },
     { ".debug_frame", NULL, NULL, 0, 0, 0 },
     { ".debug_ranges", NULL, NULL, 0, 0, 0 },
-    { ".debug_types", NULL, NULL, 0, 0, 0 },
-    { ".debug_macro", NULL, NULL, 0, 0, 0 },
-    { ".debug_gdb_scripts", NULL, NULL, 0, 0, 0 },
     { NULL, NULL, NULL, 0, 0, 0 }
   };
 
@@ -315,7 +331,7 @@ no_memory:
         }
       if (*slot != NULL)
 	{
-	  error (0, 0, "%s: Duplicate DWARF abbreviation %d", dso->filename,
+	  error (0, 0, "%s: Duplicate DWARF-2 abbreviation %d", dso->filename,
 		 t->entry);
 	  free (t);
 	  htab_delete (h);
@@ -333,10 +349,9 @@ no_memory:
 		goto no_memory;
 	    }
 	  form = read_uleb128 (ptr);
-	  if (form == 2
-	      || (form > DW_FORM_flag_present && form != DW_FORM_ref_sig8))
+	  if (form == 2 || form > DW_FORM_indirect)
 	    {
-	      error (0, 0, "%s: Unknown DWARF DW_FORM_%d", dso->filename, form);
+	      error (0, 0, "%s: Unknown DWARF-2 DW_FORM_%d", dso->filename, form);
 	      htab_delete (h);
 	      return NULL;
 	    }
@@ -346,7 +361,7 @@ no_memory:
         }
       if (read_uleb128 (ptr) != 0)
         {
-	  error (0, 0, "%s: DWARF abbreviation does not end with 2 zeros",
+	  error (0, 0, "%s: DWARF-2 abbreviation does not end with 2 zeros",
 		 dso->filename);
 	  htab_delete (h);
 	  return NULL;
@@ -363,6 +378,7 @@ static char *
 canonicalize_path (const char *s, char *d)
 {
   char *rv = d;
+  const char *sroot;
   char *droot;
 
   if (IS_DIR_SEPARATOR (*s))
@@ -378,6 +394,7 @@ canonicalize_path (const char *s, char *d)
 	s++;
     }
   droot = d;
+  sroot = s;
 
   while (*s)
     {
@@ -451,47 +468,35 @@ static int
 has_prefix (const char  *str,
 	    const char  *prefix)
 {
-  size_t str_len;
-  size_t prefix_len;
-
+  int str_len;
+  int prefix_len;
+  
   str_len = strlen (str);
   prefix_len = strlen (prefix);
 
   if (str_len < prefix_len)
     return 0;
-
+  
   return strncmp (str, prefix, prefix_len) == 0;
 }
 
-static int dirty_elf;
-static void
-dirty_section (unsigned int sec)
-{
-  elf_flagdata (debug_sections[sec].elf_data, ELF_C_SET, ELF_F_DIRTY);
-  dirty_elf = 1;
-}
-
 static int
-edit_dwarf2_line (DSO *dso, uint32_t off, char *comp_dir, int phase)
+edit_dwarf2_line (DSO *dso, uint_32 off, char *comp_dir, int phase)
 {
   unsigned char *ptr = debug_sections[DEBUG_LINE].data, *dir;
   unsigned char **dirt;
   unsigned char *endsec = ptr + debug_sections[DEBUG_LINE].size;
   unsigned char *endcu, *endprol;
   unsigned char opcode_base;
-  uint32_t value, dirt_cnt;
+  uint_32 value, dirt_cnt;
   size_t comp_dir_len = strlen (comp_dir);
   size_t abs_file_cnt = 0, abs_dir_cnt = 0;
 
   if (phase != 0)
     return 0;
-
-  /* XXX: RhBug:929365, should we error out instead of ignoring? */
-  if (ptr == NULL)
-    return 0;
-
+  
   ptr += off;
-
+  
   endcu = ptr + 4;
   endcu += read_32 (ptr);
   if (endcu == ptr + 0xffffffff)
@@ -508,13 +513,13 @@ edit_dwarf2_line (DSO *dso, uint32_t off, char *comp_dir, int phase)
     }
 
   value = read_16 (ptr);
-  if (value != 2 && value != 3 && value != 4)
+  if (value != 2)
     {
       error (0, 0, "%s: DWARF version %d unhandled", dso->filename,
 	     value);
       return 1;
     }
-
+  
   endprol = ptr + 4;
   endprol += read_32 (ptr);
   if (endprol > endcu)
@@ -523,10 +528,10 @@ edit_dwarf2_line (DSO *dso, uint32_t off, char *comp_dir, int phase)
 	     dso->filename);
       return 1;
     }
-
-  opcode_base = ptr[4 + (value >= 4)];
-  ptr = dir = ptr + 4 + (value >= 4) + opcode_base;
-
+  
+  opcode_base = ptr[4];
+  ptr = dir = ptr + 4 + opcode_base;
+  
   /* dir table: */
   value = 1;
   while (*ptr != 0)
@@ -621,12 +626,12 @@ edit_dwarf2_line (DSO *dso, uint32_t off, char *comp_dir, int phase)
 	}
 
       free (s);
-
+      
       read_uleb128 (ptr);
       read_uleb128 (ptr);
     }
   ++ptr;
-
+  
   if (dest_dir)
     {
       unsigned char *srcptr, *buf = NULL;
@@ -649,16 +654,14 @@ edit_dwarf2_line (DSO *dso, uint32_t off, char *comp_dir, int phase)
 	  size_t len = strlen ((char *)srcptr) + 1;
 	  const unsigned char *readptr = srcptr;
 
-	  char *orig = strdup ((const char *) srcptr);
-
 	  if (*srcptr == '/' && has_prefix ((char *)srcptr, base_dir))
 	    {
 	      if (dest_len < base_len)
-		++abs_dir_cnt;
+		  ++abs_dir_cnt;
 	      memcpy (ptr, dest_dir, dest_len);
 	      ptr += dest_len;
 	      readptr += base_len;
-	    }
+		}
 	  srcptr += len;
 
 	  shrank += srcptr - readptr;
@@ -667,22 +670,18 @@ edit_dwarf2_line (DSO *dso, uint32_t off, char *comp_dir, int phase)
 	  shrank -= len;
 	  ptr += len;
 
-	  if (memcmp (orig, ptr - len, len))
-	    dirty_section (DEBUG_STR);
-	  free (orig);
-	}
+	      elf_flagdata (debug_sections[DEBUG_STR].elf_data,
+			    ELF_C_SET, ELF_F_DIRTY);
+	    }
 
       if (shrank > 0)
 	{
 	  if (--shrank == 0)
 	    error (EXIT_FAILURE, 0,
 		   "canonicalization unexpectedly shrank by one character");
-	  else
-	    {
-	      memset (ptr, 'X', shrank);
-	      ptr += shrank;
-	      *ptr++ = '\0';
-	    }
+	  memset (ptr, 'X', shrank);
+	  ptr += shrank;
+	  *ptr++ = '\0';
 	}
 
       if (abs_dir_cnt + abs_file_cnt != 0)
@@ -690,7 +689,7 @@ edit_dwarf2_line (DSO *dso, uint32_t off, char *comp_dir, int phase)
 	  size_t len = (abs_dir_cnt + abs_file_cnt) * (base_len - dest_len);
 
 	  if (len == 1)
-	    error (EXIT_FAILURE, 0, "-b arg has to be either the same length as -d arg, or more than 1 char longer");
+	    error (EXIT_FAILURE, 0, "-b arg has to be either the same length as -d arg, or more than 1 char shorter");
 	  memset (ptr, 'X', len - 1);
 	  ptr += len - 1;
 	  *ptr++ = '\0';
@@ -711,7 +710,8 @@ edit_dwarf2_line (DSO *dso, uint32_t off, char *comp_dir, int phase)
 			   len - base_len);
 		  ptr += dest_len - base_len;
 		}
-	      dirty_section (DEBUG_STR);
+	      elf_flagdata (debug_sections[DEBUG_STR].elf_data,
+			    ELF_C_SET, ELF_F_DIRTY);
 	    }
 	  else if (ptr != srcptr)
 	    memmove (ptr, srcptr, len);
@@ -731,29 +731,31 @@ edit_dwarf2_line (DSO *dso, uint32_t off, char *comp_dir, int phase)
   return 0;
 }
 
+
+
 static unsigned char *
 edit_attributes (DSO *dso, unsigned char *ptr, struct abbrev_tag *t, int phase)
 {
   int i;
-  uint32_t list_offs;
+  uint_32 list_offs;
   int found_list_offs;
   char *comp_dir;
-
+  
   comp_dir = NULL;
   list_offs = 0;
   found_list_offs = 0;
   for (i = 0; i < t->nattr; ++i)
     {
-      uint32_t form = t->attr[i].form;
-      size_t len = 0;
-      size_t base_len, dest_len;
+      uint_32 form = t->attr[i].form;
+      uint_32 len = 0;
+      int base_len, dest_len;
+      
 
       while (1)
 	{
 	  if (t->attr[i].attr == DW_AT_stmt_list)
 	    {
-	      if (form == DW_FORM_data4
-		  || form == DW_FORM_sec_offset)
+	      if (form == DW_FORM_data4)
 		{
 		  list_offs = do_read_32_relocated (ptr);
 		  found_list_offs = 1;
@@ -761,53 +763,56 @@ edit_attributes (DSO *dso, unsigned char *ptr, struct abbrev_tag *t, int phase)
 	    }
 
 	  if (t->attr[i].attr == DW_AT_comp_dir)
-	    {
-	      if (form == DW_FORM_string)
-		{
+	  {
+	      if ( form == DW_FORM_string )
+	      {
 		  free (comp_dir);
 		  comp_dir = strdup ((char *)ptr);
-
+		  
 		  if (phase == 1 && dest_dir && has_prefix ((char *)ptr, base_dir))
-		    {
+		  {
 		      base_len = strlen (base_dir);
 		      dest_len = strlen (dest_dir);
-
+		      
 		      memcpy (ptr, dest_dir, dest_len);
 		      if (dest_len < base_len)
-			{
+		      {
 			  memset(ptr + dest_len, '/',
 				 base_len - dest_len);
-
-			}
-		      dirty_section (DEBUG_INFO);
-		    }
-		}
+			  
+		      }
+		      elf_flagdata (debug_sections[DEBUG_INFO].elf_data,
+				    ELF_C_SET, ELF_F_DIRTY);
+		  }
+	      }
+	  
 	      else if (form == DW_FORM_strp &&
 		       debug_sections[DEBUG_STR].data)
-		{
+	      {
 		  char *dir;
 
-		  dir = (char *) debug_sections[DEBUG_STR].data
-		    + do_read_32_relocated (ptr);
+		  dir = (char *)debug_sections[DEBUG_STR].data
+		      + do_read_32_relocated (ptr);
 
 		  free (comp_dir);
 		  comp_dir = strdup (dir);
 
 		  if (phase == 1 && dest_dir && has_prefix (dir, base_dir))
-		    {
+		  {
 		      base_len = strlen (base_dir);
 		      dest_len = strlen (dest_dir);
-
+		  
 		      memcpy (dir, dest_dir, dest_len);
 		      if (dest_len < base_len)
-			{
+		      {
 			  memmove (dir + dest_len, dir + base_len,
 				   strlen (dir + base_len) + 1);
-			}
-		      dirty_section (DEBUG_STR);
-		    }
-		}
-	    }
+		      }
+		      elf_flagdata (debug_sections[DEBUG_STR].elf_data,
+				    ELF_C_SET, ELF_F_DIRTY);
+		  }
+	      }
+	  }
 	  else if ((t->tag == DW_TAG_compile_unit
 		    || t->tag == DW_TAG_partial_unit)
 		   && t->attr[i].attr == DW_AT_name
@@ -815,9 +820,9 @@ edit_attributes (DSO *dso, unsigned char *ptr, struct abbrev_tag *t, int phase)
 		   && debug_sections[DEBUG_STR].data)
 	    {
 	      char *name;
-
-	      name = (char *) debug_sections[DEBUG_STR].data
-		+ do_read_32_relocated (ptr);
+	      
+	      name = (char *)debug_sections[DEBUG_STR].data
+		     + do_read_32_relocated (ptr);
 	      if (*name == '/' && comp_dir == NULL)
 		{
 		  char *enddir = strrchr (name, '/');
@@ -836,27 +841,21 @@ edit_attributes (DSO *dso, unsigned char *ptr, struct abbrev_tag *t, int phase)
 		{
 		  base_len = strlen (base_dir);
 		  dest_len = strlen (dest_dir);
-
+		  
 		  memcpy (name, dest_dir, dest_len);
 		  if (dest_len < base_len)
 		    {
 		      memmove (name + dest_len, name + base_len,
 			       strlen (name + base_len) + 1);
 		    }
-		  dirty_section (DEBUG_STR);
+		  elf_flagdata (debug_sections[DEBUG_STR].elf_data,
+				ELF_C_SET, ELF_F_DIRTY);
 		}
 	    }
 
 	  switch (form)
 	    {
-	    case DW_FORM_ref_addr:
-	      if (cu_version == 2)
-		ptr += ptr_size;
-	      else
-		ptr += 4;
-	      break;
-	    case DW_FORM_flag_present:
-	      break;
+	    case DW_FORM_ref_addr: /* ptr_size in DWARF 2, offset in DWARF 3 */
 	    case DW_FORM_addr:
 	      ptr += ptr_size;
 	      break;
@@ -871,12 +870,10 @@ edit_attributes (DSO *dso, unsigned char *ptr, struct abbrev_tag *t, int phase)
 	      break;
 	    case DW_FORM_ref4:
 	    case DW_FORM_data4:
-	    case DW_FORM_sec_offset:
 	      ptr += 4;
 	      break;
 	    case DW_FORM_ref8:
 	    case DW_FORM_data8:
-	    case DW_FORM_ref_sig8:
 	      ptr += 8;
 	      break;
 	    case DW_FORM_sdata:
@@ -905,51 +902,22 @@ edit_attributes (DSO *dso, unsigned char *ptr, struct abbrev_tag *t, int phase)
 	      form = DW_FORM_block1;
 	      break;
 	    case DW_FORM_block:
-	    case DW_FORM_exprloc:
 	      len = read_uleb128 (ptr);
 	      form = DW_FORM_block1;
 	      assert (len < UINT_MAX);
 	      break;
 	    default:
-	      error (0, 0, "%s: Unknown DWARF DW_FORM_%d", dso->filename,
+	      error (0, 0, "%s: Unknown DWARF-2 DW_FORM_%d", dso->filename,
 		     form);
 	      return NULL;
 	    }
 
 	  if (form == DW_FORM_block1)
 	    ptr += len;
-
+	  
 	  break;
 	}
     }
-
-  /* Ensure the CU current directory will exist even if only empty.  Source
-     filenames possibly located in its parent directories refer relatively to
-     it and the debugger (GDB) cannot safely optimize out the missing
-     CU current dir subdirectories.  */
-  if (comp_dir && list_file_fd != -1)
-    {
-      char *p;
-      size_t size;
-
-      if (base_dir && has_prefix (comp_dir, base_dir))
-	p = comp_dir + strlen (base_dir);
-      else if (dest_dir && has_prefix (comp_dir, dest_dir))
-	p = comp_dir + strlen (dest_dir);
-      else
-	p = comp_dir;
-
-      size = strlen (p) + 1;
-      while (size > 0)
-	{
-	  ssize_t ret = write (list_file_fd, p, size);
-	  if (ret == -1)
-	    break;
-	  size -= ret;
-	  p += ret;
-	}
-    }
-
   if (found_list_offs && comp_dir)
     edit_dwarf2_line (dso, list_offs, comp_dir, phase);
 
@@ -1007,7 +975,7 @@ edit_dwarf2 (DSO *dso)
 		      return 1;
 		    }
 
-		  scn = dso->scn[i];
+		  scn = dso->scn[i]; 
 		  data = elf_rawdata (scn, NULL);
 		  assert (data != NULL && data->d_buf != NULL);
 		  assert (elf_rawdata (scn, data) == NULL);
@@ -1066,7 +1034,7 @@ edit_dwarf2 (DSO *dso)
   if (debug_sections[DEBUG_INFO].data != NULL)
     {
       unsigned char *ptr, *endcu, *endsec;
-      uint32_t value;
+      uint_32 value;
       htab_t abbrev;
       struct abbrev_tag tag, *t;
       int phase;
@@ -1083,7 +1051,7 @@ edit_dwarf2 (DSO *dso)
 	  int rtype;
 
 	  i = debug_sections[DEBUG_INFO].relsec;
-	  scn = dso->scn[i];
+	  scn = dso->scn[i]; 
 	  data = elf_getdata (scn, NULL);
 	  assert (data != NULL && data->d_buf != NULL);
 	  assert (elf_getdata (scn, data) == NULL);
@@ -1160,16 +1128,6 @@ edit_dwarf2 (DSO *dso)
 		  if (rtype != R_ALPHA_REFLONG)
 		    goto fail;
 		  break;
-#if defined(EM_AARCH64) && defined(R_AARCH64_ABS32)
-		case EM_AARCH64:
-		  if (rtype != R_AARCH64_ABS32)
-		    goto fail;
-		  break;
-#endif
-		case EM_68K:
-		  if (rtype != R_68K_32)
-		    goto fail;
-		  break;
 		default:
 		fail:
 		  error (1, 0, "%s: Unhandled relocation %d in .debug_info section",
@@ -1211,54 +1169,54 @@ edit_dwarf2 (DSO *dso)
 		  error (0, 0, "%s: 64-bit DWARF not supported", dso->filename);
 		  return 1;
 		}
-
+	      
 	      if (endcu > endsec)
 		{
 		  error (0, 0, "%s: .debug_info too small", dso->filename);
 		  return 1;
 		}
-
-	      cu_version = read_16 (ptr);
-	      if (cu_version != 2 && cu_version != 3 && cu_version != 4)
+	      
+	      value = read_16 (ptr);
+	      if (value != 2)
 		{
 		  error (0, 0, "%s: DWARF version %d unhandled", dso->filename,
-			 cu_version);
+			 value);
 		  return 1;
 		}
-
+	      
 	      value = read_32_relocated (ptr);
 	      if (value >= debug_sections[DEBUG_ABBREV].size)
 		{
 		  if (debug_sections[DEBUG_ABBREV].data == NULL)
 		    error (0, 0, "%s: .debug_abbrev not present", dso->filename);
 		  else
-		    error (0, 0, "%s: DWARF CU abbrev offset too large",
+		    error (0, 0, "%s: DWARF-2 CU abbrev offset too large",
 			   dso->filename);
 		  return 1;
 		}
-
+	      
 	      if (ptr_size == 0)
 		{
 		  ptr_size = read_1 (ptr);
 		  if (ptr_size != 4 && ptr_size != 8)
 		    {
-		      error (0, 0, "%s: Invalid DWARF pointer size %d",
+		      error (0, 0, "%s: Invalid DWARF-2 pointer size %d",
 			     dso->filename, ptr_size);
 		      return 1;
 		    }
 		}
 	      else if (read_1 (ptr) != ptr_size)
 		{
-		  error (0, 0, "%s: DWARF pointer size differs between CUs",
+		  error (0, 0, "%s: DWARF-2 pointer size differs between CUs",
 			 dso->filename);
 		  return 1;
 		}
-
+	      
 	      abbrev = read_abbrev (dso,
 				    debug_sections[DEBUG_ABBREV].data + value);
 	      if (abbrev == NULL)
 		return 1;
-
+	      
 	      while (ptr < endcu)
 		{
 		  tag.entry = read_uleb128 (ptr);
@@ -1267,23 +1225,23 @@ edit_dwarf2 (DSO *dso)
 		  t = htab_find_with_hash (abbrev, &tag, tag.entry);
 		  if (t == NULL)
 		    {
-		      error (0, 0, "%s: Could not find DWARF abbreviation %d",
+		      error (0, 0, "%s: Could not find DWARF-2 abbreviation %d",
 			     dso->filename, tag.entry);
 		      htab_delete (abbrev);
 		      return 1;
 		    }
-
+		  
 		  ptr = edit_attributes (dso, ptr, t, phase);
 		  if (ptr == NULL)
 		    break;
 		}
-
+	      
 	      htab_delete (abbrev);
 	    }
 	}
       free (relbuf);
     }
-
+  
   return 0;
 }
 
@@ -1346,7 +1304,7 @@ fdopen_dso (int fd, const char *name)
     }
 
   elf_flagelf (elf, ELF_C_SET, ELF_F_LAYOUT);
-
+  
   memset (dso, 0, sizeof(DSO));
   dso->elf = elf;
   dso->ehdr = ehdr;
@@ -1374,35 +1332,27 @@ error_out:
   return NULL;
 }
 
-static const pgpHashAlgo algorithms[] = { PGPHASHALGO_MD5,
-  PGPHASHALGO_SHA1, PGPHASHALGO_SHA256, PGPHASHALGO_SHA384, PGPHASHALGO_SHA512 };
-
 /* Compute a fresh build ID bit-string from the editted file contents.  */
 static void
 handle_build_id (DSO *dso, Elf_Data *build_id,
 		 size_t build_id_offset, size_t build_id_size)
 {
-  DIGEST_CTX ctx;
-  pgpHashAlgo algorithm;
-  int i = sizeof(algorithms)/sizeof(algorithms[0]);
-  void *digest = NULL;
-  size_t len;
+  hashFunctionContext ctx;
+  const hashFunction *hf = NULL;
+  int i = hashFunctionCount ();
 
   while (i-- > 0)
     {
-      algorithm = algorithms[i];
-      if (rpmDigestLength(algorithm) == build_id_size)
+      hf = hashFunctionGet (i);
+      if (hf != NULL && hf->digestsize == build_id_size)
 	break;
     }
-  if (i < 0)
+  if (hf == NULL)
     {
       fprintf (stderr, "Cannot handle %Zu-byte build ID\n", build_id_size);
       exit (1);
     }
 
-  if (!dirty_elf)
-    goto print;
-
   if (elf_update (dso->elf, ELF_C_NULL) < 0)
     {
       fprintf (stderr, "Failed to update file: %s\n",
@@ -1413,7 +1363,7 @@ handle_build_id (DSO *dso, Elf_Data *build_id,
   /* Clear the old bits so they do not affect the new hash.  */
   memset ((char *) build_id->d_buf + build_id_offset, 0, build_id_size);
 
-  ctx = rpmDigestInit(algorithm, 0);
+  hashFunctionContextInit (&ctx, hf);
 
   /* Slurp the relevant header bits and section contents and feed them
      into the hash function.  The only bits we ignore are the offset
@@ -1424,6 +1374,12 @@ handle_build_id (DSO *dso, Elf_Data *build_id,
      or Elf64 object, only that we are consistent in what bits feed the
      hash so it comes out the same for the same file contents.  */
   {
+    inline void process (const void *data, size_t size)
+    {
+      memchunk chunk = { .data = (void *) data, .size = size };
+      hashFunctionContextUpdateMC (&ctx, &chunk);
+    }
+
     union
     {
       GElf_Ehdr ehdr;
@@ -1452,7 +1408,7 @@ handle_build_id (DSO *dso, Elf_Data *build_id,
 	  goto bad;
 	if (elf64_xlatetom (&x, &x, dso->ehdr.e_ident[EI_DATA]) == NULL)
 	  goto bad;
-	rpmDigestUpdate(ctx, x.d_buf, x.d_size);
+	process (x.d_buf, x.d_size);
       }
 
     x.d_type = ELF_T_SHDR;
@@ -1464,31 +1420,35 @@ handle_build_id (DSO *dso, Elf_Data *build_id,
 	  u.shdr.sh_offset = 0;
 	  if (elf64_xlatetom (&x, &x, dso->ehdr.e_ident[EI_DATA]) == NULL)
 	    goto bad;
-	  rpmDigestUpdate(ctx, x.d_buf, x.d_size);
+	  process (x.d_buf, x.d_size);
 
 	  if (u.shdr.sh_type != SHT_NOBITS)
 	    {
 	      Elf_Data *d = elf_rawdata (dso->scn[i], NULL);
 	      if (d == NULL)
 		goto bad;
-	      rpmDigestUpdate(ctx, d->d_buf, d->d_size);
+	      process (d->d_buf, d->d_size);
 	    }
 	}
   }
 
-  rpmDigestFinal(ctx, &digest, &len, 0);
-  memcpy((unsigned char *)build_id->d_buf + build_id_offset, digest, build_id_size);
-  free(digest);
+  hashFunctionContextDigest (&ctx, (byte *) build_id->d_buf + build_id_offset);
+  hashFunctionContextFree (&ctx);
 
   elf_flagdata (build_id, ELF_C_SET, ELF_F_DIRTY);
 
- print:
   /* Now format the build ID bits in hex to print out.  */
   {
-    const uint8_t * id = (uint8_t *)build_id->d_buf + build_id_offset;
-    char *hex = pgpHexStr(id, build_id_size);
+    const unsigned char * id = (unsigned char *) build_id->d_buf + build_id_offset;
+    char hex[build_id_size * 2 + 1];
+    int n = snprintf (hex, 3, "%02" PRIx8, id[0]);
+    assert (n == 2);
+    for (i = 1; i < build_id_size; ++i)
+      {
+	n = snprintf (&hex[i * 2], 3, "%02" PRIx8, id[i]);
+	assert (n == 2);
+      }
     puts (hex);
-    free(hex);
   }
 }
 
@@ -1506,8 +1466,9 @@ main (int argc, char *argv[])
   Elf_Data *build_id = NULL;
   size_t build_id_offset = 0, build_id_size = 0;
 
-  optCon = poptGetContext("debugedit", argc, (const char **)argv, optionsTable, 0);
-
+  optCon = poptGetContext("debugedit", argc, (const char **)argv,
+			  optionsTable, 0);
+  
   while ((nextopt = poptGetNextOpt (optCon)) > 0 || nextopt == POPT_ERROR_BADOPT)
     /* do nothing */ ;
 
@@ -1519,7 +1480,7 @@ main (int argc, char *argv[])
 	      argv[0]);
       exit (1);
     }
-
+  
   args = poptGetArgs (optCon);
   if (args == NULL || args[0] == NULL || args[1] != NULL)
     {
@@ -1536,17 +1497,11 @@ main (int argc, char *argv[])
 	}
       if (strlen (dest_dir) > strlen (base_dir))
 	{
-	  fprintf (stderr, "Dest dir longer than base dir is not supported\n");
+	  fprintf (stderr, "Only dest dir longer than base dir not supported\n");
 	  exit (1);
 	}
     }
 
-  /* Ensure clean paths, users can muck with these */
-  if (base_dir)
-    canonicalize_path(base_dir, base_dir);
-  if (dest_dir)
-    canonicalize_path(dest_dir, dest_dir);
-
   /* Make sure there are trailing slashes in dirs */
   if (base_dir != NULL && base_dir[strlen (base_dir)-1] != '/')
     {
@@ -1564,12 +1519,12 @@ main (int argc, char *argv[])
       free (dest_dir);
       dest_dir = p;
     }
-
+  
   if (list_file != NULL)
     {
       list_file_fd = open (list_file, O_WRONLY|O_CREAT|O_APPEND, 0644);
     }
-
+  
   file = args[0];
 
   if (elf_version(EV_CURRENT) == EV_NONE)
@@ -1601,17 +1556,16 @@ main (int argc, char *argv[])
   for (i = 1; i < dso->ehdr.e_shnum; i++)
     {
       const char *name;
-
+      
       switch (dso->shdr[i].sh_type)
 	{
 	case SHT_PROGBITS:
 	  name = strptr (dso, dso->ehdr.e_shstrndx, dso->shdr[i].sh_name);
 	  /* TODO: Handle stabs */
+#if 0
 	  if (strcmp (name, ".stab") == 0)
-	    {
-	      fprintf (stderr, "Stabs debuginfo not supported: %s\n", file);
-	      break;
-	    }
+	    edit_stabs (dso, i);
+#endif
 	  if (strcmp (name, ".debug_info") == 0)
 	    edit_dwarf2 (dso);
 
@@ -1631,8 +1585,7 @@ main (int argc, char *argv[])
 	      Elf_Data src = dst;
 	      src.d_buf = data->d_buf;
 	      assert (sizeof (Elf32_Nhdr) == sizeof (Elf64_Nhdr));
-	      while ((char *) data->d_buf + data->d_size -
-		     (char *) src.d_buf > (int) sizeof nh
+	      while ((char *) data->d_buf + data->d_size - (char *) src.d_buf > (int) sizeof nh
 		     && elf32_xlatetom (&dst, &src, dso->ehdr.e_ident[EI_DATA]))
 		{
 		  Elf32_Word len = sizeof nh + nh.n_namesz;
@@ -1642,8 +1595,7 @@ main (int argc, char *argv[])
 		      && !memcmp ((char *) src.d_buf + sizeof nh, "GNU", sizeof "GNU"))
 		    {
 		      build_id = data;
-		      build_id_offset = (char *) src.d_buf + len -
-					(char *) data->d_buf;
+		      build_id_offset = (char *) src.d_buf + len - (char *) data->d_buf;
 		      build_id_size = nh.n_descsz;
 		      break;
 		    }
@@ -1676,7 +1628,7 @@ main (int argc, char *argv[])
 
   /* Restore old access rights */
   chmod (file, stat_buf.st_mode);
-
+  
   poptFreeContext (optCon);
 
   return 0;
